@@ -78,10 +78,12 @@ static struct option long_opts[] = {
 void
 args_parse(args_t *args, int argc, char *argv[], const char dir[])
 {
+	int done = 0;
+
 	/* Request getopt() reinitialization. */
 	optind = 0;
 
-	while(1)
+	while(!done)
 	{
 		switch(getopt_long(argc, argv, "-c:fhv", long_opts, NULL))
 		{
@@ -110,7 +112,8 @@ args_parse(args_t *args, int argc, char *argv[], const char dir[])
 				break;
 			case 'r': /* --remote <args>... */
 				args->remote_cmds = argv + optind;
-				return;
+				done = 1;
+				break;
 			case 'R': /* --remote-expr <expr> */
 				args->remote_expr = optarg;
 				break;
@@ -179,9 +182,16 @@ args_parse(args_t *args, int argc, char *argv[], const char dir[])
 				break;
 
 			case -1: /* No more arguments. */
-				return;
+				done = 1;
+				break;
 		}
 	}
+
+	if(args->remote_cmds != NULL || args->remote_expr != NULL)
+	{
+		args->target_name = args->server_name;
+		args->server_name = NULL;
+	}
 }
 
 /* Lists names of servers on stdout. */
@@ -298,9 +308,7 @@ static void
 parse_path(const char dir[], const char path[], char buf[])
 {
 	strcpy(buf, path);
-#ifdef _WIN32
-	to_forward_slash(buf);
-#endif
+	system_to_internal_slashes(buf);
 	if(is_path_absolute(buf) || strcmp(path, "-") == 0)
 	{
 		copy_str(buf, PATH_MAX, path);
@@ -322,9 +330,7 @@ parse_path(const char dir[], const char path[], char buf[])
 		chosp(buf);
 	}
 
-#ifdef _WIN32
-	to_forward_slash(buf);
-#endif
+	system_to_internal_slashes(buf);
 }
 
 void
@@ -447,11 +453,12 @@ process_non_general_args(args_t *args)
 	{
 		fprintf(stderr, "%s\n", "--remote and --remote-expr can't be combined.");
 		quit_on_arg_parsing(EXIT_FAILURE);
+		return;
 	}
 
 	if(args->remote_cmds != NULL)
 	{
-		if(ipc_send(curr_stats.ipc, args->server_name, args->remote_cmds) != 0)
+		if(ipc_send(curr_stats.ipc, args->target_name, args->remote_cmds) != 0)
 		{
 			fprintf(stderr, "%s\n", "Sending remote commands failed.");
 			quit_on_arg_parsing(EXIT_FAILURE);
@@ -462,7 +469,7 @@ process_non_general_args(args_t *args)
 
 	if(args->remote_expr != NULL)
 	{
-		char *const result = ipc_eval(curr_stats.ipc, args->server_name,
+		char *const result = ipc_eval(curr_stats.ipc, args->target_name,
 				args->remote_expr);
 		if(result == NULL)
 		{
