@@ -23,7 +23,7 @@
 #include <assert.h> /* assert() */
 #include <ctype.h> /* tolower() */
 #include <stddef.h> /* NULL size_t wchar_t */
-#include <stdlib.h> /* free() qsort() */
+#include <stdlib.h> /* free() */
 #include <string.h> /* strcpy() strlen() */
 #include <wchar.h> /* wcscpy() wcslen() */
 #include <wctype.h> /* iswcntrl() */
@@ -31,8 +31,10 @@
 #include "compat/curses.h"
 #include "compat/reallocarray.h"
 #include "engine/text_buffer.h"
+#include "modes/wk.h"
 #include "utils/macros.h"
 #include "utils/str.h"
+#include "utils/utils.h"
 
 /* Bracket notation entry description structure. */
 typedef struct
@@ -50,6 +52,7 @@ static const char * wchar_to_spec(const wchar_t c[], size_t *len, int bs);
 /* All notation fields must be written in lower case. */
 static key_pair_t key_pairs[] = {
 	{ L"<esc>",      L"\x1b"              },
+	{ L"<c-@>",      { WC_C_SPACE }       },
 	{ L"<c-a>",      L"\x01"              },
 	{ L"<c-b>",      L"\x02"              },
 	{ L"<c-c>",      L"\x03"              },
@@ -374,6 +377,7 @@ static key_pair_t key_pairs[] = {
 	{ L"<down>",     { K(KEY_DOWN) }      },
 	{ L"<bs>",       { K(KEY_BACKSPACE) } },
 	{ L"<delete>",   { K(KEY_DC) }        },
+	{ L"<insert>",   { K(KEY_IC) }        },
 	{ L"<pageup>",   { K(KEY_PPAGE) }     },
 	{ L"<pagedown>", { K(KEY_NPAGE) }     },
 	{ L"<f0>",       { K(KEY_F(0)) }      },
@@ -500,7 +504,8 @@ init_bracket_notation(void)
 		key_pairs[i].len = wcslen(key_pairs[i].notation);
 	}
 
-	qsort(key_pairs, ARRAY_LEN(key_pairs), sizeof(key_pairs[0]), notation_sorter);
+	safe_qsort(key_pairs, ARRAY_LEN(key_pairs), sizeof(key_pairs[0]),
+			notation_sorter);
 }
 
 /* Sorter function to be called by qsort. */
@@ -649,6 +654,7 @@ wchar_to_spec(const wchar_t c[], size_t *len, int bs)
 		case K(KEY_BTAB):   strcpy(buf, "<s-tab>");    break;
 		case K(KEY_PPAGE):  strcpy(buf, "<pageup>");   break;
 		case K(KEY_NPAGE):  strcpy(buf, "<pagedown>"); break;
+		case WC_C_SPACE:    strcpy(buf, "<c-@>");      break;
 
 		case L'\b':
 			if(!bs)
