@@ -28,8 +28,10 @@
 
 #ifndef __APPLE__
 #define SAMPLE_VIFMRC "vifmrc"
+#define SAMPLE_MEDIAPRG "vifm-media"
 #else
 #define SAMPLE_VIFMRC "vifmrc-osx"
+#define SAMPLE_MEDIAPRG "vifm-media-osx"
 #endif
 
 #include <assert.h> /* assert() */
@@ -159,8 +161,9 @@ cfg_init(void)
 	cfg.grep_prg = strdup("grep -n -H -I -r %i %a %s");
 	cfg.locate_prg = strdup("locate %a");
 	cfg.delete_prg = strdup("");
-	cfg.media_prg = format_str("%s/vifm-media", get_installed_data_dir());
+	cfg.media_prg = format_str("%s/" SAMPLE_MEDIAPRG, get_installed_data_dir());
 
+	cfg.tail_tab_line_paths = 0;
 	cfg.trunc_normal_sb_msgs = 0;
 	cfg.shorten_title_paths = 1;
 	cfg.short_term_mux_titles = 0;
@@ -196,6 +199,7 @@ cfg_init(void)
 	cfg.log_file[0] = '\0';
 
 	cfg_set_shell(env_get_def("SHELL", DEFAULT_SHELL_CMD));
+	cfg.shell_cmd_flag = strdup((curr_stats.shell_type == ST_CMD) ? "/C" : "-c");
 
 	memset(&cfg.type_decs, '\0', sizeof(cfg.type_decs));
 	cfg.type_decs[FT_DIR][DECORATION_SUFFIX][0] = '/';
@@ -212,6 +216,7 @@ cfg_init(void)
 	cfg.sizefmt.base = 1024;
 	cfg.sizefmt.precision = 0;
 	cfg.sizefmt.ieci_prefixes = 0;
+	cfg.sizefmt.space = 1;
 
 	cfg.pane_tabs = 0;
 	cfg.show_tab_line = STL_MULTIPLE;
@@ -244,6 +249,8 @@ find_home_dir(void)
 	if(try_home_envvar_for_home()) return;
 	if(try_userprofile_envvar_for_home()) return;
 	if(try_homepath_envvar_for_home()) return;
+
+	vifm_finish("Failed to find user's home directory.");
 }
 
 /* Tries to use HOME environment variable to find home directory.  Returns
@@ -853,17 +860,24 @@ cfg_resize_histories(int new_size)
 int
 cfg_set_fuse_home(const char new_value[])
 {
-	char canonicalized[PATH_MAX + 1];
 #ifdef _WIN32
 	char with_forward_slashes[strlen(new_value) + 1];
 	strcpy(with_forward_slashes, new_value);
 	system_to_internal_slashes(with_forward_slashes);
 	new_value = with_forward_slashes;
 #endif
+
+	char canonicalized[PATH_MAX + 1];
 	canonicalize_path(new_value, canonicalized, sizeof(canonicalized));
 
 	if(!is_path_absolute(new_value))
 	{
+		if(cfg.fuse_home == NULL)
+		{
+			/* Do not leave cfg.fuse_home uninitialized. */
+			cfg.fuse_home = strdup("");
+		}
+
 		show_error_msgf("Error Setting FUSE Home Directory",
 				"The path is not absolute: %s", canonicalized);
 		return 1;
