@@ -180,7 +180,7 @@ typedef struct config_t
 	/* This one should be set using cfg_set_fuse_home() function. */
 	char *fuse_home;
 
-	col_scheme_t cs;
+	col_scheme_t cs; /* Storage of primary (global) color scheme. */
 
 	int undo_levels;  /* Maximum number of changes that can be undone. */
 	int sort_numbers; /* Natural sort of (version) numbers within text. */
@@ -211,7 +211,10 @@ typedef struct config_t
 	int smart_case;
 	int hl_search;
 	int vifm_info;
-	char *shell;
+
+	char *shell;          /* Shell interpreter command. */
+	char *shell_cmd_flag; /* Argument for the shell to pass command. */
+
 	int scroll_off;
 	int gdefault;
 	int scroll_bind;
@@ -244,6 +247,7 @@ typedef struct config_t
 	char *media_prg;   /* Helper for managing media devices. */
 
 	/* Message shortening controlled by 'shortmess'. */
+	int tail_tab_line_paths;   /* Display only last directory in tab line. */
 	int trunc_normal_sb_msgs;  /* Truncate normal status bar msgs if needed. */
 	int shorten_title_paths;   /* Use tilde shortening in view titles. */
 	int short_term_mux_titles; /* Use only file name for titles of screen/tmux. */
@@ -306,6 +310,8 @@ typedef struct config_t
 		int precision;     /* Number of digits after dot to consider (0 for old
 		                      behaviour). */
 		int ieci_prefixes; /* When base is 1024, whether to say KiB instead of K. */
+		int space;         /* Whether a space should be displayed between numbers
+		                      and unit symbols. */
 	}
 	sizefmt;
 
