@@ -168,7 +168,7 @@ cmds_expand_macros(const char str[], int for_shell, int *usr1, int *usr2)
 	char *result;
 	MacroFlags flags = MF_NONE;
 
-	result = expand_macros(str, NULL, &flags, for_shell);
+	result = ma_expand(str, NULL, &flags, for_shell);
 
 	*usr1 = flags;
 
@@ -925,6 +925,7 @@ get_cmdline_location(const char cmd[], const char pos[])
 
 	switch(id)
 	{
+		case COM_CDS:
 		case COM_HIGHLIGHT:
 		case COM_FILTER:
 		case COM_SELECT:
@@ -1148,6 +1149,15 @@ commands_scope_start(void)
 	(void)int_stack_push(&if_levels, IF_SCOPE_GUARD);
 }
 
+void
+commands_scope_escape(void)
+{
+	while(!is_at_scope_bottom(&if_levels))
+	{
+		int_stack_pop(&if_levels);
+	}
+}
+
 int
 commands_scope_finish(void)
 {
