@@ -73,6 +73,10 @@ int exec_command(const char cmd[], struct view_t *view, CmdInputType type);
  * script). */
 void commands_scope_start(void);
 
+/* Marks active command execution scope as escaped meaning that there is no need
+ * to check for endif. */
+void commands_scope_escape(void);
+
 /* Should terminate command execution scope (e.g. end of sourced script).
  * Performs some of internal checks.  Returns non-zero when there were errors,
  * otherwise zero is returned. */
