@@ -92,6 +92,15 @@ static void free_compare_records(void *ptr);
 int
 compare_two_panes(CompareType ct, ListType lt, int group_paths, int skip_empty)
 {
+	/* We don't compare lists of files, so skip the check if at least one of the
+	 * views is a custom one. */
+	if(!flist_custom_active(&lwin) && !flist_custom_active(&rwin) &&
+			paths_are_same(flist_get_dir(&lwin), flist_get_dir(&rwin)))
+	{
+		ui_sb_err("Both views are at the same location");
+		return 1;
+	}
+
 	int next_id = 1;
 	entries_t curr, other;
 
@@ -633,7 +642,7 @@ list_files_recursively(const char path[], int skip_dot_files, strlist_t *list)
 			continue;
 		}
 
-		full_path = format_str("%s/%s", path, lst[i]);
+		full_path = join_paths(path, lst[i]);
 		if(is_dir(full_path))
 		{
 			if(!is_symlink(full_path))
