@@ -34,12 +34,6 @@ compat_wget_wch(WINDOW *w, wint_t *wc)
 	return ((char)*wc == ERR) ? ERR : OK;
 }
 
-int
-compat_unget_wch(wint_t wc)
-{
-	return ungetch(wc);
-}
-
 int
 compat_waddwstr(WINDOW *w, const wchar_t wstr[])
 {
