@@ -38,7 +38,6 @@
 #ifndef __OpenBSD__
 
 #define compat_wget_wch wget_wch
-#define compat_unget_wch unget_wch
 #define compat_waddwstr waddwstr
 #define compat_mvwaddwstr mvwaddwstr
 
@@ -46,8 +45,6 @@
 
 int compat_wget_wch(WINDOW *w, wint_t *wc);
 
-int compat_unget_wch(wint_t wc);
-
 int compat_waddwstr(WINDOW *w, const wchar_t wstr[]);
 
 int compat_mvwaddwstr(WINDOW *w, int y, int x, const wchar_t wstr[]);
