@@ -38,48 +38,6 @@
 #define MAX_CMD_NAME_LEN 256
 #define INVALID_MARK -4096
 
-typedef enum
-{
-	BUILTIN_ABBR,
-	BUILTIN_CMD,
-	USER_CMD,
-}
-CMD_TYPE;
-
-typedef struct cmd_t
-{
-	char *name;
-	const char *descr; /* Brief description of the command. */
-	int id;
-	CMD_TYPE type;
-	int passed;
-
-	cmd_handler handler; /* Handler for builtin commands. */
-	char *cmd;           /* Command-line for user-defined commands. */
-
-	int min_args, max_args;
-
-	unsigned int range : 1;            /* Handles ranges. */
-	unsigned int cust_sep : 1;         /* Custom separator of arguments. */
-	unsigned int emark : 1;            /* Supports emark flag. */
-	unsigned int envvars : 1;          /* Expand environment variables. */
-	unsigned int select : 1;           /* Select files in a range. */
-	unsigned int bg : 1;               /* Bg (can have " &" at the end). */
-	unsigned int noescaping : 1;       /* Don't process \-escaping in unquoted
-	                                      args. */
-	unsigned int regexp : 1;           /* Process /.../-arguments. */
-	unsigned int quote : 1;            /* Process '- and "-quoted args. */
-	unsigned int comment : 1;          /* Trailing comment is allowed. */
-	unsigned int qmark : 1;            /* No args after qmark. */
-	unsigned int args_after_qmark : 1; /* Args after qmark are allowed. */
-	unsigned int macros_for_cmd : 1;   /* Expand macros w/o special escaping. */
-	unsigned int macros_for_shell : 1; /* Expand macros with shell escaping. */
-	unsigned int : 0;                  /* Padding. */
-
-	struct cmd_t *next;
-}
-cmd_t;
-
 typedef struct
 {
 	cmd_t head;
@@ -115,7 +73,7 @@ static int command_cmd(const cmd_info_t *cmd_info);
 static void init_command_flags(cmd_t *cmd, int flags);
 static const char * get_user_cmd_name(const char cmd[], char buf[],
 		size_t buf_len);
-static int is_correct_name(const char name[]);
+static int is_valid_udc_name(const char name[]);
 static cmd_t * insert_cmd(cmd_t *after);
 static int delcommand_cmd(const cmd_info_t *cmd_info);
 TSTATIC char ** dispatch_line(const char args[], int *count, char sep,
@@ -124,21 +82,24 @@ TSTATIC char ** dispatch_line(const char args[], int *count, char sep,
 static int is_separator(char c, char sep);
 
 void
-init_cmds(int udf, cmds_conf_t *conf)
+vle_cmds_init(int udf, cmds_conf_t *conf)
 {
 	static cmd_add_t commands[] = {
 		{
-			.name = "comclear",   .abbr = "comc", .handler = comclear_cmd,   .id = COMCLEAR_CMD_ID,
+			.name = "comclear",        .abbr = "comc", .handler = comclear_cmd,
+			.id = COMCLEAR_CMD_ID,
 			.descr = "remove all user-defined :commands",
 			.flags = 0,
 			.min_args = 0,             .max_args = 0,
 		}, {
-			.name = "command",    .abbr = "com",  .handler = command_cmd,    .id = COMMAND_CMD_ID,
+			.name = "command",         .abbr = "com",  .handler = command_cmd,
+			.id = COMMAND_CMD_ID,
 			.descr = "display/define user-defined :command",
 			.flags = HAS_EMARK,
 			.min_args = 0,             .max_args = NOT_DEF,
 		}, {
-			.name = "delcommand", .abbr = "delc", .handler = delcommand_cmd, .id = DELCOMMAND_CMD_ID,
+			.name = "delcommand", .abbr = "delc", .handler = delcommand_cmd,
+			.id = DELCOMMAND_CMD_ID,
 			.descr = "undefine user-defined :command",
 			.flags = HAS_EMARK,
 			.min_args = 1,             .max_args = 1,
@@ -163,12 +124,12 @@ init_cmds(int udf, cmds_conf_t *conf)
 		inner = conf->inner;
 
 		if(udf)
-			add_builtin_commands(commands, ARRAY_LEN(commands));
+			vle_cmds_add(commands, ARRAY_LEN(commands));
 	}
 }
 
 void
-reset_cmds(void)
+vle_cmds_reset(void)
 {
 	cmd_t *cur = inner->head.next;
 
@@ -189,7 +150,7 @@ reset_cmds(void)
 }
 
 int
-execute_cmd(const char cmd[])
+vle_cmds_run(const char cmd[])
 {
 	cmd_info_t cmd_info;
 	char cmd_name[MAX_CMD_NAME_LEN];
@@ -237,7 +198,7 @@ execute_cmd(const char cmd[])
 
 	/* Set background flag and remove background mark from raw arguments, when
 	 * command supports backgrounding. */
-	last_arg = get_last_argument(cmd_info.raw_args, cur->quote, &last_arg_len);
+	last_arg = vle_cmds_last_arg(cmd_info.raw_args, cur->quote, &last_arg_len);
 	if(cur->bg && *last_arg == '&' && *vle_cmds_at_arg(last_arg + 1) == '\0')
 	{
 		cmd_info.bg = 1;
@@ -309,7 +270,7 @@ execute_cmd(const char cmd[])
 	}
 	else
 	{
-		cur->passed++;
+		++cur->passed;
 
 		if(cur->type == USER_CMD)
 		{
@@ -322,7 +283,10 @@ execute_cmd(const char cmd[])
 		}
 
 		cc->post(cur->id);
-		cur->passed--;
+		if(--cur->passed == 0 && cur->deleted)
+		{
+			free(cur);
+		}
 	}
 
 	free(cmd_info.raw_args);
@@ -440,17 +404,18 @@ parse_tail(cmd_t *cur, const char cmd[], cmd_info_t *cmd_info)
 }
 
 int
-get_cmd_id(const char cmd[])
+vle_cmds_identify(const char cmd[])
 {
 	cmd_info_t info;
-	return get_cmd_info(cmd, &info);
+	const cmd_t *const c = vle_cmds_parse(cmd, &info);
+	return (c == NULL ? -1 : c->id);
 }
 
 const char *
-get_cmd_args(const char cmd[])
+vle_cmds_args(const char cmd[])
 {
 	cmd_info_t info = {};
-	(void)get_cmd_info(cmd, &info);
+	(void)vle_cmds_parse(cmd, &info);
 	return info.raw_args;
 }
 
@@ -474,33 +439,36 @@ init_cmd_info(cmd_info_t *cmd_info)
 	cmd_info->usr2 = 0;
 }
 
-/* Returns command id */
-int
-get_cmd_info(const char cmd[], cmd_info_t *info)
+const cmd_t *
+vle_cmds_parse(const char cmd[], cmd_info_t *info)
 {
 	cmd_info_t cmd_info;
-	char cmd_name[MAX_CMD_NAME_LEN];
+	char cmd_name[MAX_CMD_NAME_LEN + 1];
 	cmd_t *c;
 
 	init_cmd_info(&cmd_info);
 
 	cmd = parse_range(cmd, &cmd_info);
 	if(cmd == NULL)
-		return CMDS_ERR_INVALID_CMD;
+	{
+		return NULL;
+	}
 
 	cmd = get_cmd_name(cmd, cmd_name, sizeof(cmd_name));
 	c = find_cmd(cmd_name);
 	if(c == NULL)
-		return -1;
+	{
+		return NULL;
+	}
 
 	cmd_info.raw_args = (char *)parse_tail(c, cmd, &cmd_info);
 
 	*info = cmd_info;
-	return c->id;
+	return c;
 }
 
 int
-complete_cmd(const char cmd[], void *arg)
+vle_cmds_complete(const char cmd[], void *arg)
 {
 	cmd_info_t cmd_info;
 	const char *begin, *cmd_name_pos;
@@ -747,7 +715,14 @@ get_cmd_name(const char cmd[], char buf[], size_t buf_len)
 		{
 			if(cmp == 0)
 			{
-				/* Check for user-defined command that ends with the char. */
+				/* Complete match for a builtin with a custom separator. */
+				if(cur->cust_sep && cur->name[len] == '\0')
+				{
+					strncpy(buf, cur->name, buf_len);
+					break;
+				}
+				/* Check for user-defined command that ends with the char ('!' or
+				 * '?', see above). */
 				if(cur->type == USER_CMD && cur->name[strlen(cur->name) - 1] == *t)
 				{
 					strncpy(buf, cur->name, buf_len);
@@ -763,10 +738,11 @@ get_cmd_name(const char cmd[], char buf[], size_t buf_len)
 			}
 			cur = cur->next;
 		}
-		/* For builtin commands, the char is not part of the name. */
+
 		if(cur != NULL && cur->type == USER_CMD &&
 				strncmp(cur->name, buf, len) == 0)
 		{
+			/* For user-defined commands, the char is part of the name. */
 			++t;
 		}
 	}
@@ -849,7 +825,7 @@ complete_cmd_name(const char cmd_name[], int user_only)
 }
 
 void
-add_builtin_commands(const cmd_add_t cmds[], int count)
+vle_cmds_add(const cmd_add_t cmds[], int count)
 {
 	int i;
 	for(i = 0; i < count; ++i)
@@ -940,6 +916,7 @@ add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf)
 	new->cmd = NULL;
 	new->min_args = conf->min_args;
 	new->max_args = conf->max_args;
+	new->deleted = 0;
 	init_command_flags(new, conf->flags);
 
 	return 0;
@@ -959,7 +936,14 @@ comclear_cmd(const cmd_info_t *cmd_info)
 
 			free(this->cmd);
 			free(this->name);
-			free(this);
+			if(this->passed == 0)
+			{
+				free(this);
+			}
+			else
+			{
+				this->deleted = 1;
+			}
 		}
 		else
 		{
@@ -992,7 +976,7 @@ command_cmd(const cmd_info_t *cmd_info)
 	args = vle_cmds_at_arg(args);
 	if(args[0] == '\0')
 		return CMDS_ERR_TOO_FEW_ARGS;
-	else if(!is_correct_name(cmd_name))
+	else if(!is_valid_udc_name(cmd_name))
 		return CMDS_ERR_INCORRECT_NAME;
 
 	len = strlen(cmd_name);
@@ -1027,7 +1011,7 @@ command_cmd(const cmd_info_t *cmd_info)
 			return CMDS_ERR_NEED_BANG;
 		free(cur->name);
 		free(cur->cmd);
-    new = cur;
+		new = cur;
 	}
 	else
 	{
@@ -1045,6 +1029,7 @@ command_cmd(const cmd_info_t *cmd_info)
 	new->cmd = strdup(args);
 	new->min_args = inner->user_cmd_handler.min_args;
 	new->max_args = inner->user_cmd_handler.max_args;
+	new->deleted = 0;
 	init_command_flags(new, inner->user_cmd_handler.flags);
 
 	++inner->udf_count;
@@ -1095,15 +1080,20 @@ get_user_cmd_name(const char cmd[], char buf[], size_t buf_len)
 	return t;
 }
 
+/* Checks that the name is a valid name for a user-defined command. */
 static int
-is_correct_name(const char name[])
+is_valid_udc_name(const char name[])
 {
+	assert(name[0] != '\0' && "Command name can't be empty");
+
 	if(strcmp(name, "!") == 0)
 		return 0;
-
 	if(strcmp(name, "?") == 0)
 		return 0;
 
+	char cmd_name[MAX_CMD_NAME_LEN + 1];
+	copy_str(cmd_name, sizeof(cmd_name), name);
+
 	while(name[0] != '\0')
 	{
 		if(!isalpha(name[0]))
@@ -1115,6 +1105,17 @@ is_correct_name(const char name[])
 		}
 		name++;
 	}
+
+	/* Builtins with custom separator have higher priority.  Disallow registering
+	 * user-defined commands which will never be called. */
+	if(name[-1] == '!' || name[-1] == '?')
+	{
+		cmd_name[strlen(cmd_name) - 1] = '\0';
+	}
+	const cmd_t *const c = find_cmd(cmd_name);
+	if(c != NULL && c->cust_sep && strcmp(c->name, cmd_name) == 0)
+		return 0;
+
 	return 1;
 }
 
@@ -1161,7 +1162,7 @@ delcommand_cmd(const cmd_info_t *cmd_info)
 }
 
 char *
-get_last_argument(const char cmd[], int quotes, size_t *len)
+vle_cmds_last_arg(const char cmd[], int quotes, size_t *len)
 {
 	int argc;
 	char **argv;
@@ -1385,7 +1386,7 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 }
 
 char **
-list_udf(void)
+vle_cmds_list_udcs(void)
 {
 	char **p;
 	cmd_t *cur;
@@ -1415,7 +1416,7 @@ list_udf(void)
 }
 
 char *
-list_udf_content(const char beginning[])
+vle_cmds_print_udcs(const char beginning[])
 {
 	size_t len;
 	cmd_t *cur;
