@@ -29,15 +29,14 @@
 #include <ctype.h>
 #include <limits.h> /* INT_MAX */
 #include <stddef.h> /* NULL wchar_t */
+#include <stdio.h> /* snprintf() */
 #include <stdlib.h>
 #include <string.h>
 #include <wctype.h> /* iswdigit() */
 #include <wchar.h> /* wcscat() wcslen() */
 
-#include "../compat/reallocarray.h"
 #include "../utils/macros.h"
 #include "../utils/str.h"
-#include "../utils/string_array.h"
 #include "mode.h"
 
 /* Type of key chunk. */
@@ -321,6 +320,7 @@ static int
 dispatch_keys(const wchar_t keys[], keys_info_t *keys_info, int no_remap,
 		int prev_count)
 {
+	const wchar_t *keys_start = keys;
 	key_info_t key_info;
 	int result;
 
@@ -342,6 +342,13 @@ dispatch_keys(const wchar_t keys[], keys_info_t *keys_info, int no_remap,
 				&builtin_cmds_root[vle_mode_get()], key_info, no_remap);
 	}
 
+	if(!IS_KEYS_RET_CODE(result))
+	{
+		/* Take characters spent on count and register specification into
+		 * account. */
+		inc_counter(keys_info, keys - keys_start);
+	}
+
 	return result;
 }
 
