@@ -54,6 +54,7 @@
 #include "private/options.h"
 #include "functions.h"
 #include "options.h"
+#include "text_buffer.h"
 #include "var.h"
 #include "variables.h"
 
@@ -61,7 +62,7 @@
 #define CMD_LINE_LENGTH_MAX 4096
 
 /* Maximum number of characters in option's name. */
-const size_t OPTION_NAME_MAX = 64;
+static const size_t OPTION_NAME_MAX = 64;
 
 /* Supported types of tokens. */
 typedef enum
@@ -103,6 +104,13 @@ typedef enum
 }
 Ops;
 
+/* Evaluation context that's passed to all eval_*() functions. */
+typedef struct
+{
+	const int interactive; /* Whether call is being executed by the user. */
+}
+eval_context_t;
+
 /* Defines expression and how to evaluate its value.  Value is stored here after
  * evaluation. */
 typedef struct expr_t
@@ -124,19 +132,22 @@ typedef struct
 }
 sbuffer;
 
-static int eval_expr(expr_t *expr);
-static int eval_or_op(int nops, expr_t ops[], var_t *result);
-static int eval_and_op(int nops, expr_t ops[], var_t *result);
-static int eval_call_op(const char name[], int nops, expr_t ops[],
+static int eval_expr(eval_context_t *ctx, expr_t *expr);
+static int eval_or_op(eval_context_t *ctx, int nops, expr_t ops[],
 		var_t *result);
+static int eval_and_op(eval_context_t *ctx, int nops, expr_t ops[],
+		var_t *result);
+static int eval_call_op(eval_context_t *ctx, const char name[], int nops,
+		expr_t ops[], var_t *result);
 static int compare_variables(TOKENS_TYPE operation, var_t lhs, var_t rhs);
-static var_t eval_concat(int nops, expr_t ops[]);
+static var_t eval_concat(eval_context_t *ctx, int nops, expr_t ops[]);
 static int add_expr_op(expr_t *expr, const expr_t *arg);
 static void free_expr(const expr_t *expr);
 static expr_t parse_or_expr(const char **in);
 static expr_t parse_and_expr(const char **in);
 static expr_t parse_comp_expr(const char **in);
 static int is_comparison_operator(TOKENS_TYPE type);
+static expr_t parse_factor(const char **in);
 static expr_t parse_concat_expr(const char **in);
 static expr_t parse_term(const char **in);
 static expr_t parse_signed_number(const char **in);
@@ -199,7 +210,7 @@ get_last_parsed_char(void)
 }
 
 ParsingErrors
-parse(const char input[], var_t *result)
+parse(const char input[], int interactive, var_t *result)
 {
 	expr_t expr_root;
 
@@ -213,6 +224,11 @@ parse(const char input[], var_t *result)
 	expr_root = parse_or_expr(&last_position);
 	last_parsed_char = last_position;
 
+	var_free(res_val);
+	res_val = var_error();
+
+	eval_context_t ctx = { .interactive = interactive };
+
 	if(last_token.type != END)
 	{
 		if(last_parsed_char > input)
@@ -226,9 +242,8 @@ parse(const char input[], var_t *result)
 				/* This is a comment, just ignore it. */
 				last_position += strlen(last_position);
 			}
-			else if(eval_expr(&expr_root) == 0)
+			else if(eval_expr(&ctx, &expr_root) == 0)
 			{
-				var_free(res_val);
 				res_val = var_clone(expr_root.value);
 				last_error = PE_INVALID_EXPRESSION;
 			}
@@ -237,9 +252,8 @@ parse(const char input[], var_t *result)
 
 	if(last_error == PE_NO_ERROR)
 	{
-		if(eval_expr(&expr_root) == 0)
+		if(eval_expr(&ctx, &expr_root) == 0)
 		{
-			var_free(res_val);
 			res_val = var_clone(expr_root.value);
 			*result = var_clone(expr_root.value);
 		}
@@ -273,7 +287,7 @@ is_prev_token_whitespace(void)
 /* Evaluates values of an expression.  Returns zero on success, which means that
  * expr->value is now correct, otherwise non-zero is returned. */
 static int
-eval_expr(expr_t *expr)
+eval_expr(eval_context_t *ctx, expr_t *expr)
 {
 	int result = 1;
 	switch(expr->op_type)
@@ -282,14 +296,15 @@ eval_expr(expr_t *expr)
 			/* Do nothing, value is already available. */
 			return 0;
 		case OP_OR:
-			result = eval_or_op(expr->nops, expr->ops, &expr->value);
+			result = eval_or_op(ctx, expr->nops, expr->ops, &expr->value);
 			break;
 		case OP_AND:
-			result = eval_and_op(expr->nops, expr->ops, &expr->value);
+			result = eval_and_op(ctx, expr->nops, expr->ops, &expr->value);
 			break;
 		case OP_CALL:
 			assert(expr->func != NULL && "Function must have a name.");
-			result = eval_call_op(expr->func, expr->nops, expr->ops, &expr->value);
+			result = eval_call_op(ctx, expr->func, expr->nops, expr->ops,
+					&expr->value);
 			break;
 	}
 	if(result == 0)
@@ -302,7 +317,7 @@ eval_expr(expr_t *expr)
 /* Evaluates logical OR operation.  All operands are evaluated lazily from left
  * to right.  Returns zero on success, otherwise non-zero is returned. */
 static int
-eval_or_op(int nops, expr_t ops[], var_t *result)
+eval_or_op(eval_context_t *ctx, int nops, expr_t ops[], var_t *result)
 {
 	int val;
 	int i;
@@ -313,7 +328,7 @@ eval_or_op(int nops, expr_t ops[], var_t *result)
 		return 0;
 	}
 
-	if(eval_expr(&ops[0]) != 0)
+	if(eval_expr(ctx, &ops[0]) != 0)
 	{
 		return 1;
 	}
@@ -326,15 +341,15 @@ eval_or_op(int nops, expr_t ops[], var_t *result)
 
 	/* Conversion to integer so that strings are converted into numbers instead of
 	 * checked to be empty. */
-	val = var_to_integer(ops[0].value);
+	val = var_to_int(ops[0].value);
 
 	for(i = 1; i < nops && !val; ++i)
 	{
-		if(eval_expr(&ops[i]) != 0)
+		if(eval_expr(ctx, &ops[i]) != 0)
 		{
 			return 1;
 		}
-		val |= var_to_integer(ops[i].value);
+		val |= var_to_int(ops[i].value);
 	}
 
 	*result = var_from_bool(val);
@@ -344,7 +359,7 @@ eval_or_op(int nops, expr_t ops[], var_t *result)
 /* Evaluates logical AND operation.  All operands are evaluated lazily from left
  * to right.  Returns zero on success, otherwise non-zero is returned. */
 static int
-eval_and_op(int nops, expr_t ops[], var_t *result)
+eval_and_op(eval_context_t *ctx, int nops, expr_t ops[], var_t *result)
 {
 	int val;
 	int i;
@@ -355,7 +370,7 @@ eval_and_op(int nops, expr_t ops[], var_t *result)
 		return 0;
 	}
 
-	if(eval_expr(&ops[0]) != 0)
+	if(eval_expr(ctx, &ops[0]) != 0)
 	{
 		return 1;
 	}
@@ -368,15 +383,15 @@ eval_and_op(int nops, expr_t ops[], var_t *result)
 
 	/* Conversion to integer so that strings are converted into numbers instead of
 	 * checked to be empty. */
-	val = var_to_integer(ops[0].value);
+	val = var_to_int(ops[0].value);
 
 	for(i = 1; i < nops && val; ++i)
 	{
-		if(eval_expr(&ops[i]) != 0)
+		if(eval_expr(ctx, &ops[i]) != 0)
 		{
 			return 1;
 		}
-		val &= var_to_integer(ops[i].value);
+		val &= var_to_int(ops[i].value);
 	}
 
 	*result = var_from_bool(val);
@@ -386,13 +401,14 @@ eval_and_op(int nops, expr_t ops[], var_t *result)
 /* Evaluates invocation operation.  All operands are evaluated beforehand.
  * Returns zero on success, otherwise non-zero is returned. */
 static int
-eval_call_op(const char name[], int nops, expr_t ops[], var_t *result)
+eval_call_op(eval_context_t *ctx, const char name[], int nops, expr_t ops[],
+		var_t *result)
 {
 	int i;
 
 	for(i = 0; i < nops; ++i)
 	{
-		if(eval_expr(&ops[i]) != 0)
+		if(eval_expr(ctx, &ops[i]) != 0)
 		{
 			return 1;
 		}
@@ -430,28 +446,33 @@ eval_call_op(const char name[], int nops, expr_t ops[], var_t *result)
 	}
 	else if(strcmp(name, ".") == 0)
 	{
-		*result = eval_concat(nops, ops);
+		*result = eval_concat(ctx, nops, ops);
 	}
 	else if(strcmp(name, "!") == 0)
 	{
 		assert(nops == 1 && "Must be single argument.");
-		*result = var_from_bool(!var_to_integer(ops[0].value));
+		*result = var_from_bool(!var_to_int(ops[0].value));
 	}
 	else if(strcmp(name, "-") == 0 || strcmp(name, "+") == 0)
 	{
-		assert(nops == 1 && "Must be single argument.");
-		var_val_t val = { .integer = var_to_integer(ops[0].value) };
-		if(name[0] == '-')
+		if(nops == 1)
 		{
-			val.integer = -val.integer;
+			const int val = var_to_int(ops[0].value);
+			*result = var_from_int(name[0] == '-' ? -val : val);
+		}
+		else
+		{
+			assert(nops == 2 && "Must be two arguments.");
+			const int a = var_to_int(ops[0].value);
+			const int b = var_to_int(ops[1].value);
+			*result = var_from_int(name[0] == '-' ? a - b : a + b);
 		}
-		*result = var_new(VTYPE_INT, val);
 	}
 	else
 	{
 		int i;
 		call_info_t call_info;
-		function_call_info_init(&call_info);
+		function_call_info_init(&call_info, ctx->interactive);
 
 		for(i = 0; i < nops; ++i)
 		{
@@ -496,8 +517,8 @@ compare_variables(TOKENS_TYPE operation, var_t lhs, var_t rhs)
 	}
 	else
 	{
-		const int lhs_int = var_to_integer(lhs);
-		const int rhs_int = var_to_integer(rhs);
+		const int lhs_int = var_to_int(lhs);
+		const int rhs_int = var_to_int(rhs);
 		switch(operation)
 		{
 			case EQ: return lhs_int == rhs_int;
@@ -517,10 +538,9 @@ compare_variables(TOKENS_TYPE operation, var_t lhs, var_t rhs)
 /* Evaluates concatenation of expressions.  Returns resultant value or variable
  * of type VTYPE_ERROR. */
 static var_t
-eval_concat(int nops, expr_t ops[])
+eval_concat(eval_context_t *ctx, int nops, expr_t ops[])
 {
-	var_t result = var_error();
-	char res[CMD_LINE_LENGTH_MAX];
+	char res[CMD_LINE_LENGTH_MAX + 1];
 	size_t res_len = 0U;
 	int i;
 
@@ -535,7 +555,7 @@ eval_concat(int nops, expr_t ops[])
 
 	for(i = 0; i < nops; ++i)
 	{
-		char *const str_val = var_to_string(ops[i].value);
+		char *const str_val = var_to_str(ops[i].value);
 		if(str_val == NULL)
 		{
 			last_error = PE_INTERNAL;
@@ -547,13 +567,7 @@ eval_concat(int nops, expr_t ops[])
 		free(str_val);
 	}
 
-	if(last_error == PE_NO_ERROR)
-	{
-		const var_val_t var_val = { .string = res };
-		result = var_new(VTYPE_STRING, var_val);
-	}
-
-	return result;
+	return (last_error == PE_NO_ERROR ? var_from_str(res) : var_error());
 }
 
 /* Appends operand to an expression.  Returns zero on success, otherwise
@@ -669,7 +683,7 @@ parse_and_expr(const char **in)
 	return result;
 }
 
-/* comp_expr ::= concat_expr | concat_expr op concat_expr
+/* comp_expr ::= factor | factor op factor
  * op ::= '==' | '!=' | '<' | '<=' | '>' | '>=' */
 static expr_t
 parse_comp_expr(const char **in)
@@ -678,7 +692,7 @@ parse_comp_expr(const char **in)
 	expr_t rhs;
 	expr_t result = { .op_type = OP_CALL };
 
-	lhs = parse_concat_expr(in);
+	lhs = parse_factor(in);
 	if(last_error != PE_NO_ERROR || !is_comparison_operator(last_token.type))
 	{
 		return lhs;
@@ -694,7 +708,7 @@ parse_comp_expr(const char **in)
 	}
 
 	get_next(in);
-	rhs = parse_concat_expr(in);
+	rhs = parse_factor(in);
 	if(add_expr_op(&result, &rhs) != 0)
 	{
 		free_expr(&result);
@@ -721,6 +735,55 @@ is_comparison_operator(TOKENS_TYPE type)
 	    || type == GE || type == GT;
 }
 
+/* factor ::= concat_expr { op concat_expr }
+ * op ::= '+' | '-' */
+static expr_t
+parse_factor(const char **in)
+{
+	expr_t result = parse_concat_expr(in);
+
+	while(last_error == PE_NO_ERROR &&
+			(last_token.type == PLUS || last_token.type == MINUS))
+	{
+		expr_t intermediate = { .op_type = OP_CALL };
+		expr_t next;
+
+		intermediate.func = strdup(last_token.str);
+		if(add_expr_op(&intermediate, &result) != 0 || intermediate.func == NULL)
+		{
+			last_error = PE_INTERNAL;
+			free_expr(&intermediate);
+			return null_expr;
+		}
+
+		get_next(in);
+		next = parse_concat_expr(in);
+		if(last_error != PE_NO_ERROR)
+		{
+			free_expr(&next);
+			free_expr(&intermediate);
+			return null_expr;
+		}
+
+		if(add_expr_op(&intermediate, &next) != 0)
+		{
+			last_error = PE_INTERNAL;
+			free_expr(&intermediate);
+			return null_expr;
+		}
+
+		result = intermediate;
+	}
+
+	if(last_error == PE_INTERNAL)
+	{
+		free_expr(&result);
+		return null_expr;
+	}
+
+	return result;
+}
+
 /* concat_expr ::= term { '.' term } */
 static expr_t
 parse_concat_expr(const char **in)
@@ -772,11 +835,12 @@ parse_concat_expr(const char **in)
 }
 
 /* term ::= signed_number | number | sqstr | dqstr | envvar | builtinvar |
- *          funccall | opt | logical_not */
+ *          funccall | opt | logical_not | '(' or_expr ')' */
 static expr_t
 parse_term(const char **in)
 {
 	expr_t result = { .op_type = OP_NONE };
+	const char *old_in = *in - 1;
 
 	switch(last_token.type)
 	{
@@ -807,6 +871,22 @@ parse_term(const char **in)
 			get_next(in);
 			result = parse_logical_not(in);
 			break;
+		case LPAREN:
+			get_next(in);
+			result = parse_or_expr(in);
+			if(last_token.type == RPAREN)
+			{
+				get_next(in);
+			}
+			else
+			{
+				free_expr(&result);
+				result = null_expr;
+				result.value = var_error();
+				last_error = PE_MISSING_PAREN;
+				last_position = old_in;
+			}
+			break;
 
 		case SYM:
 			if(char_is_one_of("abcdefghijklmnopqrstuvwxyz_", tolower(last_token.c)))
@@ -865,8 +945,6 @@ parse_signed_number(const char **in)
 static var_t
 parse_number(const char **in)
 {
-	var_val_t var_val = { };
-
 	char buffer[CMD_LINE_LENGTH_MAX];
 	size_t len = 0U;
 	buffer[0] = '\0';
@@ -882,15 +960,15 @@ parse_number(const char **in)
 	}
 	while(last_token.type == DIGIT);
 
-	var_val.integer = str_to_int(buffer);
-	return var_new(VTYPE_INT, var_val);
+	return var_from_int(str_to_int(buffer));
 }
 
 /* sqstr ::= ''' sqchar { sqchar } ''' */
 static var_t
 parse_singly_quoted_string(const char **in)
 {
-	char buffer[CMD_LINE_LENGTH_MAX];
+	char buffer[CMD_LINE_LENGTH_MAX + 1];
+	const char *old_in = *in - 2;
 	sbuffer sbuf = { .data = buffer, .size = sizeof(buffer) };
 	buffer[0] = '\0';
 	while(parse_singly_quoted_char(in, &sbuf));
@@ -902,12 +980,12 @@ parse_singly_quoted_string(const char **in)
 
 	if(last_token.type == SQ)
 	{
-		const var_val_t var_val = { .string = buffer };
 		get_next(in);
-		return var_new(VTYPE_STRING, var_val);
+		return var_from_str(buffer);
 	}
 
 	last_error = PE_MISSING_QUOTE;
+	last_position = old_in;
 	return var_false();
 }
 
@@ -944,7 +1022,8 @@ parse_singly_quoted_char(const char **in, sbuffer *sbuf)
 static var_t
 parse_doubly_quoted_string(const char **in)
 {
-	char buffer[CMD_LINE_LENGTH_MAX];
+	char buffer[CMD_LINE_LENGTH_MAX + 1];
+	const char *old_in = *in - 2;
 	sbuffer sbuf = { .data = buffer, .size = sizeof(buffer) };
 	buffer[0] = '\0';
 	while(parse_doubly_quoted_char(in, &sbuf));
@@ -956,14 +1035,12 @@ parse_doubly_quoted_string(const char **in)
 
 	if(last_token.type == DQ)
 	{
-		const var_val_t var_val = { .string = buffer };
 		get_next(in);
-		return var_new(VTYPE_STRING, var_val);
+		return var_from_str(buffer);
 	}
 
-	last_error = (last_token.type == END)
-	           ? PE_INVALID_EXPRESSION
-	           : PE_MISSING_QUOTE;
+	last_error = PE_MISSING_QUOTE;
+	last_position = old_in;
 	return var_false();
 }
 
@@ -1028,9 +1105,7 @@ parse_doubly_quoted_char(const char **in, sbuffer *sbuf)
 static var_t
 eval_envvar(const char **in)
 {
-	var_val_t var_val;
-
-	char name[VAR_NAME_LENGTH_MAX];
+	char name[VAR_NAME_LENGTH_MAX + 1];
 	if(!parse_sequence(in, ENV_VAR_NAME_FIRST_CHAR, ENV_VAR_NAME_CHARS,
 		sizeof(name), name))
 	{
@@ -1038,8 +1113,7 @@ eval_envvar(const char **in)
 		return var_false();
 	}
 
-	var_val.const_string = getenv_fu(name);
-	return var_new(VTYPE_STRING, var_val);
+	return var_from_str(getenv_fu(name));
 }
 
 /* builtinvar ::= 'v:' varname */
@@ -1047,7 +1121,7 @@ static var_t
 eval_builtinvar(const char **in)
 {
 	var_t var_value;
-	char name[VAR_NAME_LENGTH_MAX];
+	char name[VAR_NAME_LENGTH_MAX + 1];
 	strcpy(name, "v:");
 
 	if(last_token.c != 'v' || **in != ':')
@@ -1083,7 +1157,6 @@ eval_opt(const char **in)
 {
 	OPT_SCOPE scope = OPT_ANY;
 	const opt_t *option;
-	var_val_t var_val;
 
 	char name[OPTION_NAME_MAX + 1];
 
@@ -1113,21 +1186,17 @@ eval_opt(const char **in)
 		case OPT_STR:
 		case OPT_STRLIST:
 		case OPT_CHARSET:
-			var_val.string = option->val.str_val;
-			return var_new(VTYPE_STRING, var_val);
+			return var_from_str(option->val.str_val);
 
 		case OPT_BOOL:
-			var_val.integer = option->val.bool_val;
-			return var_new(VTYPE_INT, var_val);
+			return var_from_bool(option->val.bool_val);
 
 		case OPT_INT:
-			var_val.integer = option->val.int_val;
-			return var_new(VTYPE_INT, var_val);
+			return var_from_int(option->val.int_val);
 
 		case OPT_ENUM:
 		case OPT_SET:
-			var_val.const_string = get_value(option);
-			return var_new(VTYPE_STRING, var_val);
+			return var_from_str(get_value(option));
 
 		default:
 			assert(0 && "Unexpected option type");
@@ -1425,5 +1494,35 @@ get_next(const char **in)
 	last_token.str[*in - start] = '\0';
 }
 
+void
+report_parsing_error(ParsingErrors error)
+{
+	switch(error)
+	{
+		case PE_NO_ERROR:
+			/* Not an error. */
+			break;
+		case PE_INVALID_EXPRESSION:
+			vle_tb_append_linef(vle_err, "%s: %s", "Invalid expression",
+					get_last_position());
+			break;
+		case PE_INVALID_SUBEXPRESSION:
+			vle_tb_append_linef(vle_err, "%s: %s", "Invalid subexpression",
+					get_last_position());
+			break;
+		case PE_MISSING_QUOTE:
+			vle_tb_append_linef(vle_err, "%s: %s",
+					"Expression is missing closing quote", get_last_position());
+			break;
+		case PE_MISSING_PAREN:
+			vle_tb_append_linef(vle_err, "%s: %s",
+					"Expression is missing closing parenthesis", get_last_position());
+			break;
+		case PE_INTERNAL:
+			vle_tb_append_line(vle_err, "Internal error");
+			break;
+	}
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
