@@ -33,7 +33,7 @@ int is_input_buf_empty(void);
 
 TSTATIC_DEFS(
 	struct view_t;
-	int process_scheduled_updates_of_view(view_t *view);
+	int process_scheduled_updates_of_view(struct view_t *view);
 )
 
 #endif /* VIFM__EVENT_LOOP_H__ */
