@@ -32,6 +32,7 @@
 
 #include <assert.h> /* assert() */
 #include <errno.h> /* errno */
+#include <limits.h> /* INT_MIN */
 #include <stddef.h> /* NULL size_t */
 #include <stdint.h> /* intptr_t uint64_t */
 #include <stdio.h> /* snprintf() */
@@ -176,7 +177,7 @@ static int init_parent_entry(view_t *view, dir_entry_t *entry,
 void
 init_filelists(void)
 {
-	fview_init();
+	fview_setup();
 
 	flist_init_view(&rwin);
 	flist_init_view(&lwin);
@@ -189,7 +190,7 @@ void
 flist_init_view(view_t *view)
 {
 	init_flist(view);
-	fview_view_init(view);
+	fview_init(view);
 
 	reset_view(view);
 
@@ -202,6 +203,7 @@ static void
 init_flist(view_t *view)
 {
 	view->list_pos = 0;
+	view->last_seen_pos = -1;
 	view->history_num = 0;
 	view->history_pos = 0;
 	view->on_slow_fs = 0;
@@ -291,6 +293,8 @@ flist_free_view(view_t *view)
 	flist_free_cache(view, &view->left_column);
 	flist_free_cache(view, &view->right_column);
 
+	update_string(&view->last_curr_file, NULL);
+
 	free_string_array(view->saved_selection, view->nsaved_selection);
 	view->nsaved_selection = 0;
 	view->saved_selection = NULL;
@@ -310,6 +314,7 @@ flist_free_view(view_t *view)
 	update_string(&view->preview_prg_g, NULL);
 
 	view_info_free(view->vi);
+	view->vi = NULL;
 
 	regfree(&view->primary_group);
 }
@@ -326,7 +331,7 @@ reset_views(void)
 static void
 reset_view(view_t *view)
 {
-	fview_view_reset(view);
+	fview_reset(view);
 	filters_view_reset(view);
 
 	view->hide_dot_g = view->hide_dot = 1;
@@ -710,6 +715,10 @@ change_directory(view_t *view, const char directory[])
 	{
 		on_location_change(view, location_changed);
 	}
+	if(location_changed)
+	{
+		view->location_changed = 1;
+	}
 	return 0;
 }
 
@@ -1101,6 +1110,10 @@ flist_custom_finish_internal(view_t *view, CVType type, int reload,
 	if(!reload)
 	{
 		on_location_change(view, 0);
+		if(cfg.cvoptions & CVO_AUTOCMDS)
+		{
+			vle_aucmd_execute("DirEnter", flist_get_dir(view), view);
+		}
 	}
 
 	sort_dir_list(0, view);
@@ -1130,10 +1143,6 @@ on_location_change(view_t *view, int force)
 	{
 		reset_local_options(view);
 	}
-	if(force || (cfg.cvoptions & CVO_AUTOCMDS))
-	{
-		vle_aucmd_execute("DirEnter", view->curr_dir, view);
-	}
 }
 
 /* Disables view sorting saving its state for the future. */
@@ -1720,10 +1729,14 @@ populate_dir_list_internal(view_t *view, int reload)
 	 * the current line. */
 	if(!reload)
 	{
+		/* XXX: why cursor is positioned in code that loads the list? */
 		flist_hist_lookup(view, view);
 	}
 
-	fview_dir_updated(view);
+	if(view->location_changed)
+	{
+		fview_dir_updated(view);
+	}
 
 	if(view->list_rows < 1)
 	{
@@ -1745,6 +1758,12 @@ populate_dir_list_internal(view_t *view, int reload)
 	 * changed while we were reading from it). */
 	update_dir_watcher(view);
 
+	if(view->location_changed)
+	{
+		view->location_changed = 0;
+		vle_aucmd_execute("DirEnter", view->curr_dir, view);
+	}
+
 	restore_cwd(saved_cwd);
 	return 0;
 }
@@ -3075,6 +3094,7 @@ go_to_sibling_dir(view_t *view, int offset, int wrap)
 		if(change_directory(view, full_path) >= 0)
 		{
 			load_dir_list(view, 0);
+			fview_position_updated(view);
 		}
 	}
 
