@@ -25,6 +25,7 @@
 #include "ui/fileview.h"
 #include "ui/ui.h"
 #include "utils/fs.h"
+#include "utils/macros.h"
 #include "utils/path.h"
 #include "utils/str.h"
 #include "filelist.h"
@@ -99,7 +100,7 @@ navigate_to_history_pos(view_t *view, int pos)
 	curr_stats.drop_new_dir_hist = 0;
 
 	load_dir_list(view, 0);
-	flist_set_pos(view, find_file_pos_in_list(view, view->history[pos].file));
+	fpos_set_pos(view, fpos_find_by_name(view, view->history[pos].file));
 
 	view->history_pos = pos;
 }
@@ -360,7 +361,7 @@ find_in_hist(const view_t *view, const view_t *source, int *pos, int *rel_pos)
 	const history_t *const hist_entry = find_hist_entry(source, view->curr_dir);
 	if(hist_entry != NULL)
 	{
-		*pos = find_file_pos_in_list(view, hist_entry->file);
+		*pos = fpos_find_by_name(view, hist_entry->file);
 		*rel_pos = hist_entry->rel_pos;
 		return 1;
 	}
@@ -373,7 +374,7 @@ find_in_hist(const view_t *view, const view_t *source, int *pos, int *rel_pos)
 		 * `cd ..` or equivalent. */
 
 		const char *const dir_name = view->last_dir + strlen(view->curr_dir) + 1U;
-		*pos = find_file_pos_in_list(view, dir_name);
+		*pos = fpos_find_by_name(view, dir_name);
 		*rel_pos = -1;
 		return 1;
 	}
@@ -422,5 +423,24 @@ find_hist_entry(const view_t *view, const char dir[])
 	return NULL;
 }
 
+void
+flist_hist_clone(view_t *dst, const view_t *src)
+{
+	int i;
+
+	free_view_history_items(dst->history, dst->history_num);
+	dst->history_pos = 0;
+	dst->history_num = 0;
+
+	for(i = 0; i < src->history_num; ++i)
+	{
+		const history_t *const hist_entry = &src->history[i];
+		flist_hist_save(dst, hist_entry->dir, hist_entry->file,
+				hist_entry->rel_pos);
+	}
+
+	dst->history_pos = MIN(src->history_pos, dst->history_num - 1);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
