@@ -595,8 +595,7 @@ static int
 complete_filename(const char str[], void *arg)
 {
 	const char *name_begin = after_last(str, '/');
-	filename_completion(str, CT_ALL_WOE, 0);
-	return name_begin - str;
+	return name_begin - str + filename_completion(str, CT_ALL_WOE, 0);
 }
 
 int
@@ -1178,8 +1177,7 @@ bg_cancellation_hook(void *arg)
 static void
 redraw_after_path_change(view_t *view, const char path[])
 {
-	if(path_starts_with(view->curr_dir, path) ||
-			flist_custom_active(view))
+	if(path_starts_with(view->curr_dir, path) || flist_custom_active(view))
 	{
 		ui_view_schedule_redraw(view);
 	}
