@@ -35,8 +35,8 @@ typedef enum
 {
 	/* Failed operation should be restarted. */
 	IO_ECR_RETRY,
-	/* Operation should be skipped. */
-	IO_ECR_SKIP,
+	/* The failure should be ignored. */
+	IO_ECR_IGNORE,
 	/* Whole action should be stopped (including all possible next operations). */
 	IO_ECR_BREAK
 }
@@ -54,8 +54,11 @@ typedef struct
 }
 ioe_err_t;
 
+/* Forward declaration for ioerr_cb. */
+struct io_args_t;
+
 /* Callback used to report information about errors occurred. */
-typedef IoErrCbResult (*ioerr_cb)(const ioe_err_t *err);
+typedef IoErrCbResult (*ioerr_cb)(struct io_args_t *args, const ioe_err_t *err);
 
 /* List of errors. */
 typedef struct
@@ -79,7 +82,11 @@ void ioe_errlst_init(ioe_errlst_t *elist);
 /* Frees resources allocated by error list.  elist can be NULL. */
 void ioe_errlst_free(ioe_errlst_t *elist);
 
+/* Converts list of errors from I/O module into multi-line string.  Returns
+ * newly allocated string. */
+char * ioe_errlst_to_str(const ioe_errlst_t *elist);
+
 #endif /* VIFM__IO__IOE_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
