@@ -168,6 +168,7 @@ ior_mv(io_args_t *args)
 	const char *const dst = args->arg2.dst;
 	const IoCrs crs = args->arg3.crs;
 	const io_confirm confirm = args->confirm;
+	int confirmed = 0;
 
 	if(crs == IO_CRS_FAIL && path_exists(dst, DEREF) && !is_case_change(src, dst))
 	{
@@ -198,6 +199,7 @@ ior_mv(io_args_t *args)
 		{
 			return 0;
 		}
+		confirmed = 1;
 	}
 
 	if(os_rename(src, dst) == 0)
@@ -217,8 +219,13 @@ ior_mv(io_args_t *args)
 		case EACCES:
 #endif
 			{
+				args->confirm = (confirmed ? NULL : confirm);
 				int result = ior_cp(args);
-				if(result == 0)
+				args->confirm = confirm;
+				/* When result is zero, there still might be errors if they were
+				 * ignored by the user.  Do not delete source in this case, some files
+				 * might be missing at the destination. */
+				if(result == 0 && args->result.errors.error_count == 0)
 				{
 					io_args_t rm_args = {
 						.arg1.path = src,
@@ -358,7 +365,7 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 	rel_part = full_path + strlen(cp_args->arg1.src);
 	dst_full_path = (rel_part[0] == '\0')
 	              ? cp_args->arg2.dst
-	              : (free_me = format_str("%s/%s", cp_args->arg2.dst, rel_part));
+	              : (free_me = join_paths(cp_args->arg2.dst, rel_part));
 
 	switch(action)
 	{
