@@ -29,6 +29,7 @@
 #include "cfg/config.h"
 #include "compat/fs_limits.h"
 #include "modes/dialogs/msg_dialog.h"
+#include "ui/quickview.h"
 #include "ui/ui.h"
 #include "utils/path.h"
 #include "utils/str.h"
@@ -66,7 +67,7 @@ static char * expand_directory_path(view_t *view, char *expanded, int quotes,
 static char * expand_register(const char curr_dir[], char expanded[],
 		int quotes, const char mod[], int key, int *well_formed, int for_shell);
 static char * expand_preview(char expanded[], int key, int *well_formed);
-static view_t * get_preview_view(view_t *view);
+static preview_area_t get_preview_area(view_t *view);
 static char * append_path_to_expanded(char expanded[], int quotes,
 		const char path[]);
 static char * append_to_expanded(char expanded[], const char str[]);
@@ -74,7 +75,7 @@ static char * add_missing_macros(char expanded[], size_t len, size_t nmacros,
 		custom_macro_t macros[]);
 
 char *
-expand_macros(const char command[], const char args[], MacroFlags *flags,
+ma_expand(const char command[], const char args[], MacroFlags *flags,
 		int for_shell)
 {
 	return expand_macros_i(command, args, flags, for_shell, &filter_all);
@@ -136,7 +137,7 @@ static char *
 expand_macros_i(const char command[], const char args[], MacroFlags *flags,
 		int for_shell, macro_filter_func filter)
 {
-	/* TODO: refactor this function expand_macros() */
+	/* TODO: refactor this function expand_macros_i() */
 	/* FIXME: repetitive len = strlen(expanded) could be optimized. */
 
 	static const char MACROS_WITH_QUOTING[] = "cCfFbdDr";
@@ -286,6 +287,12 @@ expand_macros_i(const char command[], const char args[], MacroFlags *flags,
 				{
 					return expanded;
 				}
+				/* Just skip %pd. */
+				if(key == 'd')
+				{
+					++x;
+					break;
+				}
 
 				expanded = expand_preview(expanded, key, &well_formed);
 				len = strlen(expanded);
@@ -498,31 +505,22 @@ expand_register(const char curr_dir[], char expanded[], int quotes,
 static char *
 expand_preview(char expanded[], int key, int *well_formed)
 {
-	view_t *view;
-	char num_str[32];
-	int h, w, x, y;
-	int param;
-	const int with_margin = !curr_stats.preview.clearing && cfg.extra_padding;
-
-	if(!char_is_one_of("hwxy", key))
+	*well_formed = char_is_one_of("hwxy", key);
+	if(!*well_formed)
 	{
 		*well_formed = 0;
 		return expanded;
 	}
 
-	*well_formed = 1;
-
-	view = get_preview_view(curr_view);
-
-	getbegyx(view->win, y, x);
-	getmaxyx(view->win, h, w);
+	const preview_area_t parea = get_preview_area(curr_view);
 
+	int param;
 	switch(key)
 	{
-		case 'h': param = h - 2*with_margin; break;
-		case 'w': param = w - 2*with_margin; break;
-		case 'x': param = x + 1*with_margin; break;
-		case 'y': param = y + 1*with_margin; break;
+		case 'h': param = parea.h; break;
+		case 'w': param = parea.w; break;
+		case 'x': param = getbegx(parea.view->win) + parea.x; break;
+		case 'y': param = getbegy(parea.view->win) + parea.y; break;
 
 		default:
 			assert(0 && "Unhandled preview property type");
@@ -530,31 +528,39 @@ expand_preview(char expanded[], int key, int *well_formed)
 			break;
 	}
 
+	char num_str[32];
 	snprintf(num_str, sizeof(num_str), "%d", param);
 
 	return append_to_expanded(expanded, num_str);
 }
 
-/* Applies heuristics to determine which view is going to be used for preview.
- * Returns the view. */
-static view_t *
-get_preview_view(view_t *view)
+/* Applies heuristics to determine area that is going to be used for preview.
+ * Returns the area. */
+static preview_area_t
+get_preview_area(view_t *view)
 {
 	view_t *const other = (view == curr_view) ? other_view : curr_view;
 
 	if(curr_stats.preview_hint != NULL)
 	{
-		return curr_stats.preview_hint;
+		return *(const preview_area_t *)curr_stats.preview_hint;
 	}
 
+	view_t *source = view;
 	if(curr_stats.preview.on || (!view->explore_mode && other->explore_mode))
 	{
-		return other;
-	}
-	else
-	{
-		return view;
+		view = other;
 	}
+
+	const preview_area_t parea = {
+		.source = source,
+		.view = view,
+		.x = ui_qv_left(view),
+		.y = ui_qv_top(view),
+		.w = ui_qv_width(view),
+		.h = ui_qv_height(view),
+	};
+	return parea;
 }
 
 /* Appends the path to the expanded string with either proper escaping or
@@ -618,8 +624,7 @@ ma_get_clear_cmd(const char cmd[])
 }
 
 char *
-expand_custom_macros(const char pattern[], size_t nmacros,
-		custom_macro_t macros[])
+ma_expand_custom(const char pattern[], size_t nmacros, custom_macro_t macros[])
 {
 	char *expanded = strdup("");
 	size_t len = 0;
@@ -698,7 +703,7 @@ add_missing_macros(char expanded[], size_t len, size_t nmacros,
 }
 
 const char *
-macros_to_str(MacroFlags flags)
+ma_flags_to_str(MacroFlags flags)
 {
 	switch(flags)
 	{
