@@ -57,7 +57,7 @@ show_apropos_menu(view_t *view, const char args[])
 
 	ui_sb_msg("apropos...");
 
-	cmd = expand_custom_macros(cfg.apropos_prg, ARRAY_LEN(macros), macros);
+	cmd = ma_expand_custom(cfg.apropos_prg, ARRAY_LEN(macros), macros);
 	save_msg = menus_capture(view, cmd, 0, &m, 0, 0);
 	free(cmd);
 	return save_msg;
@@ -82,7 +82,7 @@ execute_apropos_cb(view_t *view, menu_data_t *m)
 	snprintf(command, sizeof(command), "man %s %s", section, topic);
 
 	curr_stats.skip_shellout_redraw = 1;
-	exit_code = shellout(command, PAUSE_NEVER, 1);
+	exit_code = shellout(command, PAUSE_NEVER, 1, SHELL_BY_APP);
 	curr_stats.skip_shellout_redraw = 0;
 
 	if(exit_code != 0)
