@@ -21,11 +21,11 @@
 
 #include <stddef.h> /* wchar_t */
 
-#include "../ui/ui.h"
+struct view_t;
 
 /* Displays list of command-line mode abbreviations.  Returns non-zero if status
  * bar message should be saved. */
-int show_cabbrevs_menu(FileView *view);
+int show_cabbrevs_menu(struct view_t *view);
 
 /* Describes an abbreviation for printing it out.  The offset parameter
  * specifies additional left-side displacement.  Returns newly allocated
