@@ -19,7 +19,7 @@
 
 #include "colorscheme_menu.h"
 
-#include <stdlib.h> /* free() qsort() */
+#include <stdlib.h> /* free() */
 #include <string.h> /* strdup() strcmp() */
 
 #include "../compat/fs_limits.h"
@@ -28,9 +28,9 @@
 #include "../ui/ui.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
+#include "../utils/utils.h"
 #include "menus.h"
 
-static int sorter(const void *first, const void *second);
 static int execute_colorscheme_cb(view_t *view, menu_data_t *m);
 
 int
@@ -43,7 +43,7 @@ show_colorschemes_menu(view_t *view)
 
 	m.items = cs_list(&m.len);
 
-	qsort(m.items, m.len, sizeof(*m.items), &sorter);
+	safe_qsort(m.items, m.len, sizeof(*m.items), &strossorter);
 
 	/* It's safe to set m.pos to negative value, since menus.c handles this
 	 * correctly. */
@@ -56,15 +56,6 @@ show_colorschemes_menu(view_t *view)
 	return menus_enter(m.state, view);
 }
 
-/* Sorting function for qsort(). */
-static int
-sorter(const void *first, const void *second)
-{
-	const char *stra = *(const char **)first;
-	const char *strb = *(const char **)second;
-	return stroscmp(stra, strb);
-}
-
 /* Callback that is called when menu item is selected.  Should return non-zero
  * to stay in menu mode. */
 static int
