@@ -31,24 +31,25 @@
 #include "../utils/fs.h"
 #include "../utils/macros.h"
 #include "../utils/path.h"
+#include "../utils/str.h"
 #include "../utils/string_array.h"
 #include "../utils/utf8.h"
 #include "../marks.h"
 #include "menus.h"
 
-static int execute_mark_cb(FileView *view, menu_data_t *m);
-static KHandlerResponse mark_khandler(FileView *view, menu_data_t *m,
+static int execute_mark_cb(view_t *view, menu_data_t *m);
+static KHandlerResponse mark_khandler(view_t *view, menu_data_t *m,
 		const wchar_t keys[]);
 
 int
-show_marks_menu(FileView *view, const char marks[])
+show_marks_menu(view_t *view, const char marks[])
 {
 	int active_marks[NUM_MARKS];
 	int i;
 	size_t max_len;
 
 	static menu_data_t m;
-	init_menu_data(&m, view, strdup("Mark -- Directory -- File"),
+	menus_init_data(&m, view, strdup("Mark -- Directory -- File"),
 			strdup("No marks set"));
 	m.execute_handler = &execute_mark_cb;
 	m.key_handler = &mark_khandler;
@@ -70,8 +71,8 @@ show_marks_menu(FileView *view, const char marks[])
 	i = 0;
 	while(i < m.len)
 	{
-		char item_buf[PATH_MAX];
-		char *with_tilde;
+		char item_buf[PATH_MAX + 1];
+		char *display_path;
 		int overhead;
 		const mark_t *mark;
 		const char *file;
@@ -80,13 +81,6 @@ show_marks_menu(FileView *view, const char marks[])
 
 		mark = get_mark(mn);
 
-		with_tilde = replace_home_part(mark->directory);
-		if(utf8_strsw(with_tilde) > max_len - 3)
-		{
-			size_t width = utf8_nstrsnlen(with_tilde, max_len - 6);
-			strcpy(with_tilde + width, "...");
-		}
-
 		if(!is_valid_mark(mn))
 		{
 			file = "[invalid]";
@@ -97,7 +91,7 @@ show_marks_menu(FileView *view, const char marks[])
 		}
 		else
 		{
-			char path[PATH_MAX];
+			char path[PATH_MAX + 1];
 
 			file = mark->file;
 
@@ -108,21 +102,24 @@ show_marks_menu(FileView *view, const char marks[])
 			}
 		}
 
-		overhead = utf8_strso(with_tilde);
+		display_path = right_ellipsis(replace_home_part(mark->directory),
+				max_len - 3, curr_stats.ellipsis);
+		overhead = utf8_strso(display_path);
 		snprintf(item_buf, sizeof(item_buf), "%c   %-*s%s%s", index2mark(mn),
-				(int)(max_len + overhead), with_tilde, file, suffix);
+				(int)(max_len + overhead), display_path, file, suffix);
+		free(display_path);
 
 		i = add_to_string_array(&m.items, i, 1, item_buf);
 	}
 	m.len = i;
 
-	return display_menu(m.state, view);
+	return menus_enter(m.state, view);
 }
 
 /* Callback that is called when menu item is selected.  Should return non-zero
  * to stay in menu mode. */
 static int
-execute_mark_cb(FileView *view, menu_data_t *m)
+execute_mark_cb(view_t *view, menu_data_t *m)
 {
 	goto_mark(view, m->items[m->pos][0]);
 	return 0;
@@ -131,12 +128,12 @@ execute_mark_cb(FileView *view, menu_data_t *m)
 /* Menu-specific shortcut handler.  Returns code that specifies both taken
  * actions and what should be done next. */
 static KHandlerResponse
-mark_khandler(FileView *view, menu_data_t *m, const wchar_t keys[])
+mark_khandler(view_t *view, menu_data_t *m, const wchar_t keys[])
 {
 	if(wcscmp(keys, L"dd") == 0)
 	{
 		clear_mark(m->items[m->pos][0]);
-		remove_current_item(m->state);
+		menus_remove_current(m->state);
 		return KHR_REFRESH_WINDOW;
 	}
 	return KHR_UNHANDLED;
