@@ -19,10 +19,10 @@
 #ifndef VIFM__MENUS__TRASH_MENU_H__
 #define VIFM__MENUS__TRASH_MENU_H__
 
-#include "../ui/ui.h"
+struct view_t;
 
 /* Returns non-zero if status bar message should be saved. */
-int show_trash_menu(FileView *view);
+int show_trash_menu(struct view_t *view);
 
 #endif /* VIFM__MENUS__TRASH_MENU_H__ */
 
