@@ -20,10 +20,10 @@
 #ifndef VIFM__MENUS__USERS_MENU_H__
 #define VIFM__MENUS__USERS_MENU_H__
 
-#include "../ui/ui.h"
+struct view_t;
 
 /* Returns non-zero if status bar message should be saved. */
-int show_user_menu(FileView *view, const char command[], int navigate);
+int show_user_menu(struct view_t *view, const char command[], int navigate);
 
 #endif /* VIFM__MENUS__USERS_MENU_H__ */
 
