@@ -40,12 +40,12 @@
 #include "../engine/keys.h"
 #include "../engine/mode.h"
 #include "../modes/dialogs/msg_dialog.h"
-#include "../ui/color_manager.h"
 #include "../ui/color_scheme.h"
 #include "../ui/colors.h"
 #include "../ui/fileview.h"
 #include "../ui/statusbar.h"
 #include "../ui/statusline.h"
+#include "../ui/quickview.h"
 #include "../ui/ui.h"
 #include "../utils/hist.h"
 #include "../utils/macros.h"
@@ -260,11 +260,11 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_C_u,             {{&cmd_ctrl_u}, .descr = "remove line part to the left"}},
 	{WK_C_w,             {{&cmd_ctrl_w}, .descr = "remove word to the left"}},
 	{WK_C_x WK_SLASH,    {{&cmd_ctrl_xslash}, .descr = "insert last search pattern"}},
-	{WK_C_x WK_a,        {{&cmd_ctrl_xa}, .descr = "insert automatic filter value"}},
+	{WK_C_x WK_a,        {{&cmd_ctrl_xa}, .descr = "insert implicit permanent filter value"}},
 	{WK_C_x WK_c,        {{&cmd_ctrl_xc}, .descr = "insert current file name"}},
 	{WK_C_x WK_d,        {{&cmd_ctrl_xd}, .descr = "insert current directory path"}},
 	{WK_C_x WK_e,        {{&cmd_ctrl_xe}, .descr = "insert current file extension"}},
-	{WK_C_x WK_m,        {{&cmd_ctrl_xm}, .descr = "insert manual filter value"}},
+	{WK_C_x WK_m,        {{&cmd_ctrl_xm}, .descr = "insert explicit permanent filter value"}},
 	{WK_C_x WK_r,        {{&cmd_ctrl_xr}, .descr = "insert root of current file name"}},
 	{WK_C_x WK_t,        {{&cmd_ctrl_xt}, .descr = "insert name of current directory"}},
 	{WK_C_x WK_C_x WK_c, {{&cmd_ctrl_xxc}, .descr = "insert other file name"}},
@@ -379,16 +379,16 @@ update_cmdline_text(line_stats_t *stat)
 static void
 draw_cmdline_text(line_stats_t *stat)
 {
-	if(curr_stats.silent_ui)
+	if(stats_silenced_ui())
 	{
 		/* XXX: erasing status_bar causes blinking if <silent> mapping that uses
 		 *      command-line is pressed and held down.  Ideally wouldn't need this
-		 *      retur. */
+		 *      return. */
 		return;
 	}
 
 	int pair = -1;
-	int attr = 0;
+	col_attr_t prompt_col = {};
 
 	werase(status_bar);
 
@@ -397,21 +397,18 @@ draw_cmdline_text(line_stats_t *stat)
 		case PS_NORMAL:        pair = CMD_LINE_COLOR; break;
 		case PS_WRONG_PATTERN: pair = ERROR_MSG_COLOR; break;
 		case PS_NO_MATCH:      pair = CMD_LINE_COLOR;
-		                       attr = A_REVERSE;
+		                       prompt_col.attr = A_REVERSE;
 		                       break;
 	}
-	attr ^= cfg.cs.color[pair].attr;
+	prompt_col.attr ^= cfg.cs.color[pair].attr;
 
-	wattron(status_bar, COLOR_PAIR(cfg.cs.pair[pair]) | attr);
+	ui_set_attr(status_bar, &prompt_col, cfg.cs.pair[pair]);
 	compat_mvwaddwstr(status_bar, 0, 0, stat->prompt);
-	wattroff(status_bar, COLOR_PAIR(cfg.cs.pair[pair]) | attr);
 
-	attr = cfg.cs.color[CMD_LINE_COLOR].attr;
-
-	wattron(status_bar, COLOR_PAIR(cfg.cs.pair[CMD_LINE_COLOR]) | attr);
+	ui_set_attr(status_bar, &cfg.cs.color[CMD_LINE_COLOR],
+			cfg.cs.pair[CMD_LINE_COLOR]);
 	compat_mvwaddwstr(status_bar, stat->prompt_wid/line_width,
 			stat->prompt_wid%line_width, stat->line);
-	wattroff(status_bar, COLOR_PAIR(cfg.cs.pair[CMD_LINE_COLOR]) | attr);
 
 	update_cursor();
 	ui_refresh_win(status_bar);
@@ -454,6 +451,7 @@ input_line_changed(void)
 	if(prev_mode != MENU_MODE && prev_mode != VISUAL_MODE)
 	{
 		fpos_set_pos(curr_view, curr_view->list_pos);
+		qv_ui_updated();
 		redraw_current_view();
 	}
 	else if(prev_mode == MENU_MODE)
@@ -811,7 +809,6 @@ static void
 leave_cmdline_mode(void)
 {
 	const int multiline_status_bar = (getmaxy(status_bar) > 1);
-	int attr;
 
 	wresize(status_bar, 1, getmaxx(stdscr) - FIELDS_WIDTH());
 	if(multiline_status_bar)
@@ -850,9 +847,6 @@ leave_cmdline_mode(void)
 		ui_ruler_update(curr_view, 1);
 	}
 
-	attr = cfg.cs.color[CMD_LINE_COLOR].attr;
-	wattroff(status_bar, COLOR_PAIR(cfg.cs.pair[CMD_LINE_COLOR]) | attr);
-
 	if(prev_mode != MENU_MODE && prev_mode != VIEW_MODE)
 	{
 		redraw_current_view();
@@ -1180,15 +1174,13 @@ draw_wild_bar(int *last_pos, int *pos, int *len)
 		{
 			col_attr_t col = cfg.cs.color[STATUS_LINE_COLOR];
 			cs_mix_colors(&col, &cfg.cs.color[WILD_MENU_COLOR]);
-
-			wbkgdset(stat_win,
-					COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) | col.attr);
+			ui_set_attr(stat_win, &col, -1);
 		}
 		wprint(stat_win, items[i].text);
 		if(i == *pos)
 		{
-			wbkgdset(stat_win, COLOR_PAIR(cfg.cs.pair[STATUS_LINE_COLOR]) |
-					cfg.cs.color[STATUS_LINE_COLOR].attr);
+			ui_set_attr(stat_win, &cfg.cs.color[STATUS_LINE_COLOR],
+					cfg.cs.pair[STATUS_LINE_COLOR]);
 			*pos = -*pos;
 		}
 	}
@@ -1233,9 +1225,7 @@ draw_wild_popup(int *last_pos, int *pos, int *len)
 		{
 			col_attr_t col = cfg.cs.color[STATUS_LINE_COLOR];
 			cs_mix_colors(&col, &cfg.cs.color[WILD_MENU_COLOR]);
-
-			wbkgdset(stat_win,
-					COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) | col.attr);
+			ui_set_attr(stat_win, &col, -1);
 		}
 
 		checked_wmove(stat_win, j, 0);
@@ -1244,8 +1234,8 @@ draw_wild_popup(int *last_pos, int *pos, int *len)
 
 		if(i == *pos)
 		{
-			wbkgdset(stat_win, COLOR_PAIR(cfg.cs.pair[STATUS_LINE_COLOR]) |
-					cfg.cs.color[STATUS_LINE_COLOR].attr);
+			ui_set_attr(stat_win, &cfg.cs.color[STATUS_LINE_COLOR],
+					cfg.cs.pair[STATUS_LINE_COLOR]);
 			*pos = -*pos;
 		}
 	}
@@ -2426,19 +2416,18 @@ get_required_height(void)
 	return DIV_ROUND_UP(input_stat.prompt_wid + input_stat.len + 1, line_width);
 }
 
-/*
- * p - begin of part that being completed
- * completed - new part of command line
- */
+/* Replaces [ prefix_len : stat->index ) range of stat->line with wide version
+ * of the completed parameter.  Returns zero on success, otherwise non-zero is
+ * returned. */
 static int
-line_part_complete(line_stats_t *stat, const char *line_mb, const char *p,
-		const char *completed)
+line_part_complete(line_stats_t *stat, size_t prefix_len,
+		const char completed[])
 {
 	void *t;
 	wchar_t *line_ending;
 	wchar_t *wide_completed;
 
-	const size_t new_len = (p - line_mb) + wide_len(completed) +
+	const size_t new_len = prefix_len + wide_len(completed) +
 		(stat->len - stat->index) + 1;
 
 	line_ending = vifm_wcsdup(stat->line + stat->index);
@@ -2456,7 +2445,7 @@ line_part_complete(line_stats_t *stat, const char *line_mb, const char *p,
 	stat->line = t;
 
 	wide_completed = to_wide(completed);
-	vifm_swprintf(stat->line + (p - line_mb), new_len,
+	vifm_swprintf(stat->line + prefix_len, new_len,
 			L"%" WPRINTF_WSTR L"%" WPRINTF_WSTR, wide_completed, line_ending);
 	free(wide_completed);
 	free(line_ending);
@@ -2503,9 +2492,7 @@ update_cmdline_size(void)
 TSTATIC int
 line_completion(line_stats_t *stat)
 {
-	static int offset;
-	static char *line_mb;
-	static const char *line_mb_cmd;
+	static size_t prefix_len;
 
 	char *completion;
 	int result;
@@ -2520,16 +2507,14 @@ line_completion(line_stats_t *stat)
 		t = stat->line[stat->index];
 		stat->line[stat->index] = L'\0';
 
-		free(line_mb);
-		line_mb = to_multibyte(stat->line);
+		char *const line_mb = to_multibyte(stat->line);
+		stat->line[stat->index] = t;
 		if(line_mb == NULL)
 		{
 			return -1;
 		}
 
-		line_mb_cmd = find_last_command(line_mb);
-
-		stat->line[stat->index] = t;
+		const char *const line_mb_cmd = find_last_command(line_mb);
 
 		vle_compl_reset();
 
@@ -2559,7 +2544,10 @@ line_completion(line_stats_t *stat)
 			}
 		}
 
-		offset = stat->complete(line_mb_cmd, (void *)compl_func_arg);
+		const int offset = stat->complete(line_mb_cmd, (void *)compl_func_arg);
+		line_mb[line_mb_cmd + offset - line_mb] = '\0';
+		prefix_len = wide_len(line_mb);
+		free(line_mb);
 
 		vle_compl_set_add_path_hook(NULL);
 	}
@@ -2570,7 +2558,7 @@ line_completion(line_stats_t *stat)
 		return 0;
 
 	completion = vle_compl_next();
-	result = line_part_complete(stat, line_mb, line_mb_cmd + offset, completion);
+	result = line_part_complete(stat, prefix_len, completion);
 	free(completion);
 
 	if(vle_compl_get_count() >= 2)
