@@ -140,6 +140,7 @@ redraw_file_info_dialog(void)
 		return;
 	}
 
+	ui_set_attr(menu_win, &cfg.cs.color[WIN_COLOR], cfg.cs.pair[WIN_COLOR]);
 	werase(menu_win);
 
 	curr = get_current_entry(view);
@@ -361,7 +362,7 @@ show_mime_type(view_t *view, int curr_y)
 	const char *mimetype = NULL;
 
 	get_current_full_path(view, sizeof(full_path), full_path);
-	mimetype = get_mimetype(full_path);
+	mimetype = get_mimetype(full_path, 0);
 
 	mvwaddstr(menu_win, curr_y, 2, "Mime Type: ");
 
