@@ -43,6 +43,7 @@
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
+#include "../utils/test_helpers.h"
 #include "../utils/utils.h"
 #include "../cmd_core.h"
 #include "../cmd_completion.h"
@@ -71,7 +72,7 @@ static void cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_e(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_f(key_info_t key_info, keys_info_t *keys_info);
 static int can_scroll_menu_down(const menu_data_t *menu);
-static void change_menu_top(menu_data_t *const menu, int delta);
+static void change_menu_top(menu_data_t *menu, int delta);
 static void cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_return(key_info_t key_info, keys_info_t *keys_info);
 static void update_ui_on_leaving(void);
@@ -105,7 +106,7 @@ static void cmd_zh(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zl(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zt(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zz(key_info_t key_info, keys_info_t *keys_info);
-static int all_lines_visible(const menu_data_t *const menu);
+static int all_lines_visible(const menu_data_t *menu);
 static int goto_cmd(const cmd_info_t *cmd_info);
 static int nohlsearch_cmd(const cmd_info_t *cmd_info);
 static int quit_cmd(const cmd_info_t *cmd_info);
@@ -270,8 +271,8 @@ menu_init_mode(void)
 	/* Double initialization can happen in tests. */
 	if(cmds_conf.inner == NULL)
 	{
-		init_cmds(0, &cmds_conf);
-		add_builtin_commands((const cmd_add_t *)&commands, ARRAY_LEN(commands));
+		vle_cmds_init(0, &cmds_conf);
+		vle_cmds_add(commands, ARRAY_LEN(commands));
 	}
 }
 
@@ -292,7 +293,7 @@ key_handler(wchar_t key)
 void
 menu_enter_mode(menu_data_t *m, view_t *active_view)
 {
-	if(curr_stats.load_stage < 2)
+	if(curr_stats.load_stage >= 0 && curr_stats.load_stage < 2)
 	{
 		return;
 	}
@@ -307,7 +308,7 @@ menu_enter_mode(menu_data_t *m, view_t *active_view)
 	curr_stats.need_update = UT_FULL;
 	was_redraw = 0;
 
-	init_cmds(0, &cmds_conf);
+	vle_cmds_init(0, &cmds_conf);
 }
 
 void
@@ -325,7 +326,7 @@ void
 menu_pre(void)
 {
 	touchwin(ruler_win);
-	wrefresh(ruler_win);
+	ui_refresh_win(ruler_win);
 }
 
 void
@@ -436,7 +437,7 @@ can_scroll_menu_down(const menu_data_t *menu)
 
 /* Moves top line of the menu ensuring that its value is correct. */
 static void
-change_menu_top(menu_data_t *const menu, int delta)
+change_menu_top(menu_data_t *menu, int delta)
 {
 	menu->top =
 		MAX(MIN(menu->top + delta, menu->len - (getmaxy(menu_win) - 2)), 0);
@@ -560,7 +561,7 @@ cmd_percent(key_info_t key_info, keys_info_t *keys_info)
 	}
 	line = (key_info.count*menu->len)/100;
 	menus_set_pos(menu->state, line);
-	wrefresh(menu_win);
+	ui_refresh_win(menu_win);
 }
 
 static void
@@ -598,7 +599,7 @@ cmd_G(key_info_t key_info, keys_info_t *keys_info)
 
 	menus_erase_current(menu->state);
 	menus_set_pos(menu->state, key_info.count - 1);
-	wrefresh(menu_win);
+	ui_refresh_win(menu_win);
 }
 
 static void
@@ -615,7 +616,7 @@ cmd_H(key_info_t key_info, keys_info_t *keys_info)
 
 	menus_erase_current(menu->state);
 	menus_set_pos(menu->state, top);
-	wrefresh(menu_win);
+	ui_refresh_win(menu_win);
 }
 
 static void
@@ -645,7 +646,7 @@ cmd_L(key_info_t key_info, keys_info_t *keys_info)
 
 	menus_erase_current(menu->state);
 	menus_set_pos(menu->state, top - 3);
-	wrefresh(menu_win);
+	ui_refresh_win(menu_win);
 }
 
 /* Moves cursor to the middle of the window. */
@@ -664,7 +665,7 @@ cmd_M(key_info_t key_info, keys_info_t *keys_info)
 
 	menus_erase_current(menu->state);
 	menus_set_pos(menu->state, MAX(0, new_pos - 1));
-	wrefresh(menu_win);
+	ui_refresh_win(menu_win);
 }
 
 static void
@@ -732,7 +733,7 @@ pass_combination_to_khandler(const wchar_t keys[])
 	switch(handler_response)
 	{
 		case KHR_REFRESH_WINDOW:
-			wrefresh(menu_win);
+			ui_refresh_win(menu_win);
 			return 1;
 		case KHR_CLOSE_MENU:
 			leave_menu_mode(1);
@@ -754,7 +755,7 @@ cmd_gg(key_info_t key_info, keys_info_t *keys_info)
 {
 	menus_erase_current(menu->state);
 	menus_set_pos(menu->state, def_count(key_info.count) - 1);
-	wrefresh(menu_win);
+	ui_refresh_win(menu_win);
 }
 
 static void
@@ -765,7 +766,7 @@ cmd_j(key_info_t key_info, keys_info_t *keys_info)
 		menus_erase_current(menu->state);
 		menu->pos += def_count(key_info.count);
 		menus_set_pos(menu->state, menu->pos);
-		wrefresh(menu_win);
+		ui_refresh_win(menu_win);
 	}
 }
 
@@ -777,7 +778,7 @@ cmd_k(key_info_t key_info, keys_info_t *keys_info)
 		menus_erase_current(menu->state);
 		menu->pos -= def_count(key_info.count);
 		menus_set_pos(menu->state, menu->pos);
-		wrefresh(menu_win);
+		ui_refresh_win(menu_win);
 	}
 }
 
@@ -935,7 +936,7 @@ cmd_zz(key_info_t key_info, keys_info_t *keys_info)
 
 /* Returns non-zero if all menu lines are visible, so no scrolling is needed. */
 static int
-all_lines_visible(const menu_data_t *const menu)
+all_lines_visible(const menu_data_t *menu)
 {
 	return menu->len <= getmaxy(menu_win) - 2;
 }
@@ -945,7 +946,7 @@ menu_partial_redraw(void)
 {
 	menus_partial_redraw(menu->state);
 	menus_set_pos(menu->state, menu->pos);
-	wrefresh(menu_win);
+	ui_refresh_win(menu_win);
 }
 
 static int
@@ -955,7 +956,7 @@ goto_cmd(const cmd_info_t *cmd_info)
 	{
 		menus_erase_current(menu->state);
 		menus_set_pos(menu->state, cmd_info->end);
-		wrefresh(menu_win);
+		ui_refresh_win(menu_win);
 	}
 	return 0;
 }
@@ -1059,7 +1060,14 @@ menu_run_command(const char cmd[])
 	{
 		ui_sb_err("An error occurred while trying to execute command");
 	}
-	init_cmds(0, &cmds_conf);
+	vle_cmds_init(0, &cmds_conf);
+}
+
+/* Provides access to tests to a local static variable. */
+TSTATIC menu_data_t *
+menu_get_current(void)
+{
+	return menu;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
