@@ -34,6 +34,7 @@ enum
 	VIEW_MODE,
 	FILE_INFO_MODE,
 	MSG_MODE,
+	MORE_MODE,
 	MODES_COUNT
 };
 
@@ -62,4 +63,4 @@ void print_selected_msg(void);
 #endif /* VIFM__MODES__MODES_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
