@@ -387,9 +387,9 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_y WK_y,        {{&cmd_yy}, .nim = 1, .descr = "yank files"}},
 	{WK_y,             {{&cmd_y_selector}, FOLLOWED_BY_SELECTOR, .descr = "yank files"}},
 	{WK_v,             {{&cmd_V},  .descr = "go to visual mode"}},
-	{WK_z WK_M,        {{&cmd_zM}, .descr = "apply filename filters only"}},
-	{WK_z WK_O,        {{&cmd_zO}, .descr = "remove filename filters (except local one)"}},
-	{WK_z WK_R,        {{&cmd_zR}, .descr = "filter no files"}},
+	{WK_z WK_M,        {{&cmd_zM}, .descr = "restore all filters"}},
+	{WK_z WK_O,        {{&cmd_zO}, .descr = "reset permanent filter"}},
+	{WK_z WK_R,        {{&cmd_zR}, .descr = "save and reset all filters"}},
 	{WK_z WK_a,        {{&cmd_za}, .descr = "toggle dot files visibility"}},
 	{WK_z WK_d,        {{&cmd_zd}, .descr = "exclude custom view entry"}},
 	{WK_z WK_b,        {{&normal_cmd_zb}, .descr = "push cursor to the bottom"}},
@@ -533,8 +533,8 @@ cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(fpos_can_move_down(curr_view))
 	{
-		curr_view->list_pos = fpos_half_scroll(curr_view, 1);
-		redraw_current_view();
+		ui_view_schedule_redraw(curr_view);
+		fpos_set_pos(curr_view, fpos_half_scroll(curr_view, 1));
 	}
 }
 
@@ -731,8 +731,8 @@ cmd_ctrl_u(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(fpos_can_move_up(curr_view))
 	{
-		curr_view->list_pos = fpos_half_scroll(curr_view, 0);
-		redraw_current_view();
+		ui_view_schedule_redraw(curr_view);
+		fpos_set_pos(curr_view, fpos_half_scroll(curr_view, 0));
 	}
 }
 
