@@ -98,13 +98,13 @@ struct view_info_t
 	int search_repeat;        /* Saved count prefix of search commands. */
 
 	/* The rest of the state. */
-	view_t *view;   /* File view association with the view. */
-	char *filename; /* Full path to the file being viewed. */
-	char *viewer;   /* When non-NULL, specifies custom preview command (no
-	                   implicit %c). */
-	int detached;   /* Whether view mode was detached. */
-	int graphical;  /* Whether viewer presumably displays graphics. */
-	int wrap;       /* Whether lines are wrapped. */
+	view_t *view;    /* File view association with the view. */
+	char *filename;  /* Full path to the file being viewed. */
+	char *viewer;    /* When non-NULL, specifies custom preview command (no
+	                    implicit %c). */
+	int detached;    /* Whether view mode was detached. */
+	ViewerKind kind; /* Kind of preview. */
+	int wrap;        /* Whether lines are wrapped. */
 };
 
 /* View information structure indexes and count. */
@@ -475,6 +475,13 @@ try_redraw_explore_view(const view_t *view)
 void
 view_leave_mode(void)
 {
+	if(vi->kind != VK_TEXTUAL && vi->view->explore_mode)
+	{
+		const char *cmd = qv_get_viewer(vi->filename);
+		cmd = (cmd != NULL) ? ma_get_clear_cmd(cmd) : NULL;
+		qv_cleanup(vi->view, cmd);
+	}
+
 	reset_view_info(vi);
 
 	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
@@ -624,7 +631,7 @@ draw(void)
 	const int searched = (vi->last_search_backward != -1);
 	esc_state state;
 
-	if(vi->graphical)
+	if(vi->kind != VK_TEXTUAL)
 	{
 		const char *cmd = qv_get_viewer(vi->filename);
 		cmd = (cmd != NULL) ? ma_get_clear_cmd(cmd) : NULL;
@@ -632,12 +639,24 @@ draw(void)
 
 		free_string_array(vi->lines, vi->nlines);
 		(void)get_view_data(vi, vi->filename);
-		return;
+
+		if(vi->kind == VK_PASS_THROUGH)
+		{
+			strlist_t list = { .nitems = vi->nlines, .items = vi->lines };
+			ui_pass_through(&list, vi->view->win, ui_qv_left(vi->view),
+					ui_qv_top(vi->view));
+			return;
+		}
+
+		/* Proceed and draw output of the previewer instead of returning, even
+		 * though it's graphical.  This way it's possible to use an external
+		 * previewer that handles both textual and graphical previews. */
 	}
 
 	esc_state_init(&state, &cs->color[WIN_COLOR], COLORS);
 
 	ui_view_erase(vi->view);
+	ui_drop_attr(vi->view->win);
 
 	for(vl = 0, l = vi->line; l < max_l && vl < height; ++l)
 	{
@@ -651,7 +670,7 @@ draw(void)
 			const int vis = l != vi->line
 			             || vl + processed >= vi->linev - vi->widths[vi->line][0];
 			offset += esc_print_line(p + offset, vi->view->win, ui_qv_left(vi->view),
-					ui_qv_top(vi->view) + vl, width, !vis, &state, &printed);
+					ui_qv_top(vi->view) + vl, width, !vis, !vi->wrap, &state, &printed);
 			vl += vis;
 			++processed;
 		}
@@ -1035,14 +1054,17 @@ load_view_data(view_info_t *vi, const char action[], const char file_to_view[],
 			return 1;
 	}
 
-	vi->widths = reallocarray(NULL, vi->nlines, sizeof(*vi->widths));
-	if(vi->widths == NULL)
+	if(vi->nlines != 0)
 	{
-		free_string_array(vi->lines, vi->nlines);
-		vi->lines = NULL;
-		vi->nlines = 0;
-		show_error_msg(action, "Not enough memory");
-		return 1;
+		vi->widths = reallocarray(NULL, vi->nlines, sizeof(*vi->widths));
+		if(vi->widths == NULL)
+		{
+			free_string_array(vi->lines, vi->nlines);
+			vi->lines = NULL;
+			vi->nlines = 0;
+			show_error_msg(action, "Not enough memory");
+			return 1;
+		}
 	}
 
 	return 0;
@@ -1080,13 +1102,22 @@ get_view_data(view_info_t *vi, const char file_to_view[])
 	else
 	{
 		const char *const v = (vi->viewer != NULL) ? vi->viewer : viewer;
-		const int graphical = is_graphical_viewer(v);
+		const ViewerKind kind = ft_viewer_kind(v);
 		view_t *const curr = curr_view;
 		curr_view = curr_stats.preview.on ? curr_view
 		          : (vi->view != NULL) ? vi->view : curr_view;
-		curr_stats.preview_hint = vi->view;
 
-		if(graphical)
+		const preview_area_t parea = {
+			.source = vi->view,
+			.view = vi->view,
+			.x = ui_qv_left(vi->view),
+			.y = ui_qv_top(vi->view),
+			.w = ui_qv_width(vi->view),
+			.h = ui_qv_height(vi->view),
+		};
+		curr_stats.preview_hint = &parea;
+
+		if(kind != VK_TEXTUAL)
 		{
 			/* Wait a bit to let terminal emulator do actual refresh (at least some
 			 * of them need this). */
@@ -1099,7 +1130,7 @@ get_view_data(view_info_t *vi, const char file_to_view[])
 		else
 		{
 			/* Don't add implicit %c to a command with %e macro. */
-			char *const cmd = expand_macros(vi->viewer, NULL, NULL, 1);
+			char *const cmd = ma_expand(vi->viewer, NULL, NULL, 1);
 			fp = read_cmd_output(cmd, 0);
 			free(cmd);
 		}
@@ -1112,10 +1143,7 @@ get_view_data(view_info_t *vi, const char file_to_view[])
 			return 3;
 		}
 
-		if(graphical)
-		{
-			vi->graphical = 1;
-		}
+		vi->kind = kind;
 
 		ui_cancellation_reset();
 		ui_cancellation_enable();
@@ -1125,6 +1153,12 @@ get_view_data(view_info_t *vi, const char file_to_view[])
 
 	fclose(fp);
 
+	if(vi->kind != VK_TEXTUAL && vi->nlines == 0)
+	{
+		/* Exploring absent output gives error, add an empty line to allow empty
+		 * output for graphical previewers. */
+		vi->nlines = add_to_string_array(&vi->lines, vi->nlines, 1, "");
+	}
 	if(vi->lines == NULL || vi->nlines == 0)
 	{
 		return 4;
@@ -1600,7 +1634,7 @@ forward_if_changed(view_info_t *vi)
 		return 0;
 	}
 
-	if(filemon_from_file(vi->filename, &mon) != 0)
+	if(filemon_from_file(vi->filename, FMT_MODIFIED, &mon) != 0)
 	{
 		return 0;
 	}
@@ -1673,12 +1707,16 @@ view_info_alloc(void)
 }
 
 void
-view_info_free(view_info_t *vi)
+view_info_free(view_info_t *info)
 {
-	if(vi != NULL)
+	if(info != NULL)
 	{
-		free_view_info(vi);
-		free(vi);
+		free_view_info(info);
+		free(info);
+		if(info == vi)
+		{
+			vi = NULL;
+		}
 	}
 }
 
