@@ -163,10 +163,10 @@ static void select_up_one(view_t *view, int start_pos);
 static void select_down_one(view_t *view, int start_pos);
 static void apply_selection(int pos);
 static void revert_selection(int pos);
-static void update(void);
 static int find_update(view_t *view, int backward);
 static void goto_pos_force_update(int pos);
 static void goto_pos(int pos);
+static void update_ui(void);
 static int move_pos(int pos);
 
 static view_t *view;
@@ -583,7 +583,7 @@ cmd_O(key_info_t key_info, keys_info_t *keys_info)
 	int t = start_pos;
 	start_pos = view->list_pos;
 	view->list_pos = t;
-	update();
+	update_ui();
 }
 
 static void
@@ -611,9 +611,9 @@ sug_quote(vle_keys_list_cb cb)
 static void
 cmd_dollar(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_last_column(view))
+	if(!fpos_at_last_col(view))
 	{
-		goto_pos(get_end_of_line(view));
+		goto_pos(fpos_line_end(view));
 	}
 }
 
@@ -646,9 +646,9 @@ cmd_comma(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_zero(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_first_column(view))
+	if(!fpos_at_first_col(view))
 	{
-		goto_pos(get_start_of_line(view));
+		goto_pos(fpos_line_start(view));
 	}
 }
 
@@ -712,15 +712,10 @@ cmd_d(key_info_t key_info, keys_info_t *keys_info)
 static void
 delete(key_info_t key_info, int use_trash)
 {
-	int count;
-
 	check_marking(view, 0, NULL);
-	count = flist_count_marked(view);
-
-	if(count != 0 && confirm_deletion(count, use_trash))
+	if(fops_delete(view, def_reg(key_info.reg), use_trash))
 	{
-		const int save_msg = fops_delete(view, def_reg(key_info.reg), use_trash);
-		accept_and_leave(save_msg);
+		accept_and_leave(1);
 	}
 }
 
@@ -854,7 +849,7 @@ restore_previous_selection(void)
 		view->list_pos = t;
 	}
 
-	update();
+	update_ui();
 }
 
 /* Performs correct selection of item under the cursor. */
@@ -1078,7 +1073,7 @@ change_amend_type(AmendType new_amend_type)
 	select_first_one();
 	move_pos(cursor_pos);
 
-	update();
+	update_ui();
 }
 
 /* Yanks files. */
@@ -1177,7 +1172,7 @@ cmd_zf(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_left_paren(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_find_group(view, 0));
+	goto_pos(fpos_find_group(view, 0));
 }
 
 /* Moves cursor to the beginning of the next group of files defined by the
@@ -1185,77 +1180,77 @@ cmd_left_paren(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_right_paren(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_find_group(view, 1));
+	goto_pos(fpos_find_group(view, 1));
 }
 
 /* Go to previous sibling directory entry or do nothing. */
 static void
 cmd_z_k(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_prev_dir_sibling(view));
+	goto_pos(fpos_prev_dir_sibling(view));
 }
 
 /* Go to next sibling directory entry or do nothing. */
 static void
 cmd_z_j(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_next_dir_sibling(view));
+	goto_pos(fpos_next_dir_sibling(view));
 }
 
 /* Go to previous mismatched entry or do nothing. */
 static void
 cmd_lb_c(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_prev_mismatch(view));
+	goto_pos(fpos_prev_mismatch(view));
 }
 
 /* Go to next mismatched entry or do nothing. */
 static void
 cmd_rb_c(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_next_mismatch(view));
+	goto_pos(fpos_next_mismatch(view));
 }
 
 /* Go to previous directory entry or do nothing. */
 static void
 cmd_lb_d(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_prev_dir(view));
+	goto_pos(fpos_prev_dir(view));
 }
 
 /* Go to next directory entry or do nothing. */
 static void
 cmd_rb_d(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_next_dir(view));
+	goto_pos(fpos_next_dir(view));
 }
 
 /* Go to previous selected entry or do nothing. */
 static void
 cmd_lb_s(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_prev_selected(view));
+	goto_pos(fpos_prev_selected(view));
 }
 
 /* Go to next selected entry or do nothing. */
 static void
 cmd_rb_s(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_next_selected(view));
+	goto_pos(fpos_next_selected(view));
 }
 
 /* Go to first sibling of the current entry. */
 static void
 cmd_lb_z(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_first_sibling(view));
+	goto_pos(fpos_first_sibling(view));
 }
 
 /* Go to last sibling of the current entry. */
 static void
 cmd_rb_z(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_last_sibling(view));
+	goto_pos(fpos_last_sibling(view));
 }
 
 /* Moves cursor to the beginning of the previous group of files defined by them
@@ -1263,7 +1258,7 @@ cmd_rb_z(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_left_curly_bracket(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_find_dir_group(view, 0));
+	goto_pos(fpos_find_dir_group(view, 0));
 }
 
 /* Moves cursor to the beginning of the next group of files defined by them
@@ -1271,14 +1266,14 @@ cmd_left_curly_bracket(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_right_curly_bracket(key_info_t key_info, keys_info_t *keys_info)
 {
-	goto_pos(flist_find_dir_group(view, 1));
+	goto_pos(fpos_find_dir_group(view, 1));
 }
 
 /* Navigates to next/previous file which starts with given character. */
 static void
 find_goto(int ch, int count, int backward)
 {
-	int pos = flist_find_by_ch(view, ch, backward, 1);
+	int pos = fpos_find_by_ch(view, ch, backward, 1);
 	if(pos < 0 || pos == view->list_pos)
 	{
 		return;
@@ -1287,7 +1282,7 @@ find_goto(int ch, int count, int backward)
 	while(count-- > 0)
 	{
 		goto_pos(pos);
-		pos = flist_find_by_ch(view, ch, backward, 1);
+		pos = fpos_find_by_ch(view, ch, backward, 1);
 	}
 }
 
@@ -1425,13 +1420,6 @@ revert_selection(int pos)
 	}
 }
 
-static void
-update(void)
-{
-	redraw_view(view);
-	ui_ruler_update(view, 1);
-}
-
 void
 update_visual_mode(void)
 {
@@ -1444,7 +1432,7 @@ update_visual_mode(void)
 	view->list_pos = start_pos;
 	goto_pos(pos);
 
-	update();
+	update_ui();
 }
 
 int
@@ -1491,7 +1479,7 @@ static void
 goto_pos_force_update(int pos)
 {
 	(void)move_pos(pos);
-	update();
+	update_ui();
 }
 
 /* Moves cursor from its current position to specified pos selecting or
@@ -1501,10 +1489,20 @@ goto_pos(int pos)
 {
 	if(move_pos(pos))
 	{
-		update();
+		update_ui();
 	}
 }
 
+/* Updates elements of the screen after visual-mode specific elements were
+ * changed. */
+static void
+update_ui(void)
+{
+	fpos_set_pos(view, view->list_pos);
+	redraw_view(view);
+	ui_ruler_update(view, 1);
+}
+
 /* Moves cursor from its current position to specified pos selecting or
  * unselecting files while moving.  Don't call it explicitly, call goto_pos()
  * and goto_pos_force_update() instead.  Returns non-zero if cursor was
