@@ -500,6 +500,11 @@ op_cp(ops_t *ops, void *data, const char src[], const char dst[],
 #endif
 	}
 
+	if(conflict_action == CA_OVERWRITE)
+	{
+		ops->crp = CRP_OVERWRITE_ALL;
+	}
+
 	io_args_t args = {
 		.arg1.src = src,
 		.arg2.dst = dst,
