@@ -18,7 +18,7 @@
 
 #include "opt_handlers.h"
 
-#include <curses.h> /* stdscr wnoutrefresh() */
+#include <curses.h> /* stdscr */
 #include <regex.h> /* regex_t regcomp() regexec() regfree() */
 
 #include <assert.h> /* assert() */
@@ -34,7 +34,6 @@
 #include "engine/options.h"
 #include "engine/text_buffer.h"
 #include "int/term_title.h"
-#include "modes/view.h"
 #include "ui/fileview.h"
 #include "ui/quickview.h"
 #include "ui/statusbar.h"
@@ -155,6 +154,7 @@ static void runexec_handler(OPT_OP op, optval_t val);
 static void scrollbind_handler(OPT_OP op, optval_t val);
 static void scrolloff_handler(OPT_OP op, optval_t val);
 static void shell_handler(OPT_OP op, optval_t val);
+static void shellcmdflag_handler(OPT_OP op, optval_t val);
 static void shortmess_handler(OPT_OP op, optval_t val);
 static void showtabline_handler(OPT_OP op, optval_t val);
 static void sizefmt_handler(OPT_OP op, optval_t val);
@@ -174,8 +174,9 @@ static void lsview_global(OPT_OP op, optval_t val);
 static void lsview_local(OPT_OP op, optval_t val);
 static void milleroptions_global(OPT_OP op, optval_t val);
 static void milleroptions_local(OPT_OP op, optval_t val);
-static void set_milleroptions(int ratios[3], optval_t val, OPT_SCOPE scope);
-static void fill_milleroptions(optval_t *val, int ratios[3]);
+static void set_milleroptions(int ratios[3], int *preview_files, optval_t val,
+		OPT_SCOPE scope);
+static void fill_milleroptions(optval_t *val, int ratios[3], int preview_files);
 static void millerview_global(OPT_OP op, optval_t val);
 static void millerview_local(OPT_OP op, optval_t val);
 static void number_global(OPT_OP op, optval_t val);
@@ -252,6 +253,7 @@ static const char *sort_enum[] = {
 	[SK_BY_GROUPS]        = "groups",
 	[SK_BY_TARGET]        = "target",
 	[SK_BY_ROOT]          = "root",
+	[SK_BY_FILEROOT]      = "fileroot",
 #ifndef _WIN32
 	[SK_BY_GROUP_ID]      = "gid",
 	[SK_BY_GROUP_NAME]    = "gname",
@@ -348,10 +350,11 @@ static const char *iooptions_vals[][2] = {
 
 /* Possible flags of 'shortmess' and their count. */
 static const char *shortmess_vals[][2] = {
-	{ "MTp", "all shortmess values" },
-	{ "M",   "use only file name for title in terminal multiplexers" },
-	{ "T",   "shorten too long status bar messages" },
-	{ "p",   "substitute home path with ~ in view title" },
+	{ "LMTp", "all shortmess values" },
+	{ "L",    "display only last directory in tab line instead of full path" },
+	{ "M",    "use only file name for title in terminal multiplexers" },
+	{ "T",    "shorten too long status bar messages" },
+	{ "p",    "substitute home path with ~ in view title" },
 };
 
 /* Possible values of 'showtabline'. */
@@ -391,15 +394,18 @@ static const char *fillchars_enum[][2] = {
 
 /* Possible keys of 'milleroptions' option. */
 static const char *milleroptions_enum[][2] = {
-	{ "lsize:", "proportion of space given to the left column" },
-	{ "csize:", "proportion of space given to the center column" },
-	{ "rsize:", "proportion of space given to the right column" },
+	{ "lsize:",    "proportion of space given to the left column" },
+	{ "csize:",    "proportion of space given to the center column" },
+	{ "rsize:",    "proportion of space given to the right column" },
+	{ "rpreview:", "what should right pane preview: dirs or all" },
 };
 
 /* Possible keys of 'sizefmt' option. */
 static const char *sizefmt_enum[][2] = {
 	{ "units:",     "iec (1024) or si (1000) unit size" },
 	{ "precision:", "maximum width of fraction part" },
+	{ "space",      "show separator space" },
+	{ "nospace",    "hide separator space" },
 };
 
 /* Possible values of 'sort' option. */
@@ -712,6 +718,10 @@ options[] = {
 	  OPT_STR, 0, NULL, &shell_handler, NULL,
 	  { .ref.str_val = &cfg.shell },
 	},
+	{ "shellcmdflag", "shcf", "argument for the shell",
+	  OPT_STR, 0, NULL, &shellcmdflag_handler, NULL,
+	  { .ref.str_val = &cfg.shell_cmd_flag },
+	},
 	{ "shortmess", "shm", "things to shorten in TUI",
 	  OPT_CHARSET, ARRAY_LEN(shortmess_vals), shortmess_vals, &shortmess_handler,
 		NULL,
@@ -793,11 +803,11 @@ options[] = {
 	  OPT_INT, 0, NULL, &undolevels_handler, NULL,
 	  { .ref.int_val = &cfg.undo_levels },
 	},
-	{ "vicmd", "", "editor to use in console",
+	{ "vicmd", "", "command to invoke editor in console",
 	  OPT_STR, 0, NULL, &vicmd_handler, NULL,
 	  { .ref.str_val = &cfg.vi_command },
 	},
-	{ "vixcmd", "", "editor to use in X",
+	{ "vixcmd", "", "command to invoke editor in X",
 	  OPT_STR, 0, NULL, &vixcmd_handler, NULL,
 	  { .ref.str_val = &cfg.vi_x_command },
 	},
@@ -895,7 +905,7 @@ void
 init_option_handlers(void)
 {
 	static int opt_changed;
-	init_options(&opt_changed, &uni_handler);
+	vle_opts_init(&opt_changed, &uni_handler);
 	load_options_defaults();
 	add_options();
 }
@@ -1112,7 +1122,8 @@ init_lsview(optval_t *val)
 static void
 init_milleroptions(optval_t *val)
 {
-	fill_milleroptions(val, curr_view->miller_ratios_g);
+	fill_milleroptions(val, curr_view->miller_ratios_g,
+			curr_view->miller_preview_files_g);
 }
 
 /* Initializes 'millerview' option from global value. */
@@ -1134,8 +1145,11 @@ static void
 init_shortmess(optval_t *val)
 {
 	static char buf[32];
-	snprintf(buf, sizeof(buf), "%s%s%s", cfg.short_term_mux_titles ? "M" : "",
-			cfg.trunc_normal_sb_msgs ? "T" : "", cfg.shorten_title_paths ? "p" : "");
+	snprintf(buf, sizeof(buf), "%s%s%s%s",
+			cfg.tail_tab_line_paths ? "L" : "",
+			cfg.short_term_mux_titles ? "M" : "",
+			cfg.trunc_normal_sb_msgs ? "T" : "",
+			cfg.shorten_title_paths ? "p" : "");
 	val->str_val = buf;
 }
 
@@ -1286,12 +1300,12 @@ add_options(void)
 	{
 		struct opt_data_t *const opt = &options[i];
 
-		add_option(opt->name, opt->abbr, opt->descr, opt->type, OPT_GLOBAL,
+		vle_opts_add(opt->name, opt->abbr, opt->descr, opt->type, OPT_GLOBAL,
 				opt->val_count, opt->vals, opt->global_handler, opt->val);
 
 		if(opt->local_handler != NULL)
 		{
-			add_option(opt->name, opt->abbr, opt->descr, opt->type, OPT_LOCAL,
+			vle_opts_add(opt->name, opt->abbr, opt->descr, opt->type, OPT_LOCAL,
 					opt->val_count, opt->vals, opt->local_handler, opt->val);
 		}
 	}
@@ -1307,47 +1321,47 @@ reset_local_options(view_t *view)
 
 	view->hide_dot = view->hide_dot_g;
 	val.int_val = !view->hide_dot_g;
-	set_option("dotfiles", val, OPT_LOCAL);
+	vle_opts_assign("dotfiles", val, OPT_LOCAL);
 
 	view->ls_transposed = view->ls_transposed_g;
 	fill_lsoptions(&val, view->ls_transposed_g);
-	set_option("lsoptions", val, OPT_LOCAL);
+	vle_opts_assign("lsoptions", val, OPT_LOCAL);
 
 	fview_set_lsview(view, view->ls_view_g);
 	val.int_val = view->ls_view_g;
-	set_option("lsview", val, OPT_LOCAL);
+	vle_opts_assign("lsview", val, OPT_LOCAL);
 
 	memcpy(view->miller_ratios, view->miller_ratios_g,
 			sizeof(view->miller_ratios));
-	fill_milleroptions(&val, view->miller_ratios_g);
-	set_option("milleroptions", val, OPT_LOCAL);
+	fill_milleroptions(&val, view->miller_ratios_g, view->miller_preview_files_g);
+	vle_opts_assign("milleroptions", val, OPT_LOCAL);
 
 	fview_set_millerview(view, view->miller_view_g);
 	val.int_val = view->miller_view_g;
-	set_option("millerview", val, OPT_LOCAL);
+	vle_opts_assign("millerview", val, OPT_LOCAL);
 
 	view->num_type = view->num_type_g;
 	val.bool_val = view->num_type_g & NT_SEQ;
-	set_option("number", val, OPT_LOCAL);
+	vle_opts_assign("number", val, OPT_LOCAL);
 	val.bool_val = view->num_type_g & NT_REL;
-	set_option("relativenumber", val, OPT_LOCAL);
+	vle_opts_assign("relativenumber", val, OPT_LOCAL);
 
 	view->num_width = view->num_width_g;
 	val.int_val = view->num_width_g;
-	set_option("numberwidth", val, OPT_LOCAL);
+	vle_opts_assign("numberwidth", val, OPT_LOCAL);
 
 	replace_string(&view->view_columns, view->view_columns_g);
 	set_viewcolumns(view, view->view_columns);
 	val.str_val = view->view_columns;
-	set_option("viewcolumns", val, OPT_LOCAL);
+	vle_opts_assign("viewcolumns", val, OPT_LOCAL);
 
 	replace_string(&view->sort_groups, view->sort_groups_g);
 	val.str_val = view->sort_groups;
-	set_option("sortgroups", val, OPT_LOCAL);
+	vle_opts_assign("sortgroups", val, OPT_LOCAL);
 
 	replace_string(&view->preview_prg, view->preview_prg_g);
 	val.str_val = view->preview_prg;
-	set_option("previewprg", val, OPT_LOCAL);
+	vle_opts_assign("previewprg", val, OPT_LOCAL);
 }
 
 void
@@ -1358,59 +1372,59 @@ load_view_options(view_t *view)
 	load_sort_option(view);
 
 	val.str_val = view->view_columns;
-	set_option("viewcolumns", val, OPT_LOCAL);
+	vle_opts_assign("viewcolumns", val, OPT_LOCAL);
 	val.str_val = view->view_columns_g;
-	set_option("viewcolumns", val, OPT_GLOBAL);
+	vle_opts_assign("viewcolumns", val, OPT_GLOBAL);
 
 	val.str_val = view->sort_groups;
-	set_option("sortgroups", val, OPT_LOCAL);
+	vle_opts_assign("sortgroups", val, OPT_LOCAL);
 	val.str_val = view->sort_groups_g;
-	set_option("sortgroups", val, OPT_GLOBAL);
+	vle_opts_assign("sortgroups", val, OPT_GLOBAL);
 
 	val.bool_val = !view->hide_dot;
-	set_option("dotfiles", val, OPT_LOCAL);
+	vle_opts_assign("dotfiles", val, OPT_LOCAL);
 	val.bool_val = !view->hide_dot_g;
-	set_option("dotfiles", val, OPT_GLOBAL);
+	vle_opts_assign("dotfiles", val, OPT_GLOBAL);
 
 	fill_lsoptions(&val, view->ls_transposed);
-	set_option("lsoptions", val, OPT_LOCAL);
+	vle_opts_assign("lsoptions", val, OPT_LOCAL);
 	fill_lsoptions(&val, view->ls_transposed_g);
-	set_option("lsoptions", val, OPT_GLOBAL);
+	vle_opts_assign("lsoptions", val, OPT_GLOBAL);
 
 	val.bool_val = view->ls_view;
-	set_option("lsview", val, OPT_LOCAL);
+	vle_opts_assign("lsview", val, OPT_LOCAL);
 	val.bool_val = view->ls_view_g;
-	set_option("lsview", val, OPT_GLOBAL);
+	vle_opts_assign("lsview", val, OPT_GLOBAL);
 
-	fill_milleroptions(&val, view->miller_ratios);
-	set_option("milleroptions", val, OPT_LOCAL);
-	fill_milleroptions(&val, view->miller_ratios_g);
-	set_option("milleroptions", val, OPT_GLOBAL);
+	fill_milleroptions(&val, view->miller_ratios, view->miller_preview_files);
+	vle_opts_assign("milleroptions", val, OPT_LOCAL);
+	fill_milleroptions(&val, view->miller_ratios_g, view->miller_preview_files_g);
+	vle_opts_assign("milleroptions", val, OPT_GLOBAL);
 
 	val.bool_val = view->miller_view;
-	set_option("millerview", val, OPT_LOCAL);
+	vle_opts_assign("millerview", val, OPT_LOCAL);
 	val.bool_val = view->miller_view_g;
-	set_option("millerview", val, OPT_GLOBAL);
+	vle_opts_assign("millerview", val, OPT_GLOBAL);
 
 	val.bool_val = view->num_type & NT_SEQ;
-	set_option("number", val, OPT_LOCAL);
+	vle_opts_assign("number", val, OPT_LOCAL);
 	val.bool_val = view->num_type_g & NT_SEQ;
-	set_option("number", val, OPT_GLOBAL);
+	vle_opts_assign("number", val, OPT_GLOBAL);
 
 	val.int_val = view->num_width;
-	set_option("numberwidth", val, OPT_LOCAL);
+	vle_opts_assign("numberwidth", val, OPT_LOCAL);
 	val.int_val = view->num_width_g;
-	set_option("numberwidth", val, OPT_GLOBAL);
+	vle_opts_assign("numberwidth", val, OPT_GLOBAL);
 
 	val.bool_val = view->num_type & NT_REL;
-	set_option("relativenumber", val, OPT_LOCAL);
+	vle_opts_assign("relativenumber", val, OPT_LOCAL);
 	val.bool_val = view->num_type_g & NT_REL;
-	set_option("relativenumber", val, OPT_GLOBAL);
+	vle_opts_assign("relativenumber", val, OPT_GLOBAL);
 
 	val.str_val = view->preview_prg;
-	set_option("previewprg", val, OPT_LOCAL);
+	vle_opts_assign("previewprg", val, OPT_LOCAL);
 	val.str_val = view->preview_prg_g;
-	set_option("previewprg", val, OPT_GLOBAL);
+	vle_opts_assign("previewprg", val, OPT_GLOBAL);
 }
 
 void
@@ -1490,7 +1504,7 @@ load_sort_option_inner(view_t *view, char sort_keys[])
 	if(sort_keys == view->custom.sort)
 	{
 		val.str_val = "";
-		set_option("sort", val, OPT_LOCAL);
+		vle_opts_assign("sort", val, OPT_LOCAL);
 		return;
 	}
 
@@ -1513,10 +1527,10 @@ load_sort_option_inner(view_t *view, char sort_keys[])
 	}
 
 	val.str_val = opt_val;
-	set_option("sort", val, scope);
+	vle_opts_assign("sort", val, scope);
 
 	val.enum_item = (sort_keys[0] < 0);
-	set_option("sortorder", val, scope);
+	vle_opts_assign("sortorder", val, scope);
 }
 
 int
@@ -1530,7 +1544,7 @@ process_set_args(const char args[], int global, int local)
 
 	vle_tb_clear(vle_err);
 
-	/* Call of set_options() can change error. */
+	/* Call of vle_opts_set() can change error. */
 	error = 0;
 	if(local && global)
 	{
@@ -1540,7 +1554,7 @@ process_set_args(const char args[], int global, int local)
 	{
 		scope = local ? OPT_LOCAL : OPT_GLOBAL;
 	}
-	set_options_error = (set_options(args, scope) != 0);
+	set_options_error = (vle_opts_set(args, scope) != 0);
 	error |= set_options_error;
 	text_buffer = vle_tb_get_data(vle_err);
 
@@ -1618,7 +1632,7 @@ caseoptions_handler(OPT_OP op, optval_t val)
 	}
 	val.str_val = valid_val;
 
-	set_option("caseoptions", val, OPT_GLOBAL);
+	vle_opts_assign("caseoptions", val, OPT_GLOBAL);
 }
 
 /* Specifies directories to check on cding by relative path. */
@@ -1648,20 +1662,11 @@ classify_handler(OPT_OP op, optval_t val)
 
 	if(str_to_classify(val.str_val, type_decs) == 0)
 	{
-		int i;
-
 		assert(sizeof(cfg.type_decs) == sizeof(type_decs) && "Arrays diverged");
 		memcpy(&cfg.type_decs, &type_decs, sizeof(cfg.type_decs));
 
-		/* Reset cached indexes for name-dependent type_decs. */
-		for(i = 0; i < lwin.list_rows; ++i)
-		{
-			lwin.dir_entry[i].name_dec_num = -1;
-		}
-		for(i = 0; i < rwin.list_rows; ++i)
-		{
-			rwin.dir_entry[i].name_dec_num = -1;
-		}
+		ui_view_reset_decor_cache(&lwin);
+		ui_view_reset_decor_cache(&rwin);
 
 		/* 'classify' option affects columns layout, hence views must be reloaded as
 		 * loading list of files performs calculation of filename properties. */
@@ -1674,7 +1679,7 @@ classify_handler(OPT_OP op, optval_t val)
 	}
 
 	init_classify(&val);
-	set_option("classify", val, OPT_GLOBAL);
+	vle_opts_assign("classify", val, OPT_GLOBAL);
 }
 
 /* Fills the decorations array with parsed classification values from the str.
@@ -1872,7 +1877,7 @@ columns_handler(OPT_OP op, optval_t val)
 
 	/* Need to update value of option in case it was corrected above. */
 	val.int_val = cfg.columns;
-	set_option("columns", val, OPT_GLOBAL);
+	vle_opts_assign("columns", val, OPT_GLOBAL);
 }
 
 static void
@@ -2014,7 +2019,7 @@ load_fillchars(void)
 		snprintf(value, sizeof(value), "vborder:%s", cfg.border_filler);
 	}
 
-	set_option("fillchars", val, OPT_GLOBAL);
+	vle_opts_assign("fillchars", val, OPT_GLOBAL);
 }
 
 static void
@@ -2038,7 +2043,7 @@ fusehome_handler(OPT_OP op, optval_t val)
 	{
 		/* Reset the 'fusehome' options to its previous value. */
 		val.str_val = cfg.fuse_home;
-		set_option("fusehome", val, OPT_GLOBAL);
+		vle_opts_assign("fusehome", val, OPT_GLOBAL);
 	}
 	free(expanded_path);
 }
@@ -2154,7 +2159,7 @@ lines_handler(OPT_OP op, optval_t val)
 
 	/* Need to update value of option in case it was corrected above. */
 	val.int_val = cfg.lines;
-	set_option("lines", val, OPT_GLOBAL);
+	vle_opts_assign("lines", val, OPT_GLOBAL);
 }
 
 /* Handles updates of the 'locateprg' option. */
@@ -2184,7 +2189,7 @@ mintimeoutlen_handler(OPT_OP op, optval_t val)
 		vle_tb_append_linef(vle_err, "Argument must be > 0: %d", val.int_val);
 		error = 1;
 		val.int_val = 1;
-		set_option("mintimeoutlen", val, OPT_GLOBAL);
+		vle_opts_assign("mintimeoutlen", val, OPT_GLOBAL);
 		return;
 	}
 
@@ -2238,7 +2243,7 @@ scrolloff_handler(OPT_OP op, optval_t val)
 	{
 		vle_tb_append_linef(vle_err, "Invalid scroll size: %d", val.int_val);
 		error = 1;
-		reset_option_to_default("scrolloff", OPT_GLOBAL);
+		vle_opts_restore_default("scrolloff", OPT_GLOBAL);
 		return;
 	}
 
@@ -2249,37 +2254,42 @@ scrolloff_handler(OPT_OP op, optval_t val)
 	}
 }
 
+/* Handles changes of 'shell' option which specifies shell interpreter
+ * command. */
 static void
 shell_handler(OPT_OP op, optval_t val)
 {
 	cfg_set_shell(val.str_val);
 }
 
+/* Handles changes of 'shellcmdflag' option which specifies shell interpreter
+ * command-line option that enables processing of the next argument as a
+ * command. */
+static void
+shellcmdflag_handler(OPT_OP op, optval_t val)
+{
+	replace_string(&cfg.shell_cmd_flag, val.str_val);
+}
+
 /* Handles changes of 'shortmess' option by expanding flags into actual option
  * values. */
 static void
 shortmess_handler(OPT_OP op, optval_t val)
 {
-	const char *p;
-
+	cfg.tail_tab_line_paths = 0;
 	cfg.trunc_normal_sb_msgs = 0;
 	cfg.shorten_title_paths = 0;
 	cfg.short_term_mux_titles = 0;
 
-	p = val.str_val;
+	const char *p = val.str_val;
 	while(*p != '\0')
 	{
-		if(*p == 'M')
+		switch(*p)
 		{
-			cfg.short_term_mux_titles = 1;
-		}
-		else if(*p == 'T')
-		{
-			cfg.trunc_normal_sb_msgs = 1;
-		}
-		else if(*p == 'p')
-		{
-			cfg.shorten_title_paths = 1;
+			case 'L': cfg.tail_tab_line_paths = 1; break;
+			case 'M': cfg.short_term_mux_titles = 1; break;
+			case 'T': cfg.trunc_normal_sb_msgs = 1; break;
+			case 'p': cfg.shorten_title_paths = 1; break;
 		}
 		++p;
 	}
@@ -2304,7 +2314,7 @@ sizefmt_handler(OPT_OP op, optval_t val)
 	char *const new_val = strdup(val.str_val);
 	char *part = new_val, *state = NULL;
 
-	int base = -1, precision = 0;
+	int base = -1, precision = 0, space = 1;
 
 	while((part = split_and_get(part, ',', &state)) != NULL)
 	{
@@ -2334,6 +2344,14 @@ sizefmt_handler(OPT_OP op, optval_t val)
 				break;
 			}
 		}
+		else if(strcmp(part, "space") == 0)
+		{
+			space = 1;
+		}
+		else if(strcmp(part, "nospace") == 0)
+		{
+			space = 0;
+		}
 		else
 		{
 			break_at(part, ':');
@@ -2348,6 +2366,7 @@ sizefmt_handler(OPT_OP op, optval_t val)
 	{
 		cfg.sizefmt.base = base;
 		cfg.sizefmt.precision = precision;
+		cfg.sizefmt.space = space;
 
 		curr_stats.need_update = UT_REDRAW;
 	}
@@ -2358,7 +2377,7 @@ sizefmt_handler(OPT_OP op, optval_t val)
 
 	/* In case of error, restore previous value, otherwise reload it anyway to
 	 * remove any duplicates. */
-	set_option("sizefmt", make_sizefmt_value(), OPT_GLOBAL);
+	vle_opts_assign("sizefmt", make_sizefmt_value(), OPT_GLOBAL);
 }
 
 /* Makes string value describing 'sizefmt' from current configuration state.
@@ -2369,13 +2388,17 @@ make_sizefmt_value(void)
 	static char value[128];
 	static const optval_t val = { .str_val = value };
 
-	const int len = snprintf(value, sizeof(value), "units:%s",
+	int len = snprintf(value, sizeof(value), "units:%s",
 			cfg.sizefmt.base == 1024 ? "iec" : "si");
 	if(cfg.sizefmt.precision != 0)
 	{
-		snprintf(value + len, sizeof(value) - len, ",precision:%d",
+		len += snprintf(value + len, sizeof(value) - len, ",precision:%d",
 				cfg.sizefmt.precision);
 	}
+	if(cfg.sizefmt.space == 0)
+	{
+		len += snprintf(value + len, sizeof(value) - len, ",nospace");
+	}
 
 	return val;
 }
@@ -2469,7 +2492,7 @@ set_lsoptions(int *transposed, optval_t val, OPT_SCOPE scope)
 	}
 
 	fill_lsoptions(&val, *transposed);
-	set_option("lsoptions", val, scope);
+	vle_opts_assign("lsoptions", val, scope);
 }
 
 /* Loads value of lsoptions as a string. */
@@ -2499,24 +2522,28 @@ lsview_local(OPT_OP op, optval_t val)
 static void
 milleroptions_global(OPT_OP op, optval_t val)
 {
-	set_milleroptions(curr_view->miller_ratios_g, val, OPT_GLOBAL);
+	set_milleroptions(curr_view->miller_ratios_g,
+			&curr_view->miller_preview_files_g, val, OPT_GLOBAL);
 }
 
 /* Handles update of miller columns settings as a local option. */
 static void
 milleroptions_local(OPT_OP op, optval_t val)
 {
-	set_milleroptions(curr_view->miller_ratios, val, OPT_LOCAL);
+	set_milleroptions(curr_view->miller_ratios, &curr_view->miller_preview_files,
+			val, OPT_LOCAL);
 }
 
 /* Handles update of miller columns settings. */
 static void
-set_milleroptions(int ratios[3], optval_t val, OPT_SCOPE scope)
+set_milleroptions(int ratios[3], int *preview_files, optval_t val,
+		OPT_SCOPE scope)
 {
 	char *new_val = strdup(val.str_val);
 	char *part = new_val, *state = NULL;
 
 	int lsize = 0, csize = 1, rsize = 0;
+	int preview_all = 0;
 
 	while((part = split_and_get(part, ',', &state)) != NULL)
 	{
@@ -2556,6 +2583,24 @@ set_milleroptions(int ratios[3], optval_t val, OPT_SCOPE scope)
 				break;
 			}
 		}
+		else if(starts_with_lit(part, "rpreview:"))
+		{
+			const char *const str = after_first(part, ':');
+			if(strcmp(str, "all") == 0)
+			{
+				preview_all = 1;
+			}
+			else if(strcmp(str, "dirs") == 0)
+			{
+				preview_all = 0;
+			}
+			else
+			{
+				vle_tb_append_linef(vle_err, "Failed to parse \"rpreview\" value: %s",
+						str);
+				break;
+			}
+		}
 		else
 		{
 			break_at(part, ':');
@@ -2571,6 +2616,7 @@ set_milleroptions(int ratios[3], optval_t val, OPT_SCOPE scope)
 		ratios[0] = MAX(0, MIN(100, lsize));
 		ratios[1] = MAX(0, MIN(100, csize));
 		ratios[2] = MAX(0, MIN(100, rsize));
+		*preview_files = preview_all;
 
 		if(ratios == curr_view->miller_ratios)
 		{
@@ -2578,17 +2624,18 @@ set_milleroptions(int ratios[3], optval_t val, OPT_SCOPE scope)
 		}
 	}
 
-	fill_milleroptions(&val, ratios);
-	set_option("milleroptions", val, scope);
+	fill_milleroptions(&val, ratios, preview_all);
+	vle_opts_assign("milleroptions", val, scope);
 }
 
 /* Loads value of milleroptions as a string. */
 static void
-fill_milleroptions(optval_t *val, int ratios[3])
+fill_milleroptions(optval_t *val, int ratios[3], int preview_files)
 {
 	static char buf[64];
-	snprintf(buf, sizeof(buf), "lsize:%d,csize:%d,rsize:%d", ratios[0], ratios[1],
-			ratios[2]);
+	snprintf(buf, sizeof(buf), "lsize:%d,csize:%d,rsize:%d,rpreview:%s",
+			ratios[0], ratios[1], ratios[2],
+			(preview_files ? "all" : "dirs"));
 	val->str_val = buf;
 }
 
@@ -2827,7 +2874,7 @@ set_sortgroups(view_t *view, char **opt, char value[])
 
 		free(first);
 		error = 1;
-		set_option("viewcolumns", val, scope);
+		vle_opts_assign("viewcolumns", val, scope);
 		return;
 	}
 
@@ -2915,8 +2962,8 @@ void
 load_dot_filter_option(const view_t *view)
 {
 	const optval_t val = { .bool_val = !view->hide_dot };
-	set_option("dotfiles", val, OPT_GLOBAL);
-	set_option("dotfiles", val, OPT_LOCAL);
+	vle_opts_assign("dotfiles", val, OPT_GLOBAL);
+	vle_opts_assign("dotfiles", val, OPT_LOCAL);
 }
 
 void
@@ -2955,7 +3002,7 @@ set_view_columns_option(view_t *view, const char value[], int update_ui)
 		}
 
 		val.str_val = view->view_columns;
-		set_option("viewcolumns", val, OPT_LOCAL);
+		vle_opts_assign("viewcolumns", val, OPT_LOCAL);
 	}
 	else
 	{
@@ -3005,7 +3052,7 @@ load_quickview_option(void)
 {
 	optval_t val;
 	val.bool_val = curr_stats.preview.on;
-	set_option("quickview", val, OPT_GLOBAL);
+	vle_opts_assign("quickview", val, OPT_GLOBAL);
 }
 
 void
@@ -3013,9 +3060,9 @@ load_geometry(void)
 {
 	optval_t val;
 	val.int_val = cfg.columns;
-	set_option("columns", val, OPT_GLOBAL);
+	vle_opts_assign("columns", val, OPT_GLOBAL);
 	val.int_val = cfg.lines;
-	set_option("lines", val, OPT_GLOBAL);
+	vle_opts_assign("lines", val, OPT_GLOBAL);
 }
 
 static void
@@ -3149,7 +3196,7 @@ reset_suggestoptions(void)
 	}
 
 	val.str_val = (char *)vle_tb_get_data(descr);
-	set_option("suggestoptions", val, OPT_GLOBAL);
+	vle_opts_assign("suggestoptions", val, OPT_GLOBAL);
 
 	vle_tb_free(descr);
 }
@@ -3184,6 +3231,11 @@ syscalls_handler(OPT_OP op, optval_t val)
 static void
 tabscope_handler(OPT_OP op, optval_t val)
 {
+	if(curr_stats.restart_in_progress)
+	{
+		return;
+	}
+
 	/* Erase tabs on current scope before switching it. */
 	tabs_only(curr_view);
 	tabs_rename(curr_view, NULL);
@@ -3199,7 +3251,7 @@ tabstop_handler(OPT_OP op, optval_t val)
 	{
 		vle_tb_append_linef(vle_err, "Argument must be positive: %d", val.int_val);
 		error = 1;
-		reset_option_to_default("tabstop", OPT_GLOBAL);
+		vle_opts_restore_default("tabstop", OPT_GLOBAL);
 		return;
 	}
 
@@ -3225,7 +3277,7 @@ timeoutlen_handler(OPT_OP op, optval_t val)
 		vle_tb_append_linef(vle_err, "Argument must be >= 0: %d", val.int_val);
 		error = 1;
 		val.int_val = 0;
-		set_option("timeoutlen", val, OPT_GLOBAL);
+		vle_opts_assign("timeoutlen", val, OPT_GLOBAL);
 		return;
 	}
 
@@ -3261,7 +3313,7 @@ trashdir_handler(OPT_OP op, optval_t val)
 	{
 		/* Reset the 'trashdir' option to its previous value. */
 		val.str_val = cfg.trash_dir;
-		set_option("trashdir", val, OPT_GLOBAL);
+		vle_opts_assign("trashdir", val, OPT_GLOBAL);
 	}
 	free(expanded_path);
 }
@@ -3467,14 +3519,7 @@ wrap_handler(OPT_OP op, optval_t val)
 static void
 text_option_changed(void)
 {
-	if(curr_stats.preview.on)
-	{
-		qv_draw(curr_view);
-	}
-	else if(other_view->explore_mode)
-	{
-		view_redraw();
-	}
+	stats_redraw_schedule();
 }
 
 static void
