@@ -19,26 +19,6 @@
 #ifndef VIFM__OPT_HANDLERS_H__
 #define VIFM__OPT_HANDLERS_H__
 
-enum
-{
-	VIFMINFO_OPTIONS   = 1 << 0,
-	VIFMINFO_FILETYPES = 1 << 1,
-	VIFMINFO_COMMANDS  = 1 << 2,
-	VIFMINFO_MARKS     = 1 << 3,
-	VIFMINFO_BOOKMARKS = 1 << 4,
-	VIFMINFO_TUI       = 1 << 5,
-	VIFMINFO_DHISTORY  = 1 << 6,
-	VIFMINFO_STATE     = 1 << 7,
-	VIFMINFO_CS        = 1 << 8,
-	VIFMINFO_SAVEDIRS  = 1 << 9,
-	VIFMINFO_CHISTORY  = 1 << 10,
-	VIFMINFO_SHISTORY  = 1 << 11,
-	VIFMINFO_DIRSTACK  = 1 << 12,
-	VIFMINFO_REGISTERS = 1 << 13,
-	VIFMINFO_PHISTORY  = 1 << 14,
-	VIFMINFO_FHISTORY  = 1 << 15,
-};
-
 struct view_t;
 
 void init_option_handlers(void);
@@ -72,6 +52,9 @@ void load_dot_filter_option(const struct view_t *view);
  * sorting order via sort dialog. */
 void load_view_columns_option(struct view_t *view, const char value[]);
 
+/* Loads current state of quick view into corresponding option. */
+void load_quickview_option(void);
+
 /* Updates geometry related options. */
 void load_geometry(void);
 
