@@ -26,6 +26,7 @@
 #include "compat/fs_limits.h"
 #include "ui/color_scheme.h"
 #include "utils/hist.h"
+#include "filetype.h"
 
 /* Special value foe dcache fields meaning that it wasn't set. */
 #define DCACHE_UNKNOWN ((uint64_t)-1)
@@ -109,7 +110,7 @@ typedef struct
 	struct view_info_t *explore; /* State of explored quick view. */
 	unsigned int on : 1;         /* Whether preview mode is active.  Use
 	                                stats_set_quickview() to change the value. */
-	unsigned int graphical : 1;  /* Whether current preview displays graphics. */
+	ViewerKind kind : 2;         /* Kind of the preview. */
 	unsigned int clearing : 1;   /* Whether in process of clearing preview. */
 }
 preview_t;
@@ -144,7 +145,10 @@ typedef struct
 	/* Whether to skip complete UI redraw after returning from a shellout. */
 	int skip_shellout_redraw;
 
+	/* Color scheme to use for :commands that manipulate colors.  Almost always
+	 * refers to cfg.cs, except for when local color scheme is loaded. */
 	col_scheme_t *cs;
+	/* Name of the color scheme loaded from vifminfo. */
 	char color_scheme[NAME_MAX + 1];
 
 	int msg_head, msg_tail;
@@ -191,12 +195,10 @@ typedef struct
 
 	char *on_choose; /* Command to execute on picking files. */
 
-	void *preview_hint; /* Hint on which view is used for preview. */
+	const void *preview_hint; /* Hint on which view is used for preview. */
 
 	int global_local_settings; /* Set local settings globally. */
 
-	int silent_ui; /* Whether UI updates should be "paused". */
-
 	int history_size;   /* Number of elements in histories. */
 	hist_t cmd_hist;    /* History of command-line commands. */
 	hist_t search_hist; /* History of search patterns. */
@@ -266,10 +268,15 @@ void stats_save_msg(const char msg[]);
  * parts of the application. */
 void stats_set_quickview(int on);
 
+/* UI silencing. */
+
 /* Non-zero argument makes UI more silent, zero argument makes it less silent.
  * After calls with non-zero and zero arguments balance out, UI gets updated. */
 void stats_silence_ui(int more);
 
+/* Checks whether UI is currently silenced. */
+int stats_silenced_ui(void);
+
 /* Managing histories. */
 
 /* Changes size of histories stored in status_t.  Zero or negative value
