@@ -23,6 +23,7 @@ const char *tags[] = {
 	"vifm-%m",
 	"vifm-%n",
 	"vifm-%pc",
+	"vifm-%pd",
 	"vifm-%ph",
 	"vifm-%pw",
 	"vifm-%px",
@@ -96,7 +97,9 @@ const char *tags[] = {
 	"vifm-'scrolloff'",
 	"vifm-'scs'",
 	"vifm-'sh'",
+	"vifm-'shcf'",
 	"vifm-'shell'",
+	"vifm-'shellcmdflag'",
 	"vifm-'shm'",
 	"vifm-'shortmess'",
 	"vifm-'showtabline'",
@@ -178,6 +181,7 @@ const char *tags[] = {
 	"vifm-:ca",
 	"vifm-:cabbrev",
 	"vifm-:cd",
+	"vifm-:cds",
 	"vifm-:change",
 	"vifm-:chmod",
 	"vifm-:chown",
@@ -256,6 +260,7 @@ const char *tags[] = {
 	"vifm-:h",
 	"vifm-:help",
 	"vifm-:hi",
+	"vifm-:hideui",
 	"vifm-:highlight",
 	"vifm-:his",
 	"vifm-:histnext",
@@ -446,6 +451,7 @@ const char *tags[] = {
 	"vifm-F",
 	"vifm-FUSE_MOUNT",
 	"vifm-FUSE_MOUNT2",
+	"vifm-FUSE_MOUNT3",
 	"vifm-G",
 	"vifm-H",
 	"vifm-L",
@@ -589,6 +595,7 @@ const char *tags[] = {
 	"vifm-expr6",
 	"vifm-expr7",
 	"vifm-expression-syntax",
+	"vifm-extcached()",
 	"vifm-f",
 	"vifm-filetype()",
 	"vifm-filters",
