@@ -79,6 +79,10 @@ column_info_t;
 /* Registers column print function. */
 void columns_set_line_print_func(column_line_print_func func);
 
+/* Sets string to be used in place of ellipsis.  The argument is used directly,
+ * no copy is made. */
+void columns_set_ellipsis(const char ell[]);
+
 /* Registers column func by its unique column_id.
  * Returns zero on success and non-zero otherwise. */
 int columns_add_column_desc(int column_id, column_func func);
@@ -103,6 +107,10 @@ void columns_clear(columns_t *cols);
 void columns_format_line(columns_t *cols, const void *data,
 		size_t max_line_width);
 
+/* Checks if recalculation is needed.  Returns non-zero if so, otherwise zero is
+ * returned. */
+int columns_matches_width(const columns_t *cols, size_t max_width);
+
 #endif /* VIFM__UI__COLUMN_VIEW_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
