@@ -61,7 +61,8 @@ typedef struct
 {
 	view_t *view;       /* View on which cell is being drawn. */
 	dir_entry_t *entry; /* Entry that is being displayed. */
-	int line_pos;       /* File position in the file list (the view). */
+	int line_pos;       /* File position in the file list (the view).  Can be -1
+	                     * for filler (entry should still be supplied though). */
 	int line_hi_group;  /* Line highlight (to avoid per-column calculation). */
 	int current_pos;    /* Position of entry selected with the cursor. */
 	int total_width;    /* Total width available for drawing. */
@@ -96,7 +97,7 @@ static void draw_cell(columns_t *columns, const column_data_t *cdt,
 static columns_t * get_view_columns(const view_t *view, int truncated);
 static columns_t * get_name_column(int truncated);
 static void consider_scroll_bind(view_t *view);
-static int prepare_inactive_color(view_t *view, dir_entry_t *entry,
+static cchar_t prepare_inactive_color(view_t *view, dir_entry_t *entry,
 		int line_color);
 static void redraw_cell(view_t *view, int top, int cursor, int is_current);
 static void compute_and_draw_cell(column_data_t *cdt, int cell,
@@ -106,8 +107,8 @@ static void column_line_print(const void *data, int column_id, const char buf[],
 static void draw_line_number(const column_data_t *cdt, int column);
 static void highlight_search(view_t *view, dir_entry_t *entry,
 		const char full_column[], char buf[], size_t buf_len, AlignType align,
-		int line, int col, int line_attrs);
-static int prepare_col_color(const view_t *view, int primary,
+		int line, int col, const cchar_t *line_attrs);
+static cchar_t prepare_col_color(const view_t *view, int primary,
 		const column_data_t *cdt);
 static void mix_in_common_colors(col_attr_t *col, const view_t *view,
 		dir_entry_t *entry, int line_color);
@@ -141,12 +142,14 @@ static size_t calculate_columns_count(struct view_t *view);
 static size_t get_max_filename_width(const view_t *view);
 static size_t get_filename_width(const view_t *view, int i);
 static size_t get_filetype_decoration_width(const dir_entry_t *entry);
+static int cache_cursor_pos(view_t *view);
+static void invalidate_cursor_pos_cache(view_t *view);
 static void position_hardware_cursor(view_t *view);
 static int move_curr_line(view_t *view);
 static void reset_view_columns(view_t *view);
 
 void
-fview_init(void)
+fview_setup(void)
 {
 	static const struct {
 		SortingKey key;
@@ -193,10 +196,11 @@ fview_init(void)
 	}
 	columns_add_column_desc(SK_BY_ID, &format_id);
 	columns_add_column_desc(SK_BY_ROOT, &format_name);
+	columns_add_column_desc(SK_BY_FILEROOT, &format_name);
 }
 
 void
-fview_view_init(view_t *view)
+fview_init(view_t *view)
 {
 	view->curr_line = 0;
 	view->top_line = 0;
@@ -217,7 +221,7 @@ fview_view_init(view_t *view)
 }
 
 void
-fview_view_reset(view_t *view)
+fview_reset(view_t *view)
 {
 	view->ls_view_g = view->ls_view = 0;
 	view->ls_transposed_g = view->ls_transposed = 0;
@@ -227,22 +231,23 @@ fview_view_reset(view_t *view)
 	view->run_size = 1;
 
 	view->miller_view_g = view->miller_view = 0;
-	view->miller_ratios[0] = view->miller_ratios_g[0] = 1;
-	view->miller_ratios[1] = view->miller_ratios_g[1] = 1;
-	view->miller_ratios[2] = view->miller_ratios_g[2] = 1;
+	view->miller_ratios_g[0] = view->miller_ratios[0] = 1;
+	view->miller_ratios_g[1] = view->miller_ratios[1] = 1;
+	view->miller_ratios_g[2] = view->miller_ratios[2] = 1;
+	view->miller_preview_files_g = view->miller_preview_files = 0;
 
 	view->num_type_g = view->num_type = NT_NONE;
 	view->num_width_g = view->num_width = 4;
 	view->real_num_width = 0;
 
 	pthread_mutex_lock(view->timestamps_mutex);
-	view->postponed_redraw = 0;
-	view->postponed_reload = 0;
+	view->need_redraw = 0;
+	view->need_reload = 0;
 	pthread_mutex_unlock(view->timestamps_mutex);
 }
 
 void
-fview_view_cs_reset(view_t *view)
+fview_reset_cs(view_t *view)
 {
 	int i;
 	for(i = 0; i < view->list_rows; ++i)
@@ -258,7 +263,7 @@ draw_dir_list(view_t *view)
 
 	if(view != curr_view)
 	{
-		put_inactive_mark(view);
+		fview_draw_inactive_cursor(view);
 	}
 }
 
@@ -363,7 +368,8 @@ draw_left_column(view_t *view)
 static void
 draw_right_column(view_t *view)
 {
-	char path[PATH_MAX + 1];
+	view->displays_graphics = 0;
+
 	const int padding = (cfg.extra_padding ? 1 : 0);
 	const int offset = ui_view_left_reserved(view) + padding
 	                 + ui_view_available_width(view) + padding
@@ -376,6 +382,27 @@ draw_right_column(view_t *view)
 		return;
 	}
 
+	dir_entry_t *const entry = get_current_entry(view);
+	if(view->miller_preview_files && !fentry_is_dir(entry))
+	{
+		const col_scheme_t *const cs = ui_view_get_cs(view);
+		col_attr_t def_col = ui_get_win_color(view, cs);
+		cs_mix_colors(&def_col, &cs->color[AUX_WIN_COLOR]);
+
+		const preview_area_t parea = {
+			.source = view,
+			.view = view,
+			.def_col = def_col,
+			.x = offset,
+			.y = 0,
+			.w = ui_view_right_reserved(view) - 1,
+			.h = view->window_rows,
+		};
+		qv_draw_on(entry, &parea);
+		return;
+	}
+
+	char path[PATH_MAX + 1];
 	get_current_full_path(view, sizeof(path), path);
 	(void)flist_update_cache(view, &view->right_column, path);
 
@@ -465,20 +492,25 @@ print_column(view_t *view, entries_t entries, const char current[],
 static void
 fill_column(view_t *view, int start_line, int top, int width, int offset)
 {
-	int i;
-
 	char filler[width + (cfg.extra_padding ? 1 : 0) + 1];
 	memset(filler, ' ', sizeof(filler) - 1U);
 	filler[sizeof(filler) - 1U] = '\0';
 
+	char a_space[] = " ";
+	dir_entry_t non_entry = {
+		.name = a_space,
+		.origin = a_space,
+		.type = FT_UNK,
+	};
+
+	int i;
 	for(i = start_line; i - top < view->window_rows; ++i)
 	{
 		size_t prefix_len = 0U;
 		const column_data_t cdt = {
 			.view = view,
-			.entry = &view->dir_entry[0],
-			.line_pos = i,
-			.line_hi_group = WIN_COLOR,
+			.entry = &non_entry,
+			.line_pos = -1,
 			.total_width = width,
 			.current_pos = -1,
 			.current_line = i - top,
@@ -770,8 +802,12 @@ redraw_view(view_t *view)
 	if(curr_stats.need_update == UT_NONE && !curr_stats.restart_in_progress &&
 			window_shows_dirlist(view))
 	{
-		/* Make sure cursor is visible and relevant part of the view is displayed. */
+		/* Make sure cursor is visible and relevant part of the view is
+		 * displayed. */
 		(void)move_curr_line(view);
+		/* Update cursor position cache as it might have been moved outside this
+		 * unit. */
+		(void)cache_cursor_pos(view);
 		/* And then redraw the view unconditionally as requested. */
 		draw_dir_list(view);
 	}
@@ -786,29 +822,39 @@ redraw_current_view(void)
 void
 fview_cursor_redraw(view_t *view)
 {
-	if(view == curr_view)
+	/* fview_cursor_redraw() is also called in situations when file list has
+	 * changed, so just let fview_position_updated() deal with it.  With a cache
+	 * of last position, it should be fine. */
+	fview_position_updated(view);
+
+	/* Always redrawing the cell won't hurt and will account for the case when
+	 * selection state of item under the cursor has changed. */
+	if(view == other_view)
 	{
-		/* Call file list function, which will also ensure that current position in
-		 * list is correct. */
-		fpos_set_pos(view, view->list_pos);
+		fview_draw_inactive_cursor(view);
 	}
 	else
 	{
-		if(move_curr_line(view))
+		if(!ui_view_displays_columns(view))
 		{
-			draw_dir_list(view);
+			/* Inactive cell in ls-like view usually takes less space than an active
+			 * one.  Need to clear the cell before drawing over it. */
+			redraw_cell(view, view->top_line, view->curr_line, 0);
 		}
-		put_inactive_mark(view);
+		redraw_cell(view, view->top_line, view->curr_line, 1);
 	}
 }
 
 void
-put_inactive_mark(view_t *view)
+fview_draw_inactive_cursor(view_t *view)
 {
 	size_t col_width, col_count;
-	int line_attrs;
 	int line, column;
 
+	/* Reset last seen position on drawing inactive cursor or an active one won't
+	 * be drawn next time. */
+	invalidate_cursor_pos_cache(view);
+
 	if(!ui_view_displays_columns(view))
 	{
 		/* Inactive cell in ls-like view usually takes less space than an active
@@ -824,7 +870,7 @@ put_inactive_mark(view_t *view)
 
 	calculate_table_conf(view, &col_count, &col_width);
 
-	line_attrs = prepare_inactive_color(view, get_current_entry(view),
+	cchar_t line_attrs = prepare_inactive_color(view, get_current_entry(view),
 			get_line_color(view, get_current_entry(view)));
 
 	line = fpos_get_line(view, view->curr_line);
@@ -832,26 +878,24 @@ put_inactive_mark(view_t *view)
 	       + fpos_get_col(view, view->curr_line)*col_width;
 	checked_wmove(view->win, line, column);
 
-	wprinta(view->win, INACTIVE_CURSOR_MARK, line_attrs);
+	wprinta(view->win, INACTIVE_CURSOR_MARK, &line_attrs, 0);
 	ui_view_win_changed(view);
 }
 
 /* Calculate color attributes for cursor line of inactive pane.  Returns
  * attributes that can be used for drawing on a window. */
-static int
+static cchar_t
 prepare_inactive_color(view_t *view, dir_entry_t *entry, int line_color)
 {
 	const col_scheme_t *cs = ui_view_get_cs(view);
-	col_attr_t col = cs->color[WIN_COLOR];
+	col_attr_t col = ui_get_win_color(view, cs);
 
 	mix_in_common_colors(&col, view, entry, line_color);
+	cs_mix_colors(&col, &cs->color[OTHER_LINE_COLOR]);
 
-	if(cs_is_color_set(&cs->color[OTHER_LINE_COLOR]))
-	{
-		cs_mix_colors(&col, &cs->color[OTHER_LINE_COLOR]);
-	}
-
-	return COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) | col.attr;
+	cchar_t cch;
+	setcchar(&cch, L" ", col.attr, colmgr_get_pair(col.fg, col.bg), NULL);
+	return cch;
 }
 
 /* Redraws single directory list entry.  is_current defines whether element
@@ -862,7 +906,19 @@ static void
 redraw_cell(view_t *view, int top, int cursor, int is_current)
 {
 	const int pos = top + cursor;
+	if(pos < 0 || pos >= view->list_rows)
+	{
+		/* The entire list is going to be redrawn so just return. */
+		return;
+	}
+
+	if(curr_stats.load_stage < 2 || cursor < 0)
+	{
+		return;
+	}
+
 	size_t col_width, col_count;
+	calculate_table_conf(view, &col_count, &col_width);
 
 	column_data_t cdt = {
 		.view = view,
@@ -870,19 +926,6 @@ redraw_cell(view_t *view, int top, int cursor, int is_current)
 		.line_pos = pos,
 		.current_pos = is_current ? view->list_pos : -1,
 	};
-
-	if(curr_stats.load_stage < 2 || cursor < 0)
-	{
-		return;
-	}
-
-	if(pos < 0 || pos >= view->list_rows)
-	{
-		/* The entire list is going to be redrawn so just return. */
-		return;
-	}
-
-	calculate_table_conf(view, &col_count, &col_width);
 	compute_and_draw_cell(&cdt, cursor, col_width);
 }
 
@@ -1048,8 +1091,9 @@ column_line_print(const void *data, int column_id, const char buf[],
 
 	const int primary = column_id == SK_BY_NAME
 	                 || column_id == SK_BY_INAME
-	                 || column_id == SK_BY_ROOT;
-	const int line_attrs = prepare_col_color(view, primary, cdt);
+	                 || column_id == SK_BY_ROOT
+	                 || column_id == SK_BY_FILEROOT;
+	const cchar_t line_attrs = prepare_col_color(view, primary, cdt);
 
 	size_t extra_prefix = primary ? *cdt->prefix_len : 0U;
 
@@ -1085,7 +1129,8 @@ column_line_print(const void *data, int column_id, const char buf[],
 		full_column += extra_prefix;
 
 		checked_wmove(view->win, cdt->current_line, final_offset - extra_prefix);
-		wprinta(view->win, print_buf, prepare_col_color(view, 0, cdt));
+		cchar_t cch = prepare_col_color(view, 0, cdt);
+		wprinta(view->win, print_buf, &cch, 0);
 	}
 
 	checked_wmove(view->win, cdt->current_line, final_offset);
@@ -1106,12 +1151,12 @@ column_line_print(const void *data, int column_id, const char buf[],
 	{
 		print_buf[trim_pos] = '\0';
 	}
-	wprinta(view->win, print_buf, line_attrs);
+	wprinta(view->win, print_buf, &line_attrs, 0);
 
 	if(primary && view->matches != 0 && entry->search_match)
 	{
 		highlight_search(view, entry, full_column, print_buf, trim_pos, align,
-				cdt->current_line, final_offset, line_attrs);
+				cdt->current_line, final_offset, &line_attrs);
 	}
 }
 
@@ -1132,7 +1177,8 @@ draw_line_number(const column_data_t *cdt, int column)
 	snprintf(num_str, sizeof(num_str), format, cdt->number_width - 1, num);
 
 	checked_wmove(view->win, cdt->current_line, column);
-	wprinta(view->win, num_str, prepare_col_color(view, 0, cdt));
+	cchar_t cch = prepare_col_color(view, 0, cdt);
+	wprinta(view->win, num_str, &cch, 0);
 }
 
 /* Highlights search match for the entry (assumed to be a search hit).  Modifies
@@ -1140,7 +1186,7 @@ draw_line_number(const column_data_t *cdt, int column)
 static void
 highlight_search(view_t *view, dir_entry_t *entry, const char full_column[],
 		char buf[], size_t buf_len, AlignType align, int line, int col,
-		int line_attrs)
+		const cchar_t *line_attrs)
 {
 	size_t name_offset, lo, ro;
 	const char *fname;
@@ -1177,7 +1223,7 @@ highlight_search(view_t *view, dir_entry_t *entry, const char full_column[],
 		copy_str(mark, mark_len + 1, ">>>");
 
 		checked_wmove(view->win, line, col + offset);
-		wprinta(view->win, mark, line_attrs ^ A_REVERSE);
+		wprinta(view->win, mark, line_attrs, A_REVERSE);
 	}
 	else if(align == AT_RIGHT && lo < (short int)strlen(full_column) - buf_len)
 	{
@@ -1188,7 +1234,7 @@ highlight_search(view_t *view, dir_entry_t *entry, const char full_column[],
 		copy_str(mark, mark_len + 1, "<<<");
 
 		checked_wmove(view->win, line, col);
-		wprinta(view->win, mark, line_attrs ^ A_REVERSE);
+		wprinta(view->win, mark, line_attrs, A_REVERSE);
 	}
 	else
 	{
@@ -1216,42 +1262,42 @@ highlight_search(view_t *view, dir_entry_t *entry, const char full_column[],
 
 		checked_wmove(view->win, line, col + match_start);
 		buf[ro] = '\0';
-		wprinta(view->win, buf + lo, line_attrs ^ (A_REVERSE | A_UNDERLINE));
+		wprinta(view->win, buf + lo, line_attrs, (A_REVERSE | A_UNDERLINE));
 	}
 }
 
 /* Calculate color attributes for a view column.  Returns attributes that can be
  * used for drawing on a window. */
-static int
+static cchar_t
 prepare_col_color(const view_t *view, int primary, const column_data_t *cdt)
 {
 	const col_scheme_t *const cs = ui_view_get_cs(view);
-	col_attr_t col = cs->color[WIN_COLOR];
+	col_attr_t col = ui_get_win_color(view, cs);
 
 	if(!cdt->is_main)
 	{
 		cs_mix_colors(&col, &cs->color[AUX_WIN_COLOR]);
 	}
 
-	/* File-specific highlight affects only primary field for non-current lines
-	 * and whole line for the current line. */
-	const int with_line_hi = (primary || cdt->line_pos == cdt->current_pos);
-	const int line_color = with_line_hi ? cdt->line_hi_group : -1;
-	mix_in_common_colors(&col, view, cdt->entry, line_color);
-
-	if(cdt->line_pos == cdt->current_pos)
+	if(cdt->line_pos != -1)
 	{
-		if(view == curr_view)
+		/* File-specific highlight affects only primary field for non-current lines
+		 * and whole line for the current line. */
+		const int with_line_hi = (primary || cdt->line_pos == cdt->current_pos);
+		const int line_color = with_line_hi ? cdt->line_hi_group : -1;
+		mix_in_common_colors(&col, view, cdt->entry, line_color);
+
+		if(cdt->line_pos == cdt->current_pos)
 		{
-			cs_mix_colors(&col, &cs->color[CURR_LINE_COLOR]);
-		}
-		else if(cs_is_color_set(&cs->color[OTHER_LINE_COLOR]))
-		{
-			cs_mix_colors(&col, &cs->color[OTHER_LINE_COLOR]);
+			int color = (view == curr_view || !cdt->is_main) ? CURR_LINE_COLOR
+			                                                 : OTHER_LINE_COLOR;
+			cs_mix_colors(&col, &cs->color[color]);
 		}
 	}
 
-	return COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) | col.attr;
+	cchar_t cch;
+	setcchar(&cch, L" ", col.attr, colmgr_get_pair(col.fg, col.bg), NULL);
+	return cch;
 }
 
 /* Mixes in colors of current entry, mismatch and selection. */
@@ -1321,14 +1367,18 @@ mix_in_file_name_hi(const view_t *view, dir_entry_t *entry, col_attr_t *col)
 TSTATIC void
 format_name(int id, const void *data, size_t buf_len, char buf[])
 {
-	const NameFormat fmt = (id == SK_BY_ROOT ? NF_ROOT : NF_FULL);
-
 	size_t len, i;
 	dir_entry_t *child, *parent;
 
 	const column_data_t *cdt = data;
 	view_t *view = cdt->view;
 
+	NameFormat fmt = NF_FULL;
+	if(id == SK_BY_ROOT || (id == SK_BY_FILEROOT && !fentry_is_dir(cdt->entry)))
+	{
+		fmt = NF_ROOT;
+	}
+
 	if(!flist_custom_active(view))
 	{
 		/* Just file name. */
@@ -1386,7 +1436,7 @@ format_primary_group(int id, const void *data, size_t buf_len, char buf[])
 	const view_t *view = cdt->view;
 	regmatch_t match = get_group_match(&view->primary_group, cdt->entry->name);
 
-	copy_str(buf, MIN(buf_len + 1U, match.rm_eo - match.rm_so + 1U),
+	copy_str(buf, MIN(buf_len + 1U, (size_t)match.rm_eo - match.rm_so + 1U),
 			cdt->entry->name + match.rm_so);
 }
 
@@ -1523,8 +1573,7 @@ format_time(int id, const void *data, size_t buf_len, char buf[])
 
 	if(tm_ptr != NULL)
 	{
-		buf[0] = ' ';
-		strftime(buf + 1, buf_len, cfg.time_format, tm_ptr);
+		strftime(buf, buf_len + 1, cfg.time_format, tm_ptr);
 	}
 	else
 	{
@@ -1664,6 +1713,8 @@ fview_list_updated(view_t *view)
 {
 	/* Invalidate maximum file name widths cache. */
 	view->max_filename_width = 0;
+	/* Even if position will remain the same, we might need to redraw it. */
+	invalidate_cursor_pos_cache(view);
 }
 
 /* Evaluates number of columns in the view.  Returns the number. */
@@ -1733,8 +1784,6 @@ fview_position_updated(view_t *view)
 	const int old_top = view->top_line;
 	const int old_curr = view->curr_line;
 
-	int redraw;
-
 	if(view->curr_line > view->list_rows - 1)
 	{
 		view->curr_line = view->list_rows - 1;
@@ -1747,6 +1796,7 @@ fview_position_updated(view_t *view)
 
 	if(view == other_view)
 	{
+		invalidate_cursor_pos_cache(view);
 		if(move_curr_line(view))
 		{
 			draw_dir_list(view);
@@ -1754,18 +1804,23 @@ fview_position_updated(view_t *view)
 		else
 		{
 			redraw_cell(view, old_top, old_curr, 0);
-			put_inactive_mark(view);
+			fview_draw_inactive_cursor(view);
 		}
 		return;
 	}
 
-	redraw = move_curr_line(view);
-
 	if(curr_stats.load_stage < 2)
 	{
 		return;
 	}
 
+	int redraw = move_curr_line(view);
+	int up_to_date = cache_cursor_pos(view);
+	if(!redraw && up_to_date)
+	{
+		return;
+	}
+
 	if(redraw)
 	{
 		draw_dir_list(view);
@@ -1774,10 +1829,9 @@ fview_position_updated(view_t *view)
 	{
 		redraw_cell(view, old_top, old_curr, 0);
 		redraw_cell(view, view->top_line, view->curr_line, 1);
+		draw_right_column(view);
 	}
 
-	draw_right_column(view);
-
 	refresh_view_win(view);
 	ui_stat_update(view, 0);
 
@@ -1796,6 +1850,36 @@ fview_position_updated(view_t *view)
 	}
 }
 
+/* Compares current cursor position against previously cached one and updates
+ * the cache if necessary.  Returns non-zero if cache is up to date, otherwise
+ * zero is returned. */
+static int
+cache_cursor_pos(view_t *view)
+{
+	char path[PATH_MAX + 1];
+	get_current_full_path(view, sizeof(path), path);
+
+	if(view->list_pos == view->last_seen_pos &&
+		 view->curr_line == view->last_curr_line &&
+		 view->last_curr_file != NULL &&
+		 strcmp(view->last_curr_file, path) == 0)
+	{
+		return 1;
+	}
+
+	view->last_seen_pos = view->list_pos;
+	view->last_curr_line = view->curr_line;
+	replace_string(&view->last_curr_file, path);
+	return 0;
+}
+
+/* Invalidates cache of current cursor position for the specified view. */
+static void
+invalidate_cursor_pos_cache(view_t *view)
+{
+	view->last_seen_pos = -1;
+}
+
 /* Moves hardware cursor to the beginning of the name of current entry. */
 static void
 position_hardware_cursor(view_t *view)
