@@ -19,7 +19,7 @@
 
 #include "quickview.h"
 
-#include <curses.h> /* mvwaddstr() wattrset() */
+#include <curses.h> /* mvwaddstr() */
 #include <unistd.h> /* usleep() */
 
 #include <limits.h> /* INT_MAX */
@@ -27,7 +27,7 @@
 #include <stdio.h> /* FILE SEEK_SET fclose() fdopen() feof() fseek()
                       tmpfile() */
 #include <stdlib.h> /* free() */
-#include <string.h> /* memmove() strcat() strlen() strncat() */
+#include <string.h> /* strcat() strlen() strncat() */
 
 #include "../cfg/config.h"
 #include "../compat/fs_limits.h"
@@ -37,6 +37,7 @@
 #include "../modes/modes.h"
 #include "../modes/view.h"
 #include "../utils/file_streams.h"
+#include "../utils/filemon.h"
 #include "../utils/fs.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
@@ -58,8 +59,23 @@
 #include "statusbar.h"
 #include "ui.h"
 
-/* Size of buffer holding preview line (in characters). */
-#define PREVIEW_LINE_BUF_LEN 4096
+/* Maximum number of lines used for preview. */
+enum { MAX_PREVIEW_LINES = 256 };
+
+/* Cached information about a single file's preview. */
+typedef struct
+{
+	char *path;        /* Full path to the file. */
+	char *viewer;      /* Viewer of the file. */
+	filemon_t filemon; /* Timestamp for the file. */
+	strlist_t lines;   /* Top MAX_PREVIEW_LINES of preview contents. */
+	preview_area_t pa; /* Where preview is being drawn. */
+	int beg_x;         /* Original x coordinate of host window. */
+	int beg_y;         /* Original y coordinate of host window. */
+	ViewerKind kind;   /* Kind of preview. */
+	int graphics_lost; /* Whether graphics was invalidated on the screen. */
+}
+quickview_cache_t;
 
 /* State of directory tree print functions. */
 typedef struct
@@ -73,8 +89,15 @@ typedef struct
 }
 tree_print_state_t;
 
-static void view_entry(const dir_entry_t *entry);
-static void view_file(const char path[]);
+static void view_entry(const dir_entry_t *entry, const preview_area_t *parea,
+		quickview_cache_t *cache);
+static void view_file(const char path[], const preview_area_t *parea,
+		quickview_cache_t *cache);
+static int is_cache_valid(const quickview_cache_t *cache, const char path[],
+		const char viewer[], const preview_area_t *parea);
+static void fill_cache(quickview_cache_t *cache, FILE *fp, const char path[],
+		const char viewer[], ViewerKind kind, const preview_area_t *parea);
+TSTATIC strlist_t read_lines(FILE *fp, int max_lines);
 static FILE * view_dir(const char path[], int max_lines);
 static int print_dir_tree(tree_print_state_t *s, const char path[], int last);
 static int enter_dir(tree_print_state_t *s, const char path[], int last);
@@ -88,12 +111,16 @@ static void set_prefix_char(tree_print_state_t *s, char c);
 static void print_tree_entry(tree_print_state_t *s, const char path[],
 		int end_line);
 static void print_entry_prefix(tree_print_state_t *s);
-TSTATIC void view_stream(FILE *fp, int wrapped);
-static int shift_line(char line[], size_t len, size_t offset);
-static size_t add_to_line(FILE *fp, size_t max, char line[], size_t len);
-static void write_message(const char msg[]);
-static void cleanup_for_text(void);
+static void draw_lines(const strlist_t *lines, int wrapped,
+		const preview_area_t *parea, ViewerKind kind);
+static void write_message(const char msg[], const preview_area_t *parea);
+static void cleanup_for_text(const preview_area_t *parea);
 static char * expand_viewer_command(const char viewer[]);
+static void cleanup_area(const preview_area_t *parea, const char cmd[]);
+static void wipe_area(const preview_area_t *parea);
+
+/* Cached preview data for a single file entry. */
+static quickview_cache_t qv_cache;
 
 int
 qv_ensure_is_shown(void)
@@ -125,32 +152,33 @@ qv_can_show(void)
 void
 qv_toggle(void)
 {
-	if(curr_stats.preview.on)
-	{
-		stats_set_quickview(0);
-
-		if(ui_view_is_visible(other_view))
-		{
-			/* Force cleaning possible leftovers of graphics, otherwise curses
-			 * internal structures don't know that those parts need to be redrawn on
-			 * the screen. */
-			if(curr_stats.preview.cleanup_cmd != NULL || curr_stats.preview.graphical)
-			{
-				qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
-			}
-
-			draw_dir_list(other_view);
-			refresh_view_win(other_view);
-		}
-
-		update_string(&curr_stats.preview.cleanup_cmd, NULL);
-		curr_stats.preview.graphical = 0;
-	}
-	else
+	if(!curr_stats.preview.on)
 	{
 		stats_set_quickview(1);
-		qv_draw(curr_view);
+		stats_redraw_schedule();
+		return;
 	}
+
+	stats_set_quickview(0);
+
+	if(ui_view_is_visible(other_view))
+	{
+		/* Force cleaning possible leftovers of graphics, otherwise curses internal
+		 * structures don't know that those parts need to be redrawn on the
+		 * screen. */
+		if(curr_stats.preview.cleanup_cmd != NULL ||
+				curr_stats.preview.kind != VK_TEXTUAL)
+		{
+			qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
+		}
+
+		draw_dir_list(other_view);
+		refresh_view_win(other_view);
+	}
+
+	update_string(&curr_stats.preview.cleanup_cmd, NULL);
+	curr_stats.preview.kind = VK_TEXTUAL;
+	qv_ui_updated();
 }
 
 void
@@ -179,16 +207,54 @@ qv_draw(view_t *view)
 	curr = get_current_entry(view);
 	if(!fentry_is_fake(curr))
 	{
-		view_entry(curr);
+		const preview_area_t parea = {
+			.source = view,
+			.view = other_view,
+			.def_col = ui_view_get_cs(other_view)->color[WIN_COLOR],
+			.x = ui_qv_left(other_view),
+			.y = ui_qv_top(other_view),
+			.w = ui_qv_width(other_view),
+			.h = ui_qv_height(other_view),
+		};
+		view_entry(curr, &parea, &qv_cache);
 	}
 
 	refresh_view_win(other_view);
 	ui_view_title_update(other_view);
 }
 
+void
+qv_draw_on(const dir_entry_t *entry, const preview_area_t *parea)
+{
+	static quickview_cache_t lwin_cache, rwin_cache;
+
+	char filler[parea->w + 1];
+	memset(filler, ' ', sizeof(filler) - 1U);
+	filler[sizeof(filler) - 1U] = '\0';
+
+	ui_set_attr(parea->view->win, &parea->def_col, -1);
+
+	int line;
+	for(line = parea->y; line < parea->y + parea->h; ++line)
+	{
+		mvwaddstr(parea->view->win, line, parea->x, filler);
+	}
+
+	quickview_cache_t *cache = (parea->view == &lwin ? &lwin_cache : &rwin_cache);
+
+	view_entry(entry, parea, cache);
+
+	parea->view->displays_graphics = (cache->kind != VK_TEXTUAL);
+
+	/* Unconditionally invalidate graphics cache, since we don't keep track of its
+	 * validity in any way. */
+	cache->graphics_lost = 1;
+}
+
 /* Draws preview of the entry in the other view. */
 static void
-view_entry(const dir_entry_t *entry)
+view_entry(const dir_entry_t *entry, const preview_area_t *parea,
+		quickview_cache_t *cache)
 {
 	char path[PATH_MAX + 1];
 	qv_get_path_to_explore(entry, path, sizeof(path));
@@ -196,23 +262,23 @@ view_entry(const dir_entry_t *entry)
 	switch(entry->type)
 	{
 		case FT_CHAR_DEV:
-			write_message("File is a Character Device");
+			write_message("File is a Character Device", parea);
 			break;
 		case FT_BLOCK_DEV:
-			write_message("File is a Block Device");
+			write_message("File is a Block Device", parea);
 			break;
 #ifndef _WIN32
 		case FT_SOCK:
-			write_message("File is a Socket");
+			write_message("File is a Socket", parea);
 			break;
 #endif
 		case FT_FIFO:
-			write_message("File is a Named Pipe");
+			write_message("File is a Named Pipe", parea);
 			break;
 		case FT_LINK:
 			if(get_link_target_abs(path, entry->origin, path, sizeof(path)) != 0)
 			{
-				write_message("Cannot resolve Link");
+				write_message("Cannot resolve Link", parea);
 				break;
 			}
 			if(!ends_with_slash(path) && is_dir(path))
@@ -222,7 +288,7 @@ view_entry(const dir_entry_t *entry)
 			/* break is omitted intentionally. */
 		case FT_UNK:
 		default:
-			view_file(path);
+			view_file(path, parea, cache);
 			break;
 	}
 }
@@ -230,15 +296,22 @@ view_entry(const dir_entry_t *entry)
 /* Displays contents of file or output of its viewer in the other pane
  * starting from the second line and second column. */
 static void
-view_file(const char path[])
+view_file(const char path[], const preview_area_t *parea,
+		quickview_cache_t *cache)
 {
-	int graphical = 0;
-	const char *viewer;
-	const char *clear_cmd;
+	const char *viewer = qv_get_viewer(path);
+
+	if(is_cache_valid(cache, path, viewer, parea))
+	{
+		/* Update area as we might draw preview at a different location. */
+		cache->pa = *parea;
+		draw_lines(&cache->lines, cfg.wrap_quick_view, &cache->pa, cache->kind);
+		return;
+	}
+
+	ViewerKind kind = VK_TEXTUAL;
+
 	FILE *fp;
-
-	viewer = qv_get_viewer(path);
-
 	if(viewer == NULL && is_dir(path))
 	{
 		ui_cancellation_reset();
@@ -248,7 +321,7 @@ view_file(const char path[])
 
 		if(fp == NULL)
 		{
-			write_message("Failed to view directory");
+			write_message("Failed to view directory", parea);
 			return;
 		}
 	}
@@ -257,24 +330,32 @@ view_file(const char path[])
 		fp = os_fopen(path, "rb");
 		if(fp == NULL)
 		{
-			write_message("Cannot open file");
+			write_message("Cannot open file", parea);
 			return;
 		}
 	}
 	else
 	{
-		graphical = is_graphical_viewer(viewer);
+		kind = ft_viewer_kind(viewer);
+
 		/* If graphics will be displayed, clear the window and wait a bit to let
 		 * terminal emulator do actual refresh (at least some of them need this). */
-		if(graphical)
+		if(kind != VK_TEXTUAL)
 		{
-			qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
+			cleanup_area(parea, curr_stats.preview.cleanup_cmd);
 			usleep(50000);
 		}
+
+		view_t *const curr = curr_view;
+		curr_view = parea->source;
+		curr_stats.preview_hint = parea;
 		fp = qv_execute_viewer(viewer);
+		curr_stats.preview_hint = NULL;
+		curr_view = curr;
+
 		if(fp == NULL)
 		{
-			write_message("Cannot read viewer output");
+			write_message("Cannot read viewer output", parea);
 			return;
 		}
 	}
@@ -284,20 +365,100 @@ view_file(const char path[])
 
 	/* We want to wipe the view if it was displaying graphics, but won't anymore.
 	 * Do this only if we didn't already cleared the window. */
-	if(!graphical)
+	if(kind == VK_TEXTUAL)
 	{
-		cleanup_for_text();
+		cleanup_for_text(parea);
 	}
-	curr_stats.preview.graphical = graphical;
+	curr_stats.preview.kind = kind;
 
-	clear_cmd = (viewer != NULL) ? ma_get_clear_cmd(viewer) : NULL;
+	const char *clear_cmd = (viewer != NULL) ? ma_get_clear_cmd(viewer) : NULL;
 	update_string(&curr_stats.preview.cleanup_cmd, clear_cmd);
 
-	wattrset(other_view->win, 0);
-	view_stream(fp, cfg.wrap_quick_view);
+	fill_cache(cache, fp, path, viewer, kind, parea);
+
 	fclose(fp);
 
 	ui_cancellation_disable();
+
+	draw_lines(&cache->lines, cfg.wrap_quick_view, &cache->pa, cache->kind);
+}
+
+/* Checks whether data in the cache is up to date with the file on disk.
+ * Returns non-zero if so, otherwise zero is returned. */
+static int
+is_cache_valid(const quickview_cache_t *cache, const char path[],
+		const char viewer[], const preview_area_t *parea)
+{
+	int same_viewer = (cache->viewer == NULL && viewer == NULL)
+	               || (cache->viewer != NULL && viewer != NULL &&
+	                   strcmp(cache->viewer, viewer) == 0);
+
+	filemon_t filemon;
+	if(same_viewer &&
+			filemon_from_file(path, FMT_MODIFIED, &filemon) == 0 &&
+			cache->path != NULL &&
+			paths_are_equal(cache->path, path) &&
+			filemon_equal(&cache->filemon, &filemon))
+	{
+		if(cache->kind == VK_TEXTUAL)
+		{
+			return 1;
+		}
+
+		return !cache->graphics_lost
+		    && cache->pa.h == parea->h
+		    && cache->pa.w == parea->w
+		    && cache->beg_x + cache->pa.x == getbegx(parea->view->win) + parea->x
+		    && cache->beg_y + cache->pa.y == getbegy(parea->view->win) + parea->y;
+	}
+
+	return 0;
+}
+
+/* Fills the cache data with file's contents. */
+static void
+fill_cache(quickview_cache_t *cache, FILE *fp, const char path[],
+		const char viewer[], ViewerKind kind, const preview_area_t *parea)
+{
+	/* File monitor must always be initialized, because it's used below. */
+	filemon_t filemon = {};
+	(void)filemon_from_file(path, FMT_MODIFIED, &filemon);
+	filemon_assign(&cache->filemon, &filemon);
+
+	replace_string(&cache->path, path);
+	update_string(&cache->viewer, viewer);
+
+	free_string_array(cache->lines.items, cache->lines.nitems);
+	cache->lines = read_lines(fp, MAX_PREVIEW_LINES);
+
+	cache->pa = *parea;
+	cache->beg_x = getbegx(parea->view->win);
+	cache->beg_y = getbegy(parea->view->win);
+	cache->kind = kind;
+	cache->graphics_lost = 0;
+}
+
+/* Reads at most max_lines from the stream ignoring BOM.  Returns the lines
+ * read. */
+TSTATIC strlist_t
+read_lines(FILE *fp, int max_lines)
+{
+	strlist_t lines = {};
+	skip_bom(fp);
+
+	char *next_line;
+	while(lines.nitems < max_lines && (next_line = read_line(fp, NULL)) != NULL)
+	{
+		const int old_len = lines.nitems;
+		lines.nitems = put_into_string_array(&lines.items, lines.nitems, next_line);
+		if(lines.nitems == old_len)
+		{
+			free(next_line);
+			break;
+		}
+	}
+
+	return lines;
 }
 
 FILE *
@@ -529,107 +690,73 @@ print_entry_prefix(tree_print_state_t *s)
 	}
 }
 
-/* Displays contents read from the fp in the other pane starting from the second
- * line and second column.  The wrapped parameter determines whether lines
- * should be wrapped. */
-TSTATIC void
-view_stream(FILE *fp, int wrapped)
+/* Displays lines in the other pane.  The wrapped parameter determines whether
+ * lines should be wrapped. */
+static void
+draw_lines(const strlist_t *lines, int wrapped, const preview_area_t *parea,
+		ViewerKind kind)
 {
-	const size_t left = ui_qv_left(other_view);
-	const size_t top = ui_qv_top(other_view);
-	const size_t max_width = ui_qv_width(other_view);
-	const size_t max_height = ui_qv_height(other_view);
+	if(kind == VK_PASS_THROUGH)
+	{
+		ui_pass_through(lines, parea->view->win, parea->x, parea->y);
+		return;
+	}
+
+	const size_t left = parea->x;
+	const size_t top = parea->y;
+	const size_t max_width = parea->w;
+	const size_t max_height = parea->h;
 
-	const col_scheme_t *cs = ui_view_get_cs(other_view);
-	char line[PREVIEW_LINE_BUF_LEN];
 	int line_continued = 0;
 	size_t y = top;
-	const char *res;
+
 	esc_state state;
+	esc_state_init(&state, &parea->def_col, COLORS);
 
-	esc_state_init(&state, &cs->color[WIN_COLOR], COLORS);
-
-	skip_bom(fp);
-	res = get_line(fp, line, sizeof(line));
-
-	while(res != NULL && y < top + max_height)
+	int next_line = 0;
+	const char *input_line = (next_line == lines->nitems)
+	                       ? NULL
+	                       : lines->items[next_line++];
+	while(input_line != NULL && y < top + max_height)
 	{
-		int offset;
 		int printed;
-		const size_t len = add_to_line(fp, max_width, line, sizeof(line));
-		if(!wrapped && line[len - 1] != '\n')
-		{
-			skip_until_eol(fp);
-		}
+		input_line += esc_print_line(input_line, parea->view->win, left, y,
+				max_width, 0, !wrapped, &state, &printed);
 
-		offset = esc_print_line(line, other_view->win, left, y, max_width, 0,
-				&state, &printed);
 		y += !wrapped || (!line_continued || printed);
-		line_continued = line[len - 1] != '\n';
+		line_continued = (*input_line != '\0');
 
-		if(y < top + max_height && (!wrapped || shift_line(line, len, offset)))
+		if(y < top + max_height && (!wrapped || !line_continued))
 		{
-			res = get_line(fp, line, sizeof(line));
+			input_line = (next_line == lines->nitems) ? NULL
+			                                          : lines->items[next_line++];
 		}
 	}
 }
 
-/* Shifts characters in the line of length len, so that characters at the offset
- * position are moved to the beginning of the line.  Returns non-zero if new
- * buffer should be treated as empty. */
-static int
-shift_line(char line[], size_t len, size_t offset)
-{
-	const size_t shift_width = len - offset;
-	if(shift_width != 0)
-	{
-		memmove(line, line + offset, shift_width + 1);
-		return (shift_width == 1 && line[0] == '\n');
-	}
-	return 1;
-}
-
-/* Tries to add more characters from the fp file, but not exceed length of the
- * line buffer (the len parameter) and maximum number of printable character
- * positions (the max parameter).  Returns new length of the line buffer. */
-static size_t
-add_to_line(FILE *fp, size_t max, char line[], size_t len)
-{
-	size_t n_len = utf8_nstrlen(line) - esc_str_overhead(line);
-	size_t curr_len = strlen(line);
-	while(n_len < max && line[curr_len - 1] != '\n' && !feof(fp))
-	{
-		if(get_line(fp, line + curr_len, len - curr_len) == NULL)
-		{
-			break;
-		}
-		n_len = utf8_nstrlen(line) - esc_str_overhead(line);
-		curr_len = strlen(line);
-	}
-	return curr_len;
-}
-
 /* Writes single line message of error or information kind instead of real
  * preview. */
 static void
-write_message(const char msg[])
+write_message(const char msg[], const preview_area_t *parea)
 {
-	cleanup_for_text();
-	wattrset(other_view->win, 0);
-	mvwaddstr(other_view->win, ui_qv_top(other_view), ui_qv_left(other_view),
-			msg);
+	cleanup_for_text(parea);
+
+	char *items[] = { (char *)msg };
+	const strlist_t lines = { .items = items, .nitems = 1 };
+	draw_lines(&lines, 1, parea, VK_TEXTUAL);
 }
 
-/* Ensures that view is ready to display regular text. */
+/* Ensures that area is ready to display regular text. */
 static void
-cleanup_for_text(void)
+cleanup_for_text(const preview_area_t *parea)
 {
-	if(curr_stats.preview.cleanup_cmd != NULL || curr_stats.preview.graphical)
+	if(curr_stats.preview.cleanup_cmd != NULL ||
+			curr_stats.preview.kind != VK_TEXTUAL)
 	{
-		qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
+		cleanup_area(parea, curr_stats.preview.cleanup_cmd);
 	}
 	update_string(&curr_stats.preview.cleanup_cmd, NULL);
-	curr_stats.preview.graphical = 0;
+	curr_stats.preview.kind = VK_TEXTUAL;
 }
 
 void
@@ -676,7 +803,7 @@ expand_viewer_command(const char viewer[])
 	}
 	else
 	{
-		result = expand_macros(viewer, NULL, NULL, 1);
+		result = ma_expand(viewer, NULL, NULL, 1);
 	}
 	return result;
 }
@@ -684,25 +811,68 @@ expand_viewer_command(const char viewer[])
 void
 qv_cleanup(view_t *view, const char cmd[])
 {
-	view_t *const curr = curr_view;
-	FILE *fp;
+	const preview_area_t parea = {
+		.source = view,
+		.view = view,
+		.def_col = ui_view_get_cs(view)->color[WIN_COLOR],
+		.x = 0,
+		.y = 0,
+		.w = view->window_cols,
+		.h = view->window_rows,
+	};
+	cleanup_area(&parea, cmd);
+}
 
+/* Erases area using external command if available. */
+static void
+cleanup_area(const preview_area_t *parea, const char cmd[])
+{
 	if(cmd == NULL)
 	{
-		ui_view_wipe(view);
+		wipe_area(parea);
 		return;
 	}
 
-	curr_view = view;
+	view_t *const curr = curr_view;
+	curr_view = parea->view;
+
 	curr_stats.preview.clearing = 1;
-	fp = qv_execute_viewer(cmd);
+	curr_stats.preview_hint = parea;
+	FILE *fp = qv_execute_viewer(cmd);
+	curr_stats.preview_hint = NULL;
 	curr_stats.preview.clearing = 0;
+
 	curr_view = curr;
 
 	while(fgetc(fp) != EOF);
 	fclose(fp);
 
-	ui_view_wipe(view);
+	wipe_area(parea);
+}
+
+/* Ensures that area of a view is updated on the screen (e.g. to clear anything
+ * put there by other programs as well). */
+static void
+wipe_area(const preview_area_t *parea)
+{
+	/* User doesn't need to see fake filling so draw it with the color of
+	 * background. */
+	col_attr_t col = { .fg = parea->def_col.fg, .bg = parea->def_col.bg };
+	ui_set_attr(parea->view->win, &col, -1);
+
+	char filler[parea->w + 1];
+	memset(filler, ' ', sizeof(filler) - 1U);
+	filler[sizeof(filler) - 1U] = '\0';
+
+	int line;
+	for(line = parea->y; line < parea->y + parea->h; ++line)
+	{
+		mvwaddstr(parea->view->win, line, parea->x, filler);
+	}
+	redrawwin(parea->view->win);
+	ui_refresh_win(parea->view->win);
+
+	ui_set_attr(parea->view->win, &parea->def_col, -1);
 }
 
 const char *
@@ -746,5 +916,12 @@ qv_get_path_to_explore(const dir_entry_t *entry, char buf[], size_t buf_len)
 	get_full_path_of(entry, buf_len, buf);
 }
 
+void
+qv_ui_updated(void)
+{
+	/* Invalidate graphical cache. */
+	qv_cache.graphics_lost = 1;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
