@@ -23,13 +23,12 @@
 
 #include "ui.h"
 
-#include <curses.h> /* mvwin() wbkgdset() werase() */
+#include <curses.h> /* mvwin() werase() */
 
 #ifndef _WIN32
 #include <sys/ioctl.h>
 #include <termios.h> /* struct winsize */
 #endif
-#include <sys/time.h> /* gettimeofday() */
 #include <unistd.h>
 
 #include <assert.h> /* assert() */
@@ -40,7 +39,6 @@
 #include <stdlib.h> /* abs() free() */
 #include <stdio.h> /* snprintf() vsnprintf() */
 #include <string.h> /* memset() strcat() strcmp() strcpy() strdup() strlen() */
-#include <time.h> /* time() */
 #include <wchar.h> /* wint_t wcslen() */
 
 #include "../cfg/config.h"
@@ -60,6 +58,7 @@
 #include "../utils/matchers.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
+#include "../utils/string_array.h"
 #include "../utils/utf8.h"
 #include "../utils/utils.h"
 #include "../event_loop.h"
@@ -82,6 +81,13 @@
 /* Type of path transformation function for format_view_title(). */
 typedef char * (*path_func)(const char[]);
 
+/* Window configured to do not wait for input.  Never appears on the screen,
+ * sometimes used for reading input. */
+static WINDOW *no_delay_window;
+/* Window configured to wait for input indefinitely.  Never appears on the
+ * screen, sometimes used for reading input. */
+static WINDOW *inf_delay_window;
+
 static WINDOW *ltop_line1;
 static WINDOW *ltop_line2;
 static WINDOW *rtop_line1;
@@ -100,6 +106,7 @@ static void move_pair(short int from, short int to);
 static void create_windows(void);
 static void update_geometry(void);
 static int get_working_area_height(void);
+static void resize_all(void);
 static void clear_border(WINDOW *border);
 static int middle_border_is_visible(void);
 static void update_views(int reload);
@@ -129,7 +136,6 @@ static void print_view_title(const view_t *view, int active_view, char title[]);
 static col_attr_t fixup_titles_attributes(const view_t *view, int active_view);
 static int is_in_miller_view(const view_t *view);
 static int is_forced_list_mode(const view_t *view);
-static uint64_t get_updated_time(uint64_t prev);
 
 void
 ui_ruler_update(view_t *view, int lazy_redraw)
@@ -224,7 +230,7 @@ setup_ncurses_interface(void)
 #endif
 #endif
 
-	update_geometry();
+	resize_all();
 
 	return 1;
 }
@@ -275,6 +281,18 @@ move_pair(short int from, short int to)
 static void
 create_windows(void)
 {
+	no_delay_window = newwin(1, 1, 0, 0);
+	wtimeout(no_delay_window, 0);
+
+	inf_delay_window = newwin(1, 1, 0, 0);
+	wtimeout(inf_delay_window, -1);
+
+	/* These refreshes prevent curses from drawing these windows on first read
+	 * from them.  It seems that this way they get updated early and this keeps
+	 * them from being drawn again. */
+	wnoutrefresh(no_delay_window);
+	wnoutrefresh(inf_delay_window);
+
 	menu_win = newwin(1, 1, 0, 0);
 	sort_win = newwin(1, 1, 0, 0);
 	change_win = newwin(1, 1, 0, 0);
@@ -345,27 +363,47 @@ ui_update_term_state(void)
 int
 ui_char_pressed(wint_t c)
 {
-	wint_t pressed = L'\0';
-	const int cancellation_state = ui_cancellation_pause();
-
-	/* Query single character in non-blocking mode. */
-	wtimeout(status_bar, 0);
-	if(compat_wget_wch(status_bar, &pressed) != ERR)
+	if(curr_stats.load_stage < 2)
 	{
-		if(pressed != c && pressed != NC_C_c)
+		return 0;
+	}
+
+	const int cancellation_state = ui_cancellation_pause();
+	wint_t pressed;
+
+	while(1)
+	{
+		pressed = L'\0';
+		/* Query single character in non-blocking mode. */
+		if(compat_wget_wch(no_delay_window, &pressed) == ERR)
 		{
-			compat_unget_wch(pressed);
+			break;
+		}
+
+		if(c != NC_C_c && pressed == NC_C_c)
+		{
+			ui_cancellation_request();
+		}
+
+		if(pressed == c)
+		{
+			break;
 		}
 	}
 
 	ui_cancellation_resume(cancellation_state);
 
-	if(c != NC_C_c && pressed == NC_C_c)
-	{
-		ui_cancellation_request();
-	}
+	return (pressed == c);
+}
 
-	return pressed == c;
+void
+ui_drain_input(void)
+{
+	wint_t c;
+	while(compat_wget_wch(no_delay_window, &c) != ERR)
+	{
+		/* Discard input. */
+	}
 }
 
 static void
@@ -433,8 +471,7 @@ vertical_layout(int screen_x)
 	wresize(lwin.win, border_height, splitter_pos + vborder_size_correction);
 	mvwin(lwin.win, y + 1, vborder_pos_correction);
 
-	wbkgdset(mborder, COLOR_PAIR(cfg.cs.pair[BORDER_COLOR]) |
-			cfg.cs.color[BORDER_COLOR].attr);
+	ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 	wresize(mborder, border_height, splitter_width);
 	mvwin(mborder, y + 1, splitter_pos);
 
@@ -493,8 +530,7 @@ horizontal_layout(int screen_x, int screen_y)
 			screen_x + vborder_size_correction);
 	mvwin(rwin.win, splitter_pos + 1, vborder_pos_correction);
 
-	wbkgdset(mborder, COLOR_PAIR(cfg.cs.pair[BORDER_COLOR]) |
-			cfg.cs.color[BORDER_COLOR].attr);
+	ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 	wresize(mborder, 1, screen_x);
 	mvwin(mborder, splitter_pos, 0);
 
@@ -517,89 +553,6 @@ horizontal_layout(int screen_x, int screen_y)
 	mvwin(rborder, y, screen_x - 1);
 }
 
-static void
-resize_all(void)
-{
-	static float prev_x = -1.f, prev_y = -1.f;
-
-	int screen_x, screen_y;
-	int border_height;
-
-	update_geometry();
-	getmaxyx(stdscr, screen_y, screen_x);
-
-	LOG_INFO_MSG("screen_y = %d; screen_x = %d", screen_y, screen_x);
-
-	if(stats_update_term_state(screen_x, screen_y) != TS_NORMAL)
-	{
-		return;
-	}
-
-	if(prev_x < 0)
-	{
-		prev_x = screen_x;
-		prev_y = screen_y;
-	}
-
-	if(curr_stats.splitter_pos >= 0)
-	{
-		if(curr_stats.split == HSPLIT)
-			curr_stats.splitter_pos *= screen_y/prev_y;
-		else
-			curr_stats.splitter_pos *= screen_x/prev_x;
-	}
-
-	prev_x = screen_x;
-	prev_y = screen_y;
-
-	wresize(stdscr, screen_y, screen_x);
-	wresize(menu_win, screen_y - 1, screen_x);
-
-	border_height = get_working_area_height();
-
-	wbkgdset(lborder, COLOR_PAIR(cfg.cs.pair[BORDER_COLOR]) |
-			cfg.cs.color[BORDER_COLOR].attr);
-	wresize(lborder, border_height, 1);
-	mvwin(lborder, 1, 0);
-
-	wbkgdset(rborder, COLOR_PAIR(cfg.cs.pair[BORDER_COLOR]) |
-			cfg.cs.color[BORDER_COLOR].attr);
-	wresize(rborder, border_height, 1);
-	mvwin(rborder, 1, screen_x - 1);
-
-	/* These need a resize at least after terminal size was zero or they grow and
-	 * produce bad looking effect. */
-	wresize(ltop_line1, 1, 1);
-	wresize(ltop_line2, 1, 1);
-	wresize(rtop_line1, 1, 1);
-	wresize(rtop_line2, 1, 1);
-
-	if(curr_stats.number_of_windows == 1)
-	{
-		only_layout(&lwin, screen_x);
-		only_layout(&rwin, screen_x);
-	}
-	else
-	{
-		if(curr_stats.split == HSPLIT)
-			horizontal_layout(screen_x, screen_y);
-		else
-			vertical_layout(screen_x);
-	}
-
-	correct_size(&lwin);
-	correct_size(&rwin);
-
-	wresize(stat_win, 1, screen_x);
-	(void)ui_stat_reposition(1, 0);
-
-	wresize(job_bar, 1, screen_x);
-
-	update_statusbar_layout();
-
-	curs_set(0);
-}
-
 /* Calculates height available for main area that contains file lists.  Returns
  * the height. */
 static int
@@ -701,6 +654,7 @@ update_screen(UpdateType update_kind)
 	}
 
 	curr_stats.need_update = UT_NONE;
+	qv_ui_updated();
 
 	update_views(update_kind == UT_FULL);
 	/* Redraw message dialog over updated panes.  It's not very nice to do it
@@ -758,6 +712,89 @@ update_screen(UpdateType update_kind)
 	curr_stats.need_update = UT_NONE;
 }
 
+/* Resizes all windows according to current screen size and TUI
+ * configuration. */
+static void
+resize_all(void)
+{
+	static float prev_w = -1.f, prev_h = -1.f;
+
+	update_geometry();
+
+	int screen_w, screen_h;
+	getmaxyx(stdscr, screen_h, screen_w);
+	LOG_INFO_MSG("screen_h = %d; screen_w = %d", screen_h, screen_w);
+
+	if(stats_update_term_state(screen_w, screen_h) != TS_NORMAL)
+	{
+		return;
+	}
+
+	if(prev_w < 0)
+	{
+		prev_w = screen_w;
+		prev_h = screen_h;
+	}
+
+	if(curr_stats.splitter_pos >= 0)
+	{
+		if(curr_stats.split == HSPLIT)
+			curr_stats.splitter_pos *= screen_h/prev_h;
+		else
+			curr_stats.splitter_pos *= screen_w/prev_w;
+	}
+
+	prev_w = screen_w;
+	prev_h = screen_h;
+
+	wresize(stdscr, screen_h, screen_w);
+	wresize(menu_win, screen_h - 1, screen_w);
+
+	int border_h = get_working_area_height();
+	int border_y = 1 + get_tabline_height();
+
+	/* TODO: ideally we shouldn't set any colors here (why do we do it?). */
+	ui_set_bg(lborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
+	wresize(lborder, border_h, 1);
+	mvwin(lborder, border_y, 0);
+
+	ui_set_bg(rborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
+	wresize(rborder, border_h, 1);
+	mvwin(rborder, border_y, screen_w - 1);
+
+	/* These need a resize at least after terminal size was zero or they grow and
+	 * produce bad looking effect. */
+	wresize(ltop_line1, 1, 1);
+	wresize(ltop_line2, 1, 1);
+	wresize(rtop_line1, 1, 1);
+	wresize(rtop_line2, 1, 1);
+
+	if(curr_stats.number_of_windows == 1)
+	{
+		only_layout(&lwin, screen_w);
+		only_layout(&rwin, screen_w);
+	}
+	else
+	{
+		if(curr_stats.split == HSPLIT)
+			horizontal_layout(screen_w, screen_h);
+		else
+			vertical_layout(screen_w);
+	}
+
+	correct_size(&lwin);
+	correct_size(&rwin);
+
+	wresize(stat_win, 1, screen_w);
+	(void)ui_stat_reposition(1, 0);
+
+	wresize(job_bar, 1, screen_w);
+
+	update_statusbar_layout();
+
+	curs_set(0);
+}
+
 /* Clears border, possibly by filling it with a pattern (depends on
  * configuration). */
 static void
@@ -846,7 +883,15 @@ change_window(void)
 
 	if(window_shows_dirlist(other_view))
 	{
-		put_inactive_mark(other_view);
+		const col_scheme_t *cs = ui_view_get_cs(other_view);
+		if(cs_is_color_set(&cs->color[OTHER_WIN_COLOR]))
+		{
+			draw_dir_list(other_view);
+		}
+		else
+		{
+			fview_draw_inactive_cursor(other_view);
+		}
 	}
 
 	if(curr_stats.preview.on && !is_dir_list_loaded(curr_view))
@@ -966,7 +1011,8 @@ update_view(view_t *view)
 
 	/* If view displays graphics, we don't want to update it or the image will be
 	 * lost. */
-	if(!view->explore_mode && !(curr_stats.preview.on && view == other_view))
+	if(!view->explore_mode && !view->displays_graphics &&
+			!(curr_stats.preview.on && view == other_view))
 	{
 		update_window_lazy(view->win);
 	}
@@ -1082,78 +1128,66 @@ redraw_lists(void)
 void
 update_attributes(void)
 {
-	int attr;
-
 	if(curr_stats.load_stage < 2)
 		return;
 
-	attr = cfg.cs.color[BORDER_COLOR].attr;
 	if(cfg.side_borders_visible)
 	{
-		wbkgdset(lborder, COLOR_PAIR(cfg.cs.pair[BORDER_COLOR]) | attr);
+		ui_set_bg(lborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 		werase(lborder);
-		wbkgdset(rborder, COLOR_PAIR(cfg.cs.pair[BORDER_COLOR]) | attr);
+		ui_set_bg(rborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 		werase(rborder);
 	}
-	wbkgdset(mborder, COLOR_PAIR(cfg.cs.pair[BORDER_COLOR]) | attr);
+	ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 	werase(mborder);
 
-	wbkgdset(ltop_line1, COLOR_PAIR(cfg.cs.pair[TOP_LINE_COLOR]) |
-			(cfg.cs.color[TOP_LINE_COLOR].attr & A_REVERSE));
-	wattrset(ltop_line1, cfg.cs.color[TOP_LINE_COLOR].attr & ~A_REVERSE);
+	ui_set_bg(ltop_line1, &cfg.cs.color[TOP_LINE_COLOR],
+			cfg.cs.pair[TOP_LINE_COLOR]);
 	werase(ltop_line1);
 
-	wbkgdset(ltop_line2, COLOR_PAIR(cfg.cs.pair[TOP_LINE_COLOR]) |
-			(cfg.cs.color[TOP_LINE_COLOR].attr & A_REVERSE));
-	wattrset(ltop_line2, cfg.cs.color[TOP_LINE_COLOR].attr & ~A_REVERSE);
+	ui_set_bg(ltop_line2, &cfg.cs.color[TOP_LINE_COLOR],
+			cfg.cs.pair[TOP_LINE_COLOR]);
 	werase(ltop_line2);
 
-	wbkgdset(tab_line, COLOR_PAIR(cfg.cs.pair[TAB_LINE_COLOR]) |
-			(cfg.cs.color[TAB_LINE_COLOR].attr & A_REVERSE));
-	wattrset(tab_line, cfg.cs.color[TAB_LINE_COLOR].attr & ~A_REVERSE);
-	werase(tab_line);
-
-	wbkgdset(top_line, COLOR_PAIR(cfg.cs.pair[TOP_LINE_COLOR]) |
-			(cfg.cs.color[TOP_LINE_COLOR].attr & A_REVERSE));
-	wattrset(top_line, cfg.cs.color[TOP_LINE_COLOR].attr & ~A_REVERSE);
+	ui_set_bg(top_line, &cfg.cs.color[TOP_LINE_COLOR],
+			cfg.cs.pair[TOP_LINE_COLOR]);
 	werase(top_line);
 
-	wbkgdset(rtop_line1, COLOR_PAIR(cfg.cs.pair[TOP_LINE_COLOR]) |
-			(cfg.cs.color[TOP_LINE_COLOR].attr & A_REVERSE));
-	wattrset(rtop_line1, cfg.cs.color[TOP_LINE_COLOR].attr & ~A_REVERSE);
+	ui_set_bg(rtop_line1, &cfg.cs.color[TOP_LINE_COLOR],
+			cfg.cs.pair[TOP_LINE_COLOR]);
 	werase(rtop_line1);
 
-	wbkgdset(rtop_line2, COLOR_PAIR(cfg.cs.pair[TOP_LINE_COLOR]) |
-			(cfg.cs.color[TOP_LINE_COLOR].attr & A_REVERSE));
-	wattrset(rtop_line2, cfg.cs.color[TOP_LINE_COLOR].attr & ~A_REVERSE);
+	ui_set_bg(rtop_line2, &cfg.cs.color[TOP_LINE_COLOR],
+			cfg.cs.pair[TOP_LINE_COLOR]);
 	werase(rtop_line2);
 
-	attr = cfg.cs.color[STATUS_LINE_COLOR].attr;
-	wbkgdset(stat_win, COLOR_PAIR(cfg.cs.pair[STATUS_LINE_COLOR]) | attr);
+	ui_set_bg(tab_line, &cfg.cs.color[TAB_LINE_COLOR],
+			cfg.cs.pair[TAB_LINE_COLOR]);
+	werase(tab_line);
 
-	attr = cfg.cs.color[JOB_LINE_COLOR].attr;
-	wbkgdset(job_bar, COLOR_PAIR(cfg.cs.pair[JOB_LINE_COLOR]) | attr);
+	ui_set_bg(stat_win, &cfg.cs.color[STATUS_LINE_COLOR],
+			cfg.cs.pair[STATUS_LINE_COLOR]);
 
-	attr = cfg.cs.color[WIN_COLOR].attr;
-	wbkgdset(menu_win, COLOR_PAIR(cfg.cs.pair[WIN_COLOR]) | attr);
-	wbkgdset(sort_win, COLOR_PAIR(cfg.cs.pair[WIN_COLOR]) | attr);
-	wbkgdset(change_win, COLOR_PAIR(cfg.cs.pair[WIN_COLOR]) | attr);
-	wbkgdset(error_win, COLOR_PAIR(cfg.cs.pair[WIN_COLOR]) | attr);
+	ui_set_bg(job_bar, &cfg.cs.color[JOB_LINE_COLOR],
+			cfg.cs.pair[JOB_LINE_COLOR]);
 
-	wattrset(status_bar, cfg.cs.color[CMD_LINE_COLOR].attr);
-	wbkgdset(status_bar, COLOR_PAIR(cfg.cs.pair[CMD_LINE_COLOR]));
+	ui_set_bg(menu_win, &cfg.cs.color[WIN_COLOR], cfg.cs.pair[WIN_COLOR]);
+	ui_set_bg(sort_win, &cfg.cs.color[WIN_COLOR], cfg.cs.pair[WIN_COLOR]);
+	ui_set_bg(change_win, &cfg.cs.color[WIN_COLOR], cfg.cs.pair[WIN_COLOR]);
+	ui_set_bg(error_win, &cfg.cs.color[WIN_COLOR], cfg.cs.pair[WIN_COLOR]);
 
-	wattrset(ruler_win, cfg.cs.color[CMD_LINE_COLOR].attr);
-	wbkgdset(ruler_win, COLOR_PAIR(cfg.cs.pair[CMD_LINE_COLOR]));
-
-	wattrset(input_win, cfg.cs.color[CMD_LINE_COLOR].attr);
-	wbkgdset(input_win, COLOR_PAIR(cfg.cs.pair[CMD_LINE_COLOR]));
+	ui_set_bg(status_bar, &cfg.cs.color[CMD_LINE_COLOR],
+			cfg.cs.pair[CMD_LINE_COLOR]);
+	ui_set_bg(ruler_win, &cfg.cs.color[CMD_LINE_COLOR],
+			cfg.cs.pair[CMD_LINE_COLOR]);
+	ui_set_bg(input_win, &cfg.cs.color[CMD_LINE_COLOR],
+			cfg.cs.pair[CMD_LINE_COLOR]);
 }
 
 void
 ui_refresh_win(WINDOW *win)
 {
-	if(!curr_stats.silent_ui)
+	if(!stats_silenced_ui())
 	{
 		use_wrefresh(win);
 	}
@@ -1178,11 +1212,17 @@ wprint(WINDOW *win, const char str[])
 }
 
 void
-wprinta(WINDOW *win, const char str[], int line_attrs)
+wprinta(WINDOW *win, const char str[], const cchar_t *line_attrs,
+		int attrs_xors)
 {
-	wattron(win, line_attrs);
+	attr_t attrs;
+	short color_pair;
+	wchar_t wch[getcchar(line_attrs, NULL, &attrs, &color_pair, NULL)];
+	getcchar(line_attrs, wch, &attrs, &color_pair, NULL);
+
+	col_attr_t col = { .attr = attrs ^ attrs_xors };
+	ui_set_attr(win, &col, color_pair);
 	wprint(win, str);
-	wattroff(win, line_attrs);
 	wnoutrefresh(win);
 }
 
@@ -1235,6 +1275,12 @@ ui_setup_for_menu_like(void)
 static void
 update_term_size(void)
 {
+	if(curr_stats.load_stage < 1)
+	{
+		/* No terminal in tests. */
+		return;
+	}
+
 #ifndef _WIN32
 	struct winsize ws = { .ws_col = -1, .ws_row = -1 };
 
@@ -1434,10 +1480,6 @@ ui_swap_view_data(view_t *left, view_t *right)
 	left->window_cols = right->window_cols;
 	right->window_cols = t;
 
-	t = left->local_cs;
-	left->local_cs = right->local_cs;
-	right->local_cs = t;
-
 	tmp = left->title;
 	left->title = right->title;
 	right->title = tmp;
@@ -1612,6 +1654,27 @@ ui_get_decors(const dir_entry_t *entry, const char **prefix,
 	}
 }
 
+void
+ui_view_reset_decor_cache(const view_t *view)
+{
+	int i;
+
+	for(i = 0; i < view->list_rows; ++i)
+	{
+		view->dir_entry[i].name_dec_num = -1;
+	}
+
+	for(i = 0; i < view->left_column.entries.nentries; ++i)
+	{
+		view->left_column.entries.entries[i].name_dec_num = -1;
+	}
+
+	for(i = 0; i < view->right_column.entries.nentries; ++i)
+	{
+		view->right_column.entries.entries[i].name_dec_num = -1;
+	}
+}
+
 /* Gets real type of file view entry.  Returns type of entry, resolving symbolic
  * link if needed. */
 static FileType
@@ -1733,7 +1796,7 @@ ui_view_title_update(view_t *view)
 		return;
 	}
 
-	if(view == selected && cfg.set_title && !curr_stats.silent_ui)
+	if(view == selected && cfg.set_title && !stats_silenced_ui())
 	{
 		char *const term_title = format_view_title(view, pf);
 		term_title_update(term_title);
@@ -1803,10 +1866,7 @@ print_tab_title(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
 	int width_used = 0;
 	int avg_width, spare_width;
 
-	wbkgdset(win, COLOR_PAIR(colmgr_get_pair(base_col.fg, base_col.bg)) |
-			(base_col.attr & A_REVERSE));
-	wattrset(win, base_col.attr & ~A_REVERSE);
-
+	ui_set_bg(win, &base_col, -1);
 	werase(win);
 	checked_wmove(win, 0, 0);
 
@@ -1848,9 +1908,7 @@ print_tab_title(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
 			title = ellipsed;
 		}
 
-		wbkgdset(win, COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) |
-				(col.attr & A_REVERSE));
-		wattrset(win, col.attr & ~A_REVERSE);
+		ui_set_attr(win, &col, -1);
 
 		if(width > extra_width)
 		{
@@ -1929,8 +1987,19 @@ compute_avg_width(int *avg_width, int *spare_width, int max_width, view_t *view,
 static char *
 make_tab_title(const tab_info_t *tab_info, path_func pf)
 {
-	return (tab_info->name == NULL) ? format_view_title(tab_info->view, pf)
-	                                : strdup(tab_info->name);
+	if(tab_info->name != NULL)
+	{
+		return strdup(tab_info->name);
+	}
+
+	if(cfg.tail_tab_line_paths)
+	{
+		/* We can just do the replacement, because shortening home part doesn't make
+		 * sense for a single path entry. */
+		pf = &get_last_path_component;
+	}
+
+	return format_view_title(tab_info->view, pf);
 }
 
 /* Formats title for the view.  The pf function will be applied to full paths.
@@ -2005,19 +2074,15 @@ fixup_titles_attributes(const view_t *view, int active_view)
 	{
 		cs_mix_colors(&col, &cfg.cs.color[TOP_LINE_SEL_COLOR]);
 
-		wbkgdset(view->title, COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) |
-				(col.attr & A_REVERSE));
-		wattrset(view->title, col.attr & ~A_REVERSE);
+		ui_set_bg(view->title, &col, -1);
+		ui_set_attr(view->title, &col, -1);
 	}
 	else
 	{
-		const int bg_attr = COLOR_PAIR(cfg.cs.pair[TOP_LINE_COLOR])
-		                  | (col.attr & A_REVERSE);
+		ui_set_bg(view->title, &col, cfg.cs.pair[TOP_LINE_COLOR]);
+		ui_set_attr(view->title, &col, cfg.cs.pair[TOP_LINE_COLOR]);
 
-		wbkgdset(view->title, bg_attr);
-		wattrset(view->title, col.attr & ~A_REVERSE);
-		wbkgdset(top_line, bg_attr);
-		wattrset(top_line, col.attr & ~A_REVERSE);
+		ui_set_bg(top_line, &col, cfg.cs.pair[TOP_LINE_COLOR]);
 		werase(top_line);
 	}
 
@@ -2127,7 +2192,8 @@ ui_view_right_reserved(const view_t *view)
 	const int total = view->miller_ratios[0] + view->miller_ratios[1]
 	                + view->miller_ratios[2];
 	return is_in_miller_view(view)
-	    && fentry_is_dir(entry) && !is_parent_dir(entry->name)
+	    && !is_parent_dir(entry->name)
+	    && (fentry_is_dir(entry) || view->miller_preview_files)
 	     ? (view->window_cols*view->miller_ratios[2])/total
 	     : 0;
 }
@@ -2153,25 +2219,52 @@ is_forced_list_mode(const view_t *view)
 int
 ui_qv_left(const view_t *view)
 {
-	return cfg.extra_padding ? 1 : 0;
+	const int with_margin = (!curr_stats.preview.clearing && cfg.extra_padding);
+	return with_margin ? 1 : 0;
 }
 
 int
 ui_qv_top(const view_t *view)
 {
-	return cfg.extra_padding ? 1 : 0;
+	const int with_margin = (!curr_stats.preview.clearing && cfg.extra_padding);
+	return with_margin ? 1 : 0;
 }
 
 int
 ui_qv_height(const view_t *view)
 {
-	return cfg.extra_padding ? view->window_rows - 2 : view->window_rows;
+	const int with_margin = (!curr_stats.preview.clearing && cfg.extra_padding);
+	return with_margin ? view->window_rows - 2 : view->window_rows;
 }
 
 int
 ui_qv_width(const view_t *view)
 {
-	return cfg.extra_padding ? view->window_cols - 2 : view->window_cols;
+	const int with_margin = (!curr_stats.preview.clearing && cfg.extra_padding);
+	return with_margin ? view->window_cols - 2 : view->window_cols;
+}
+
+void
+ui_invalidate_cs(const col_scheme_t *cs)
+{
+	int i;
+	tab_info_t tab_info;
+
+	for(i = 0; tabs_enum(curr_view, i, &tab_info); ++i)
+	{
+		if(ui_view_get_cs(tab_info.view) == cs)
+		{
+			fview_reset_cs(tab_info.view);
+		}
+	}
+
+	for(i = 0; tabs_enum(other_view, i, &tab_info); ++i)
+	{
+		if(ui_view_get_cs(tab_info.view) == cs)
+		{
+			fview_reset_cs(tab_info.view);
+		}
+	}
 }
 
 const col_scheme_t *
@@ -2184,34 +2277,20 @@ void
 ui_view_erase(view_t *view)
 {
 	const col_scheme_t *cs = ui_view_get_cs(view);
-	const int bg = COLOR_PAIR(cs->pair[WIN_COLOR]) | cs->color[WIN_COLOR].attr;
-	wbkgdset(view->win, bg);
+	col_attr_t col = ui_get_win_color(view, cs);
+	ui_set_bg(view->win, &col, -1);
 	werase(view->win);
 }
 
-void
-ui_view_wipe(view_t *view)
+col_attr_t
+ui_get_win_color(const view_t *view, const col_scheme_t *cs)
 {
-	int i;
-	int height;
-	short int fg, bg;
-	char line_filler[getmaxx(view->win) + 1];
-
-	line_filler[sizeof(line_filler) - 1] = '\0';
-	height = getmaxy(view->win);
-
-	/* User doesn't need to see fake filling so draw it with the color of
-	 * background. */
-	(void)pair_content(PAIR_NUMBER(getbkgd(view->win)), &fg, &bg);
-	wattrset(view->win, COLOR_PAIR(colmgr_get_pair(bg, bg)));
-
-	memset(line_filler, '\t', sizeof(line_filler));
-	for(i = 0; i < height; ++i)
+	col_attr_t col = cs->color[WIN_COLOR];
+	if(view == other_view && window_shows_dirlist(view))
 	{
-		mvwaddstr(view->win, i, 0, line_filler);
+		cs_mix_colors(&col, &cs->color[OTHER_WIN_COLOR]);
 	}
-	redrawwin(view->win);
-	ui_refresh_win(view->win);
+	return col;
 }
 
 int
@@ -2230,11 +2309,102 @@ ui_shutdown(void)
 	}
 }
 
+void
+ui_pause(void)
+{
+	/* Show previous screen state. */
+	ui_shutdown();
+	/* Yet restore program mode to read input without waiting for Enter. */
+	reset_prog_mode();
+	/* Refresh the window, because otherwise curses redraws the screen on call to
+	 * `compat_wget_wch()` (why does it do this?). */
+	wnoutrefresh(inf_delay_window);
+
+	/* Ignore window resize. */
+	wint_t pressed;
+	do
+	{
+		/* Nothing. */
+	}
+	while(compat_wget_wch(inf_delay_window, &pressed) != ERR &&
+			pressed == KEY_RESIZE);
+
+	/* Redraw UI to account for all things including graphical preview. */
+	curr_stats.need_update = UT_REDRAW;
+}
+
+void
+ui_set_bg(WINDOW *win, const col_attr_t *col, int pair)
+{
+	if(curr_stats.load_stage < 1)
+	{
+		return;
+	}
+
+	if(pair < 0)
+	{
+		pair = colmgr_get_pair(col->fg, col->bg);
+	}
+
+	cchar_t bg;
+	setcchar(&bg, L" ", col->attr, pair, NULL);
+	wbkgrndset(win, &bg);
+}
+
+void
+ui_set_attr(WINDOW *win, const col_attr_t *col, int pair)
+{
+	/* win can be NULL in tests. */
+	if(curr_stats.load_stage < 1 || win == NULL)
+	{
+		return;
+	}
+
+	if(pair < 0)
+	{
+		pair = colmgr_get_pair(col->fg, col->bg);
+	}
+
+	/* Compiler complains about unused result of comma operator, because
+	 * wattr_set() is a macro and it uses comma to evaluate multiple expresions.
+	 * So cast result to void.*/
+	(void)wattr_set(win, col->attr, pair, NULL);
+}
+
+void
+ui_drop_attr(WINDOW *win)
+{
+	if(curr_stats.load_stage >= 1)
+	{
+		wattrset(win, 0);
+	}
+}
+
+void
+ui_pass_through(const strlist_t *lines, WINDOW *win, int x, int y)
+{
+	/* Position hardware cursor on the screen. */
+	checked_wmove(win, y, x);
+	use_wrefresh(win);
+
+	int i;
+	for(i = 0; i < lines->nitems; ++i)
+	{
+		puts(lines->items[i]);
+	}
+
+	/* Make curses synchronize its idea about where cursor is with the terminal
+	 * state. */
+	touchwin(status_bar);
+	checked_wmove(status_bar, 0, 0);
+	use_wrefresh(status_bar);
+}
+
 void
 ui_view_schedule_redraw(view_t *view)
 {
 	pthread_mutex_lock(view->timestamps_mutex);
-	view->postponed_redraw = get_updated_time(view->postponed_redraw);
+	view->need_redraw = 1;
 	pthread_mutex_unlock(view->timestamps_mutex);
 }
 
@@ -2242,34 +2412,15 @@ void
 ui_view_schedule_reload(view_t *view)
 {
 	pthread_mutex_lock(view->timestamps_mutex);
-	view->postponed_reload = get_updated_time(view->postponed_reload);
+	view->need_reload = 1;
 	pthread_mutex_unlock(view->timestamps_mutex);
 }
 
-/* Gets updated timestamp ensuring that it differs from the previous value.
- * Returns the timestamp. */
-static uint64_t
-get_updated_time(uint64_t prev)
-{
-	struct timeval tv = {0};
-	uint64_t new;
-
-	(void)gettimeofday(&tv, NULL);
-
-	new = tv.tv_sec*1000000ULL + tv.tv_usec;
-	if(new == prev)
-	{
-		++new;
-	}
-
-	return new;
-}
-
 void
 ui_view_redrawn(view_t *view)
 {
 	pthread_mutex_lock(view->timestamps_mutex);
-	view->last_redraw = view->postponed_redraw;
+	view->need_redraw = 0;
 	pthread_mutex_unlock(view->timestamps_mutex);
 }
 
@@ -2280,11 +2431,11 @@ ui_view_query_scheduled_event(view_t *view)
 
 	pthread_mutex_lock(view->timestamps_mutex);
 
-	if(view->postponed_reload != view->last_reload)
+	if(view->need_reload)
 	{
 		event = UUE_RELOAD;
 	}
-	else if(view->postponed_redraw != view->last_redraw)
+	else if(view->need_redraw)
 	{
 		event = UUE_REDRAW;
 	}
@@ -2293,8 +2444,8 @@ ui_view_query_scheduled_event(view_t *view)
 		event = UUE_NONE;
 	}
 
-	view->last_redraw = view->postponed_redraw;
-	view->last_reload = view->postponed_reload;
+	view->need_redraw = 0;
+	view->need_reload = 0;
 
 	pthread_mutex_unlock(view->timestamps_mutex);
 
