@@ -38,6 +38,7 @@
 #include "../status.h"
 #include "../types.h"
 #include "color_scheme.h"
+#include "colors.h"
 
 #define SORT_WIN_WIDTH 32
 
@@ -120,8 +121,11 @@ enum
 	/* Defined here to do not add it to sorting keys. */
 	/* Id ordering. */
 	SK_BY_ID = SK_NONE + 1,
-	/* Displaying root of file name only. */
+	/* Displaying only root of file name. */
 	SK_BY_ROOT,
+	/* Displaying only root of file names for entries that aren't directories or
+	 * symbolic links to directories. */
+	SK_BY_FILEROOT,
 
 	/* Total number of SK_*, including those missing from sorting keys and other
 	 * counts. */
@@ -206,7 +210,6 @@ struct dir_entry_t
 	time_t mtime;     /* Modification time. */
 	time_t atime;     /* Access time. */
 	time_t ctime;     /* Creation time. */
-	FileType type;    /* File type. */
 	int nlinks;       /* Number of hard links to the entry. */
 
 	int id;           /* File uniqueness identifier on comparison. */
@@ -215,7 +218,8 @@ struct dir_entry_t
 	                     e.g. by sorting comparer to perform stable sort or item
 	                     mapping during tree filtering. */
 
-	int hi_num;       /* File highlighting parameters cache (initially -1). */
+	int hi_num;       /* File highlighting parameters cache.  Initially -1.
+	                     INT_MAX signifies absence of a match. */
 	int name_dec_num; /* File decoration parameters cache (initially -1).  The
 	                     value is shifted by one, 0 means type decoration. */
 
@@ -228,6 +232,7 @@ struct dir_entry_t
 	short int match_left;  /* Starting position of search match. */
 	short int match_right; /* Ending position of search match. */
 
+	FileType type : 4;             /* File type. */
 	unsigned int selected : 1;     /* Whether file is selected. */
 	unsigned int was_selected : 1; /* Previous selection state for Visual mode. */
 	unsigned int marked : 1;       /* Whether file should be processed. */
@@ -338,6 +343,8 @@ struct view_t
 	int miller_view, miller_view_g;
 	/* Proportions of columns. */
 	int miller_ratios[3], miller_ratios_g[3];
+	/* Whether right column should also preview files. */
+	int miller_preview_files, miller_preview_files_g;
 	/* Caches of file lists for miller mode. */
 	cached_entries_t left_column;
 	cached_entries_t right_column;
@@ -364,9 +371,13 @@ struct view_t
 	int window_cols; /* Number of columns in the window. */
 	int filtered;  /* number of files filtered out and not shown in list */
 	int selected_files; /* Number of currently selected files. */
-	int local_cs; /* Whether directory-specific color scheme is in use. */
 	dir_entry_t *dir_entry; /* Must be handled via dynarray unit. */
 
+	/* Last position that was displayed on the screen. */
+	char *last_curr_file; /* To account for file replacement. */
+	int last_seen_pos;    /* To account for movement. */
+	int last_curr_line;   /* To account for scrolling. */
+
 	int nsaved_selection;   /* Number of items in saved_selection. */
 	char **saved_selection; /* Names of selected files. */
 
@@ -401,7 +412,8 @@ struct view_t
 	int history_pos;    /* Current position in history. */
 	history_t *history; /* Directory history itself. */
 
-	col_scheme_t cs;
+	int local_cs;    /* Whether directory-specific color scheme is in use. */
+	col_scheme_t cs; /* Storage of local (tree-specific) color scheme. */
 
 	/* Handle for column_view unit.  Contains view columns configuration even when
 	 * 'lsview' is on. */
@@ -430,22 +442,19 @@ struct view_t
 	int num_width, num_width_g;
 	int real_num_width; /* Real character count reserved for number field. */
 
-	/* Timestamps for controlling of scheduling update requests.  They are in
-	 * microseconds.  Real resolution is bigger than microsecond, but it's not
-	 * critical. */
-
-	uint64_t postponed_redraw;         /* Time of last redraw request. */
-	uint64_t postponed_reload;         /* Time of last reload request. */
+	int need_redraw;                   /* Whether view should be redrawn. */
+	int need_reload;                   /* Whether view should be reloaded. */
 	pthread_mutex_t *timestamps_mutex; /* Protects access to above variables.
 	                                      This is a pointer, because mutexes
-	                                      shouldn't be copied*/
-
-	uint64_t last_redraw; /* Time of last redraw. */
-	uint64_t last_reload; /* Time of last [full] reload. */
+	                                      shouldn't be copied. */
 
 	int on_slow_fs; /* Whether current directory has access penalties. */
 	int has_dups;   /* Whether current directory has duplicated file entries (FS
 	                   issue). */
+
+	int location_changed; /* Whether location was recently changed. */
+
+	int displays_graphics; /* Whether window of the view contains graphics. */
 };
 
 extern view_t lwin;
@@ -481,9 +490,13 @@ void ui_ruler_set(const char val[]);
  * term_state in status structure. */
 void ui_update_term_state(void);
 
-/* Checks whether given character was pressed, ignores any other characters. */
+/* Checks whether given character was pressed discarding any other characters
+ * from the input stream. */
 int ui_char_pressed(wint_t c);
 
+/* Reads buffered input until it's empty. */
+void ui_drain_input(void);
+
 int setup_ncurses_interface(void);
 
 /* Closes current tab if it's not the last one, closes whole application
@@ -542,9 +555,10 @@ void ui_refresh_win(WINDOW *win);
 /* Prints str in current window position. */
 void wprint(WINDOW *win, const char str[]);
 
-/* Prints str in current window position with specified line attributes, which
- * set during print operation only. */
-void wprinta(WINDOW *win, const char str[], int line_attrs);
+/* Prints str in current window position with specified line attributes.  Value
+ * of attrs_xors is xored with attributes of line_attrs. */
+void wprinta(WINDOW *win, const char str[], const cchar_t *line_attrs,
+		int attrs_xors);
 
 /* Performs resizing of some of TUI elements for menu like modes.  Returns zero
  * on success, and non-zero otherwise. */
@@ -601,6 +615,9 @@ void format_entry_name(const dir_entry_t *entry, NameFormat fmt, size_t buf_len,
 void ui_get_decors(const dir_entry_t *entry, const char **prefix,
 		const char **suffix);
 
+/* Resets cached indexes for name-dependent type_decs. */
+void ui_view_reset_decor_cache(const view_t *view);
+
 /* Moves cursor to position specified by coordinates checking result of the
  * movement. */
 void checked_wmove(WINDOW *win, int y, int x);
@@ -680,6 +697,9 @@ int ui_qv_height(const view_t *view);
 /* Retrieves width of quickview area.  Returns the width. */
 int ui_qv_width(const view_t *view);
 
+/* Invalidates views-specific knowledge about given color scheme. */
+void ui_invalidate_cs(const col_scheme_t *cs);
+
 /* Gets color scheme that corresponds to the view.  Returns pointer to the color
  * scheme. */
 const col_scheme_t * ui_view_get_cs(const view_t *view);
@@ -687,9 +707,8 @@ const col_scheme_t * ui_view_get_cs(const view_t *view);
 /* Erases view window by filling it with the background color. */
 void ui_view_erase(view_t *view);
 
-/* Same as erase, but ensures that view is updated in all its size on the
- * screen (e.g. to clear anything put there by other programs as well). */
-void ui_view_wipe(view_t *view);
+/* Figures out base color for the pane.  Returns the color. */
+col_attr_t ui_get_win_color(const view_t *view, const col_scheme_t *cs);
 
 /* Checks whether custom view type of specified view is unsorted.  It doesn't
  * need the view to be custom, checks just the type.  Returns non-zero if so,
@@ -700,6 +719,28 @@ int ui_view_unsorted(const view_t *view);
  * or when terminal might be used by another application that vifm runs). */
 void ui_shutdown(void);
 
+/* Temporarily shuts down UI until a key is pressed. */
+void ui_pause(void);
+
+/* Sets background color of the window.  Allocates pair if passed in pair number
+ * is negative.  When pair is passed in, only attribute part of the col
+ * parameter is used. */
+void ui_set_bg(WINDOW *win, const col_attr_t *col, int pair);
+
+/* Sets current color of the window.  Allocates pair if passed in pair number is
+ * negative.  When pair is passed in, only attribute part of the col parameter
+ * is used. */
+void ui_set_attr(WINDOW *win, const col_attr_t *col, int pair);
+
+/* Clears attribute of the window. */
+void ui_drop_attr(WINDOW *win);
+
+struct strlist_t;
+
+/* Outputs lines into terminal circumventing curses.  Positions cursor at the
+ * specified point on the window beforehand. */
+void ui_pass_through(const struct strlist_t *lines, WINDOW *win, int x, int y);
+
 /* View update scheduling. */
 
 /* Schedules redraw of the view for the future.  Doesn't perform any actual
