@@ -37,7 +37,7 @@
 #include <errno.h> /* errno */
 #include <stddef.h> /* NULL */
 #include <stdio.h> /* snprintf() remove() */
-#include <stdlib.h> /* free() qsort() */
+#include <stdlib.h> /* free() */
 #include <string.h> /* strcpy() strdup() strlen() strncmp() strncpy() */
 
 #include "../compat/dtype.h"
@@ -53,7 +53,6 @@
 static int is_dir_fast(const char path[]);
 static int path_exists_internal(const char path[], const char filename[],
 		int deref);
-static int path_sorter(const void *first, const void *second);
 
 #ifndef _WIN32
 static int is_directory(const char path[], int dereference_links);
@@ -169,7 +168,7 @@ path_exists_at(const char path[], const char filename[], int deref)
 static int
 path_exists_internal(const char path[], const char filename[], int deref)
 {
-	char full[PATH_MAX];
+	char full[PATH_MAX + 1];
 	if(path == NULL)
 	{
 		copy_str(full, sizeof(full), filename);
@@ -197,8 +196,8 @@ path_exists_internal(const char path[], const char filename[], int deref)
 int
 paths_are_same(const char s[], const char t[])
 {
-	char s_real[PATH_MAX];
-	char t_real[PATH_MAX];
+	char s_real[PATH_MAX + 1];
+	char t_real[PATH_MAX + 1];
 
 	if(os_realpath(s, s_real) != s_real || os_realpath(t, t_real) != t_real)
 	{
@@ -214,7 +213,7 @@ is_symlink(const char path[])
 	struct stat st;
 	return os_lstat(path, &st) == 0 && S_ISLNK(st.st_mode);
 #else
-	char filename[PATH_MAX];
+	char filename[PATH_MAX + 1];
 	DWORD attr;
 	wchar_t *utf16_filename;
 	HANDLE hfind;
@@ -257,7 +256,7 @@ is_symlink(const char path[])
 SymLinkType
 get_symlink_type(const char path[])
 {
-	char cwd[PATH_MAX];
+	char cwd[PATH_MAX + 1];
 	char linkto[PATH_MAX + NAME_MAX];
 	int saved_errno;
 	char *filename_copy;
@@ -305,7 +304,7 @@ int
 get_link_target_abs(const char link[], const char cwd[], char buf[],
 		size_t buf_len)
 {
-	char link_target[PATH_MAX];
+	char link_target[PATH_MAX + 1];
 	if(get_link_target(link, link_target, sizeof(link_target)) != 0)
 	{
 		return 1;
@@ -349,7 +348,7 @@ get_link_target(const char *link, char *buf, size_t buf_len)
 	buf[len] = '\0';
 	return 0;
 #else
-	char filename[PATH_MAX];
+	char filename[PATH_MAX + 1];
 	DWORD attr;
 	wchar_t *utf16_filename;
 	HANDLE hfile;
@@ -399,7 +398,7 @@ get_link_target(const char *link, char *buf, size_t buf_len)
 		strncpy(buf, t, buf_len);
 	buf[buf_len - 1] = '\0';
 	free(t);
-	to_forward_slash(buf);
+	system_to_internal_slashes(buf);
 	return 0;
 #endif
 }
@@ -527,7 +526,7 @@ list_regular_files(const char path[], char *list[], int *len)
 
 	while((d = os_readdir(dir)) != NULL)
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		snprintf(full_path, sizeof(full_path), "%s/%s", path, d->d_name);
 
 		if(is_regular_file(full_path))
@@ -573,26 +572,45 @@ list_sorted_files(const char path[], int *len)
 	char **const list = list_all_files(path, len);
 	if(*len > 0)
 	{
-		qsort(list, *len, sizeof(*list), &path_sorter);
+		/* The check above is needed because *len might be negative. */
+		safe_qsort(list, *len, sizeof(*list), &strossorter);
 	}
 	return list;
 }
 
-/* Wraps stroscmp() for use with qsort(). */
-static int
-path_sorter(const void *first, const void *second)
-{
-	const char *const *const a = first;
-	const char *const *const b = second;
-	return stroscmp(*a, *b);
-}
-
 int
 is_regular_file(const char path[])
+{
+	char resolved_link[PATH_MAX + 1];
+	if(is_symlink(path))
+	{
+		char *const symlink_base = strdup(path);
+
+		if(!is_root_dir(symlink_base))
+		{
+			remove_last_path_component(symlink_base);
+		}
+
+		if(get_link_target_abs(path, symlink_base, resolved_link,
+					sizeof(resolved_link)) != 0)
+		{
+			free(symlink_base);
+			return 0;
+		}
+		free(symlink_base);
+
+		path = resolved_link;
+	}
+
+	return is_regular_file_noderef(path);
+}
+
+int
+is_regular_file_noderef(const char path[])
 {
 #ifndef _WIN32
 	struct stat s;
-	return os_stat(path, &s) == 0 && (s.st_mode & S_IFMT) == S_IFREG;
+	return os_lstat(path, &s) == 0 && (s.st_mode & S_IFMT) == S_IFREG;
 #else
 	const DWORD attrs = win_get_file_attrs(path);
 	if(attrs == INVALID_FILE_ATTRIBUTES)
@@ -636,6 +654,9 @@ remove_dir_content(const char path[])
 			char *const full_path = format_str("%s/%s", path, d->d_name);
 			if(entry_is_dir(full_path, d))
 			{
+				/* Attempt to make sure that we can change the directory we are
+				 * descending into. */
+				(void)os_chmod(full_path, 0777);
 				remove_dir_content(full_path);
 			}
 			(void)remove(full_path);
@@ -692,20 +713,25 @@ is_dirent_targets_dir(const char full_path[], const struct dirent *d)
 }
 
 int
-is_in_subtree(const char path[], const char root[])
+is_in_subtree(const char path[], const char root[], int include_root)
 {
-	char path_copy[PATH_MAX];
-	char path_real[PATH_MAX];
-	char root_real[PATH_MAX];
+	/* This variable must remain on this level because it can be being pointed
+	 * to. */
+	char path_copy[PATH_MAX + 1];
+	if(!include_root)
+	{
+		copy_str(path_copy, sizeof(path_copy), path);
+		remove_last_path_component(path_copy);
+		path = path_copy;
+	}
 
-	copy_str(path_copy, sizeof(path_copy), path);
-	remove_last_path_component(path_copy);
-
-	if(os_realpath(path_copy, path_real) != path_real)
+	char path_real[PATH_MAX + 1];
+	if(os_realpath(path, path_real) != path_real)
 	{
 		return 0;
 	}
 
+	char root_real[PATH_MAX + 1];
 	if(os_realpath(root, root_real) != root_real)
 	{
 		return 0;
@@ -772,7 +798,7 @@ enum_dir_content(const char path[], dir_content_client_func client, void *param)
 
 	return 0;
 #else
-	char find_pat[PATH_MAX];
+	char find_pat[PATH_MAX + 1];
 	wchar_t *utf16_path;
 	HANDLE hfind;
 	WIN32_FIND_DATAW ffd;
@@ -880,8 +906,9 @@ is_directory(const char path[], int dereference_links)
 
 #else
 
-/* Obtains attributes of a file.  Skips check for unmounted disks.  Returns the
- * attributes, which is INVALID_FILE_ATTRIBUTES on error. */
+/* Obtains attributes of a file.  Skips check for unmounted disks.  Doesn't
+ * dereference symbolic links.  Returns the attributes, which is
+ * INVALID_FILE_ATTRIBUTES on error. */
 static DWORD
 win_get_file_attrs(const char path[])
 {
