@@ -218,7 +218,7 @@ update_file_stats(fswatch_t *w, const struct inotify_event *e, time_t now)
 	}
 
 	/* Treat events happened in the next second as a sequence. */
-	stats->count = (now - stats->last_update <= 1U) ? (stats->count + 1) : 1;
+	stats->count = (now - stats->last_update <= 1) ? (stats->count + 1) : 1;
 
 	/* Files that cause relatively long sequence of events are banned for a
 	 * while.  Don't ban the directory itself, we don't want to miss changes of
@@ -256,7 +256,7 @@ fswatch_create(const char path[])
 		return NULL;
 	}
 
-	if(filemon_from_file(path, &w->filemon) != 0)
+	if(filemon_from_file(path, FMT_MODIFIED, &w->filemon) != 0)
 	{
 		free(w);
 		return NULL;
@@ -288,7 +288,7 @@ fswatch_changed(fswatch_t *w, int *error)
 	int changed;
 
 	filemon_t filemon;
-	if(filemon_from_file(w->path, &filemon) != 0)
+	if(filemon_from_file(w->path, FMT_MODIFIED, &filemon) != 0)
 	{
 		*error = 1;
 		return 1;
