@@ -196,15 +196,20 @@ surrounded_with(const char str[], char left, char right)
 }
 
 char *
-to_multibyte(const wchar_t *s)
+to_multibyte(const wchar_t s[])
 {
 #ifndef _WIN32
-	size_t len;
-	char *result;
-
-	len = wcstombs(NULL, s, 0) + 1;
-	if((result = malloc(len)) == NULL)
+	const size_t len = wcstombs(NULL, s, 0) + 1;
+	if(len == 0U)
+	{
 		return NULL;
+	}
+
+	char *const result = malloc(len);
+	if(result == NULL)
+	{
+		return NULL;
+	}
 
 	wcstombs(result, s, len);
 	return result;
@@ -792,7 +797,9 @@ get_first_wchar(const char str[])
 {
 #ifndef _WIN32
 	wchar_t wc[2] = {};
-	return (mbstowcs(wc, str, ARRAY_LEN(wc)) >= 1) ? wc[0] : str[0];
+	return (mbstowcs(wc, str, ARRAY_LEN(wc)) == (size_t)-1)
+	     ? (unsigned char)str[0]
+	     : wc[0];
 #else
 	return utf8_first_char(str);
 #endif
