@@ -83,7 +83,7 @@ ARRAY_GUARD(iec_units, ARRAY_LEN(iec_i_units));
 ARRAY_GUARD(si_units, ARRAY_LEN(iec_units));
 
 int
-vifm_system(char command[])
+vifm_system(char command[], ShellRequester by)
 {
 #ifdef _WIN32
 	/* The check is primarily for tests, otherwise screen is reset. */
@@ -93,7 +93,7 @@ vifm_system(char command[])
 	}
 #endif
 	LOG_INFO_MSG("Shell command: %s", command);
-	return run_in_shell_no_cls(command);
+	return run_in_shell_no_cls(command, by);
 }
 
 int
@@ -253,12 +253,13 @@ friendly_size_notation(uint64_t num, int str_size, char str[])
 
 	if(fraction == 0)
 	{
-		snprintf(str, str_size, "%.0f %s", d, units[u]);
+		snprintf(str, str_size, "%.0f%s%s", d, cfg.sizefmt.space ? " " : "",
+				units[u]);
 	}
 	else
 	{
-		snprintf(str, str_size, "%.0f.%0*" PRINTF_ULL " %s", d, fraction_width, fraction,
-				units[u]);
+		snprintf(str, str_size, "%.0f.%0*" PRINTF_ULL "%s%s", d, fraction_width,
+				fraction, cfg.sizefmt.space ? " " : "", units[u]);
 	}
 
 	return u > 0U;
@@ -277,7 +278,8 @@ get_size_suffixes(void)
  * *fraction_width.  Returns new value for the size (truncated and/or
  * rounded). */
 static double
-split_size_double(double size, unsigned long long *ifraction, int *fraction_width)
+split_size_double(double size, unsigned long long *ifraction,
+		int *fraction_width)
 {
 	double ten_power, dfraction, integer;
 
@@ -716,14 +718,6 @@ is_line_spec(const char str[])
 	return (endptr != str && errno == 0 && *endptr == ':');
 }
 
-int
-is_graphical_viewer(const char viewer[])
-{
-	/* %pw and %ph can be useful for text output, but %px and %py are useful
-	 * for graphics and basically must have both. */
-	return (strstr(viewer, "%px") != NULL && strstr(viewer, "%py") != NULL);
-}
-
 void
 safe_qsort(void *base, size_t nmemb, size_t size,
 		int (*compar)(const void *, const void *))
