@@ -26,11 +26,14 @@
 #include <sys/user.h>
 #endif
 
+#include <sys/types.h> /* gid_t mode_t pid_t uid_t */
+#ifdef HAVE_XATTRS
+#include <sys/xattr.h> /* XATTR_CREATE getxattr() listxattr() setxattr() */
+#endif
 #include <sys/select.h> /* select() FD_SET FD_ZERO */
 #include <sys/stat.h> /* O_* S_* */
 #include <sys/statvfs.h> /* statvfs statvfs() */
 #include <sys/time.h> /* timeval futimens() utimes() */
-#include <sys/types.h> /* gid_t mode_t pid_t uid_t */
 #include <sys/wait.h> /* waitpid */
 #include <fcntl.h> /* open() close() */
 #include <grp.h> /* getgrnam() getgrgid_r() */
@@ -41,11 +44,11 @@
 
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
-#include <errno.h> /* EINTR errno */
+#include <errno.h> /* EINTR ENOTSUP errno */
 #include <signal.h> /* SIG* SIG_* sigset_t kill() sigaddset() sigemptyset()
                        sigfillset() signal() sigprocmask() */
 #include <stddef.h> /* NULL size_t */
-#include <stdio.h> /* FILE stderr fdopen() fprintf() snprintf() */
+#include <stdio.h> /* FILE stderr fclose() fdopen() fprintf() snprintf() */
 #include <stdlib.h> /* atoi() free() */
 #include <string.h> /* strchr() strdup() strerror() strlen() strncmp() */
 
@@ -54,6 +57,7 @@
 #include "../compat/mntent.h" /* mntent setmntent() getmntent() endmntent() */
 #include "../compat/os.h"
 #include "../compat/reallocarray.h"
+#include "../ui/tabs.h"
 #include "../ui/ui.h"
 #include "../running.h"
 #include "../status.h"
@@ -95,15 +99,18 @@ static void free_mnt_entry(struct mntent *entry);
 static int starts_with_list_item(const char str[], const char list[]);
 static int find_path_prefix_index(const char path[], const char list[]);
 static int open_tty(void);
+static void clone_timestamps(const char path[], const char from[],
+		const struct stat *st);
+static void clone_xattrs(const char path[], const char from[]);
 
 void
 pause_shell(void)
 {
-	run_in_shell_no_cls(PAUSE_CMD);
+	run_in_shell_no_cls(PAUSE_CMD, SHELL_BY_APP);
 }
 
 int
-run_in_shell_no_cls(char command[])
+run_in_shell_no_cls(char command[], ShellRequester by)
 {
 	int pid;
 	int result;
@@ -136,8 +143,10 @@ run_in_shell_no_cls(char command[])
 		signal(SIGINT, SIG_DFL);
 		(void)set_sigchld(0);
 
-		execve(get_execv_path(cfg.shell), make_execv_array(cfg.shell, command),
-				environ);
+		prepare_for_exec();
+		char *sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "-c");
+		execve(get_execv_path(cfg.shell),
+				make_execv_array(cfg.shell, sh_flag, command), environ);
 		_Exit(127);
 	}
 
@@ -233,7 +242,8 @@ get_proc_exit_status(pid_t pid)
 }
 
 void _gnuc_noreturn
-run_from_fork(int pipe[2], int err_only, int preserve_stdin, char cmd[])
+run_from_fork(int pipe[2], int err_only, int preserve_stdin, char cmd[],
+		ShellRequester by)
 {
 	/* Close read end of the pipe. */
 	(void)close(pipe[0]);
@@ -274,10 +284,40 @@ run_from_fork(int pipe[2], int err_only, int preserve_stdin, char cmd[])
 		(void)close(null_fd);
 	}
 
-	execvp(get_execv_path(cfg.shell), make_execv_array(cfg.shell, cmd));
+	prepare_for_exec();
+	char *sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "-c");
+	execvp(get_execv_path(cfg.shell), make_execv_array(cfg.shell, sh_flag, cmd));
 	_Exit(127);
 }
 
+void
+prepare_for_exec(void)
+{
+	if(curr_stats.original_stdout != NULL)
+	{
+		(void)fclose(curr_stats.original_stdout);
+	}
+
+	/* More generic cleanup functions aren't called here to do not waste time on
+	 * freeing resources which will be replaced by exec(). */
+	tab_info_t tab_info;
+	int i;
+	for(i = 0; tabs_enum(&lwin, i, &tab_info); ++i)
+	{
+		view_t *view = tab_info.view;
+		fswatch_free(view->watch);
+		fswatch_free(view->left_column.watch);
+		fswatch_free(view->right_column.watch);
+	}
+	for(i = 0; tabs_enum(&rwin, i, &tab_info); ++i)
+	{
+		view_t *view = tab_info.view;
+		fswatch_free(view->watch);
+		fswatch_free(view->left_column.watch);
+		fswatch_free(view->right_column.watch);
+	}
+}
+
 char *
 get_execv_path(char shell[])
 {
@@ -287,14 +327,14 @@ get_execv_path(char shell[])
 }
 
 char **
-make_execv_array(char shell[], char cmd[])
+make_execv_array(char shell[], char shell_flag[], char cmd[])
 {
 #ifdef HAVE_MAX_ARG_STRLEN
 #ifndef PAGE_SIZE
 #define PAGE_SIZE sysconf(_SC_PAGESIZE)
 #endif
 	/* Don't use maximum length, leave some room or commands fail to run. */
-	const size_t safe_arg_len = MIN(MAX_ARG_STRLEN, MAX_ARG_STRLEN - 4096U);
+	const size_t safe_arg_len = MIN(MAX_ARG_STRLEN, MAX_ARG_STRLEN - 4096);
 	const size_t npieces = DIV_ROUND_UP(strlen(cmd), safe_arg_len);
 #else
 	/* Actual value doesn't matter in this case. */
@@ -323,7 +363,7 @@ make_execv_array(char shell[], char cmd[])
 		{
 			args[i++] = sh_arg;
 		}
-		args[i++] = "-c";
+		args[i++] = shell_flag;
 		args[i++] = cmd;
 		args[i++] = NULL;
 		return args;
@@ -358,7 +398,7 @@ make_execv_array(char shell[], char cmd[])
 	{
 		args[i++] = sh_arg;
 	}
-	args[i++] = "-c";
+	args[i++] = shell_flag;
 	args[i++] = eval_cmd;
 	args[i++ + npieces] = NULL;
 
@@ -508,7 +548,7 @@ traverse_mount_points(mptraverser client, void *arg)
 	unsigned int i;
 
 	/* Check for cache validity. */
-	if(filemon_from_file("/etc/mtab", &mon) != 0 ||
+	if(filemon_from_file("/etc/mtab", FMT_MODIFIED, &mon) != 0 ||
 			!filemon_equal(&mon, &mtab_mon))
 	{
 		filemon_assign(&mtab_mon, &mon);
@@ -761,7 +801,7 @@ format_help_cmd(char cmd[], size_t cmd_size)
 void
 display_help(const char cmd[])
 {
-	(void)shellout(cmd, PAUSE_ON_ERROR, 1);
+	(void)shellout(cmd, PAUSE_ON_ERROR, 1, SHELL_BY_APP);
 }
 
 void
@@ -885,6 +925,7 @@ reopen_term_stdout(void)
 	fp = fdopen(outfd, "w");
 	if(fp == NULL)
 	{
+		close(outfd);
 		fprintf(stderr, "Failed to open original output stream.\n");
 		return NULL;
 	}
@@ -988,7 +1029,7 @@ read_cmd_output(const char cmd[], int preserve_stdin)
 
 	if(pid == 0)
 	{
-		run_from_fork(out_pipe, 0, preserve_stdin, (char *)cmd);
+		run_from_fork(out_pipe, 0, preserve_stdin, (char *)cmd, SHELL_BY_USER);
 		return NULL;
 	}
 
@@ -1013,7 +1054,14 @@ void
 clone_attribs(const char path[], const char from[], const struct stat *st)
 {
 	chown(path, st->st_uid, st->st_gid);
+	clone_timestamps(path, from, st);
+	clone_xattrs(path, from);
+}
 
+/* Clones timestamps from file specified by from to file at path. */
+static void
+clone_timestamps(const char path[], const char from[], const struct stat *st)
+{
 #if defined(HAVE_STRUCT_STAT_ST_MTIM) && defined(HAVE_FUTIMENS)
 	const int fd = open(path, O_WRONLY);
 	if(fd != -1)
@@ -1044,6 +1092,58 @@ clone_attribs(const char path[], const char from[], const struct stat *st)
 	utimes(path, tv);
 }
 
+/* Clones extended attributes from file specified by from to file at path. */
+static void
+clone_xattrs(const char path[], const char from[])
+{
+#ifdef HAVE_XATTRS
+	char empty[1];
+
+	ssize_t list_size = llistxattr(from, empty, 0U);
+	if(list_size < 0)
+	{
+		return;
+	}
+
+	/* Size of VLA should never be zero. */
+	char list[list_size + 1];
+	list_size = llistxattr(from, list, list_size);
+	if(list_size < 0)
+	{
+		return;
+	}
+
+	const char *name = list;
+	while(name - list < list_size)
+	{
+		ssize_t value_size = lgetxattr(from, name, empty, 0U);
+		if(value_size < 0)
+		{
+			break;
+		}
+
+		/* Size of VLA should never be zero. */
+		char value[value_size + 1];
+		value_size = lgetxattr(from, name, value, value_size);
+		if(value_size < 0)
+		{
+			break;
+		}
+
+		if(lsetxattr(path, name, value, value_size, XATTR_CREATE) != 0)
+		{
+			if(errno == ENOTSUP)
+			{
+				/* Mount of the destination doesn't support extended attributes. */
+				break;
+			}
+		}
+
+		name += strlen(name) + 1U;
+	}
+#endif
+}
+
 uint64_t
 get_free_space(const char at[])
 {
