@@ -20,7 +20,8 @@
 #define VIFM__VIFM_H__
 
 #include "utils/macros.h"
-#include "ui/ui.h"
+
+struct view_t;
 
 /* Resets internal state and reloads configuration files. */
 void vifm_restart(void);
@@ -30,13 +31,17 @@ void vifm_restart(void);
 void vifm_try_leave(int write_info, int cquit, int force);
 
 /* Communicates chosen files to something external. */
-void _gnuc_noreturn vifm_choose_files(const FileView *view, int nfiles,
+void _gnuc_noreturn vifm_choose_files(const struct view_t *view, int nfiles,
 		char *files[]);
 
 /* Quits vifm with error after deinitializing ncurses, saving state to vifminfo
  * and displaying the message. */
 void _gnuc_noreturn vifm_finish(const char message[]);
 
+/* Quits vifm with the specified exit code after performing some clean up.
+ * This function should be used instead of using exit() directly. */
+void _gnuc_noreturn vifm_exit(int exit_code);
+
 #endif /* VIFM__VIFM_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
