@@ -36,21 +36,7 @@ view_setup(view_t *view)
 void
 view_teardown(view_t *view)
 {
-	int i;
-
-	for(i = 0; i < view->list_rows; ++i)
-	{
-		fentry_free(view, &view->dir_entry[i]);
-	}
-	dynarray_free(view->dir_entry);
-
-	filter_dispose(&view->local_filter.filter);
-	matcher_free(view->manual_filter);
-	view->manual_filter = NULL;
-	filter_dispose(&view->auto_filter);
-
-	fswatch_free(view->watch);
-	view->watch = NULL;
+	flist_free_view(view);
 }
 
 void
@@ -92,7 +78,7 @@ set_to_sandbox_path(char buf[], size_t buf_len)
 	}
 	else
 	{
-		char cwd[PATH_MAX];
+		char cwd[PATH_MAX + 1];
 		assert_non_null(get_cwd(cwd, sizeof(cwd)));
 		snprintf(buf, buf_len, "%s/%s", cwd, SANDBOX_PATH);
 	}
@@ -102,14 +88,23 @@ void
 make_abs_path(char buf[], size_t buf_len, const char base[], const char sub[],
 		const char cwd[])
 {
+	char local_buf[buf_len];
+
 	if(is_path_absolute(base))
 	{
-		snprintf(buf, buf_len, "%s%s%s", base, (sub[0] == '\0' ? "" : "/"), sub);
+		snprintf(local_buf, buf_len, "%s%s%s", base, (sub[0] == '\0' ? "" : "/"),
+				sub);
 	}
 	else
 	{
-		snprintf(buf, buf_len, "%s/%s%s%s", cwd, base, (sub[0] == '\0' ? "" : "/"),
-				sub);
+		snprintf(local_buf, buf_len, "%s/%s%s%s", cwd, base,
+				(sub[0] == '\0' ? "" : "/"), sub);
+	}
+
+	canonicalize_path(local_buf, buf, buf_len);
+	if(!ends_with_slash(sub) && !is_root_dir(buf))
+	{
+		chosp(buf);
 	}
 }
 
