@@ -1,7 +1,5 @@
 #include <stic.h>
 
-#include <stdio.h> /* FILE fopen() fclose() */
-
 #include <unistd.h> /* F_OK access() */
 
 #include "../../src/compat/os.h"
@@ -9,8 +7,10 @@
 #include "../../src/io/ior.h"
 #include "../../src/utils/fs.h"
 
-static const char *const FILE_NAME = "file-to-remove";
-static const char *const DIRECTORY_NAME = "directory-to-remove";
+#include "utils.h"
+
+#define DIRECTORY_NAME SANDBOX_PATH "/directory-to-remove"
+#define FILE_NAME DIRECTORY_NAME "/file-to-remove"
 
 TEST(empty_directory_is_removed)
 {
@@ -34,14 +34,7 @@ TEST(non_empty_directory_is_not_removed)
 {
 	os_mkdir(DIRECTORY_NAME, 0700);
 	assert_true(is_dir(DIRECTORY_NAME));
-
-	assert_success(chdir(DIRECTORY_NAME));
-	{
-		FILE *const f = fopen(FILE_NAME, "w");
-		fclose(f);
-		assert_success(access(FILE_NAME, F_OK));
-	}
-	assert_success(chdir(".."));
+	create_test_file(FILE_NAME);
 
 	{
 		io_args_t args = {
@@ -71,4 +64,4 @@ TEST(non_empty_directory_is_not_removed)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
