@@ -5,6 +5,7 @@
 #include <stdio.h> /* EOF FILE fclose() fgetc() fopen() */
 
 #include "../../src/io/iop.h"
+#include "../../src/utils/utils.h"
 
 void
 create_test_file(const char name[])
@@ -41,6 +42,8 @@ delete_test_file(const char name[])
 
 	assert_success(iop_rmfile(&args));
 	assert_int_equal(0, args.result.errors.error_count);
+
+	ioe_errlst_free(&args.result.errors);
 }
 
 int
@@ -76,5 +79,11 @@ files_are_identical(const char a[], const char b[])
 	return a_data == b_data && a_data == EOF;
 }
 
+int
+not_windows(void)
+{
+	return get_env_type() != ET_WIN;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
