@@ -8,12 +8,9 @@
 #include "../../src/io/iop.h"
 #include "../../src/io/ior.h"
 #include "../../src/utils/fs.h"
-#include "../../src/utils/utils.h"
 
 #include "utils.h"
 
-static int not_windows(void);
-
 TEST(file_is_copied)
 {
 	{
@@ -599,11 +596,5 @@ TEST(symlink_to_dir_is_symlink_after_copy, IF(not_windows))
 	}
 }
 
-static int
-not_windows(void)
-{
-	return get_env_type() != ET_WIN;
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
