@@ -3,6 +3,7 @@
 #include <stdint.h> /* uint64_t */
 #include <stdio.h> /* remove() */
 
+#include <sys/stat.h> /* chmod() */
 #include <unistd.h> /* F_OK access() */
 
 #include "../../src/io/iop.h"
@@ -43,7 +44,7 @@ TEST(confirm_is_not_called_for_no_overwrite)
 	delete_file(SANDBOX_PATH "/empty");
 }
 
-TEST(confirm_is_called_for_overwrite)
+TEST(confirm_is_called_for_file_overwrite)
 {
 	create_empty_file(SANDBOX_PATH "/empty");
 	clone_file(TEST_DATA_PATH "/read/two-lines", SANDBOX_PATH "/two-lines");
@@ -68,6 +69,37 @@ TEST(confirm_is_called_for_overwrite)
 	delete_file(SANDBOX_PATH "/empty");
 }
 
+TEST(confirm_is_called_for_dir_overwrite, IF(not_windows))
+{
+	create_empty_dir(SANDBOX_PATH "/src");
+	create_empty_file(SANDBOX_PATH "/src/file");
+	create_empty_file(SANDBOX_PATH "/file");
+
+	assert_success(chmod(SANDBOX_PATH "/src", 0500));
+
+	{
+		io_args_t args = {
+			.arg1.src = SANDBOX_PATH "/src/file",
+			.arg2.dst = SANDBOX_PATH "/file",
+			.arg3.crs = IO_CRS_REPLACE_FILES,
+
+			.confirm = &confirm_overwrite,
+		};
+		ioe_errlst_init(&args.result.errors);
+
+		confirm_called = 0;
+		assert_failure(ior_mv(&args));
+		assert_int_equal(1, confirm_called);
+
+		assert_int_equal(1, args.result.errors.error_count);
+		ioe_errlst_free(&args.result.errors);
+	}
+
+	assert_success(chmod(SANDBOX_PATH "/src", 0700));
+	delete_tree(SANDBOX_PATH "/src");
+	delete_file(SANDBOX_PATH "/file");
+}
+
 TEST(deny_to_overwrite_is_considered)
 {
 	create_empty_file(SANDBOX_PATH "/empty");
