@@ -2,9 +2,10 @@
 
 #include <unistd.h> /* chdir() rmdir() symlink() */
 
+#include <locale.h> /* LC_ALL setlocale() */
 #include <stddef.h> /* NULL */
 #include <stdlib.h> /* fclose() fopen() free() */
-#include <string.h> /* strdup() */
+#include <string.h> /* strdup() strstr() */
 #include <wchar.h> /* wcsdup() */
 
 #include "../../src/compat/fs_limits.h"
@@ -80,13 +81,13 @@ SETUP()
 	vle_cmds_run("command baz b");
 	vle_cmds_run("command foo c");
 
-	init_options(&option_changed, NULL);
-	add_option("fusehome", "fh", "descr", OPT_STR, OPT_GLOBAL, 0, NULL,
+	vle_opts_init(&option_changed, NULL);
+	vle_opts_add("fusehome", "fh", "descr", OPT_STR, OPT_GLOBAL, 0, NULL,
 			&dummy_handler, def);
-	add_option("path", "pt", "descr", OPT_STR, OPT_GLOBAL, 0, NULL,
+	vle_opts_add("path", "pt", "descr", OPT_STR, OPT_GLOBAL, 0, NULL,
+			&dummy_handler, def);
+	vle_opts_add("path", "pt", "descr", OPT_STR, OPT_LOCAL, 0, NULL,
 			&dummy_handler, def);
-	add_option("path", "pt", "descr", OPT_STR, OPT_LOCAL, 0, NULL, &dummy_handler,
-			def);
 
 	saved_cwd = save_cwd();
 	assert_success(chdir(TEST_DATA_PATH "/compare"));
@@ -107,7 +108,7 @@ TEARDOWN()
 
 	free(stats.line);
 	vle_cmds_reset();
-	clear_options();
+	vle_opts_reset();
 
 	function_reset_all();
 }
@@ -335,10 +336,22 @@ TEST(dirs_are_completed_with_trailing_slash)
 	ASSERT_NEXT_MATCH("read/");
 }
 
+TEST(tabnew_has_directory_only_completion)
+{
+	make_abs_path(curr_view->curr_dir, sizeof(curr_view->curr_dir),
+			TEST_DATA_PATH, "tree/dir1", saved_cwd);
+	assert_success(chdir(curr_view->curr_dir));
+
+	ASSERT_COMPLETION(L"tabnew ", L"tabnew dir2/");
+	ASSERT_NEXT_MATCH("dir2/");
+	ASSERT_NEXT_MATCH("dir2/");
+}
+
 TEST(function_name_completion)
 {
 	ASSERT_COMPLETION(L"echo e", L"echo executable(");
 	ASSERT_NEXT_MATCH("expand(");
+	ASSERT_NEXT_MATCH("extcached(");
 	ASSERT_NEXT_MATCH("e");
 }
 
@@ -428,11 +441,34 @@ TEST(bang_abs_path_completion)
 	free(wcwd);
 }
 
+TEST(standalone_tilde_is_expanded)
+{
+	make_abs_path(cfg.home_dir, sizeof(cfg.home_dir), TEST_DATA_PATH, "/",
+			saved_cwd);
+
+	char *expected = format_str("cd %s", cfg.home_dir);
+	wchar_t *wexpected = to_wide(expected);
+
+	ASSERT_COMPLETION(L"cd ~", wexpected);
+
+	free(wexpected);
+	free(expected);
+}
+
 TEST(tilde_is_completed_after_emark)
 {
 	make_abs_path(cfg.home_dir, sizeof(cfg.home_dir), TEST_DATA_PATH, "/",
 			saved_cwd);
-	ASSERT_COMPLETION(L"!~/", L"!~/compare/");
+	ASSERT_COMPLETION(L"!~/", L"!~/color-schemes/");
+}
+
+TEST(user_name_is_completed_after_tilde, IF(not_windows))
+{
+	prepare_for_line_completion(L"cd ~roo");
+	assert_success(line_completion(&stats));
+	char *narrow = to_multibyte(stats.line);
+	assert_string_equal(NULL, strstr(narrow, "\\~"));
+	free(narrow);
 }
 
 TEST(bmark_tags_are_completed)
@@ -482,12 +518,13 @@ TEST(colorscheme_completion)
 			"scripts", saved_cwd);
 	ASSERT_COMPLETION(L"colorscheme set-", L"colorscheme set-env");
 	ASSERT_COMPLETION(L"colorscheme set-env ../",
-			L"colorscheme set-env ../compare/");
+			L"colorscheme set-env ../color-schemes/");
 	ASSERT_COMPLETION(L"colorscheme ../", L"colorscheme ../");
 
 	make_abs_path(curr_view->curr_dir, sizeof(curr_view->curr_dir),
 			TEST_DATA_PATH, "", saved_cwd);
-	ASSERT_COMPLETION(L"colorscheme set-env ", L"colorscheme set-env compare/");
+	ASSERT_COMPLETION(L"colorscheme set-env ",
+			L"colorscheme set-env color-schemes/");
 }
 
 TEST(wincmd_completion)
@@ -506,7 +543,7 @@ TEST(grep_completion)
 			TEST_DATA_PATH, "", saved_cwd);
 	assert_success(chdir(curr_view->curr_dir));
 
-	ASSERT_COMPLETION(L"grep -o ", L"grep -o compare/");
+	ASSERT_COMPLETION(L"grep -o ", L"grep -o color-schemes/");
 }
 
 TEST(find_completion)
@@ -525,7 +562,7 @@ TEST(find_completion)
 			TEST_DATA_PATH, "", saved_cwd);
 	assert_success(chdir(curr_view->curr_dir));
 
-	ASSERT_COMPLETION(L"find ", L"find compare/");
+	ASSERT_COMPLETION(L"find ", L"find color-schemes/");
 }
 
 TEST(aucmd_events_are_completed)
@@ -650,6 +687,27 @@ TEST(symlinks_in_paths_are_not_resolved, IF(not_windows))
 	assert_success(remove(SANDBOX_PATH "/dir-link"));
 }
 
+TEST(failure_to_turn_into_multibyte_is_handled, IF(not_windows))
+{
+	/* On Windows narrow locale doesn't really matter because UTF-8 is used
+	 * internally by vifm. */
+	prepare_for_line_completion(L"абвгд | set  all");
+	stats.index = 12;
+
+	(void)setlocale(LC_ALL, "C");
+	assert_failure(line_completion(&stats));
+	try_enable_utf8_locale();
+	assert_wstring_equal(L"абвгд | set  all", stats.line);
+}
+
+TEST(non_latin_prefix_does_not_break_completion, IF(utf8_locale))
+{
+	prepare_for_line_completion(L"абвгд | set ");
+
+	assert_success(line_completion(&stats));
+	assert_wstring_equal(L"абвгд | set all", stats.line);
+}
+
 static int
 dquotes_allowed_in_paths(void)
 {
