@@ -59,8 +59,10 @@ SETUP()
 
 #ifndef _WIN32
 	replace_string(&cfg.shell, "/bin/sh");
+	update_string(&cfg.shell_cmd_flag, "-c");
 #else
 	replace_string(&cfg.shell, "cmd");
+	update_string(&cfg.shell_cmd_flag, "/C");
 #endif
 
 	stats_update_shell_type(cfg.shell);
@@ -81,6 +83,7 @@ TEARDOWN()
 	update_string(&cfg.slow_fs_list, NULL);
 
 	stats_update_shell_type("/bin/sh");
+	update_string(&cfg.shell_cmd_flag, NULL);
 	update_string(&cfg.shell, NULL);
 
 	view_teardown(&lwin);
@@ -119,6 +122,77 @@ TEST(double_cd_uses_same_base_for_rel_paths)
 	assert_true(paths_are_equal(rwin.curr_dir, path));
 }
 
+TEST(cds_does_the_replacement)
+{
+	char path[PATH_MAX + 1];
+	snprintf(path, sizeof(path), "%s/spaces-in-names", test_data);
+	assert_success(chdir(path));
+	strcpy(lwin.curr_dir, path);
+
+	assert_success(exec_commands("cds spaces-in-names rename", &lwin,
+				CIT_COMMAND));
+
+	snprintf(path, sizeof(path), "%s/rename", test_data);
+	assert_string_equal(path, lwin.curr_dir);
+}
+
+TEST(cds_aborts_on_broken_)
+{
+	char dst[PATH_MAX + 1];
+	snprintf(dst, sizeof(dst), "%s/rename", test_data);
+
+	assert_success(chdir(dst));
+
+	strcpy(lwin.curr_dir, dst);
+
+	assert_failure(exec_commands("cds/rename/read/t", &lwin, CIT_COMMAND));
+
+	assert_string_equal(dst, lwin.curr_dir);
+}
+
+TEST(cds_acts_like_substitute)
+{
+	char path[PATH_MAX + 1];
+	snprintf(path, sizeof(path), "%s/spaces-in-names", test_data);
+	assert_success(chdir(path));
+	strcpy(lwin.curr_dir, path);
+
+	assert_success(exec_commands("cds/spaces-IN-[a-z]*/rename/i", &lwin,
+				CIT_COMMAND));
+
+	snprintf(path, sizeof(path), "%s/rename", test_data);
+	assert_string_equal(path, lwin.curr_dir);
+}
+
+TEST(cds_can_change_path_of_both_panes)
+{
+	char path[PATH_MAX + 1];
+	snprintf(path, sizeof(path), "%s/spaces-in-names", test_data);
+	assert_success(chdir(path));
+	strcpy(lwin.curr_dir, path);
+
+	assert_success(exec_commands("cds! spaces-in-names rename", &lwin,
+				CIT_COMMAND));
+
+	snprintf(path, sizeof(path), "%s/rename", test_data);
+	assert_string_equal(path, lwin.curr_dir);
+	assert_string_equal(path, rwin.curr_dir);
+}
+
+TEST(cds_is_noop_when_pattern_not_found)
+{
+	assert_success(chdir(test_data));
+
+	strcpy(lwin.curr_dir, test_data);
+	strcpy(rwin.curr_dir, sandbox);
+
+	assert_failure(exec_commands("cds asdlfkjasdlkfj rename", &lwin,
+				CIT_COMMAND));
+
+	assert_string_equal(test_data, lwin.curr_dir);
+	assert_string_equal(sandbox, rwin.curr_dir);
+}
+
 TEST(cpmv_does_not_crash_on_wrong_list_access)
 {
 	char path[PATH_MAX + 1];
@@ -329,46 +403,12 @@ TEST(symlinks_in_paths_are_not_resolved, IF(not_windows))
 	assert_success(rmdir(SANDBOX_PATH "/dir1"));
 }
 
-TEST(find_command, IF(not_windows))
-{
-	opt_handlers_setup();
-
-	replace_string(&cfg.shell, "/bin/sh");
-
-	assert_success(chdir(TEST_DATA_PATH));
-	strcpy(lwin.curr_dir, test_data);
-
-	assert_success(exec_commands("set findprg='find %s %a %u'", &lwin,
-				CIT_COMMAND));
-
-	/* Nothing to repeat. */
-	assert_failure(exec_commands("find", &lwin, CIT_COMMAND));
-
-	assert_success(exec_commands("find a", &lwin, CIT_COMMAND));
-	assert_int_equal(3, lwin.list_rows);
-
-	assert_success(exec_commands("find . -name aaa", &lwin, CIT_COMMAND));
-	assert_int_equal(1, lwin.list_rows);
-
-	assert_success(exec_commands("find -name '*.vifm'", &lwin, CIT_COMMAND));
-	assert_int_equal(4, lwin.list_rows);
-
-	view_teardown(&lwin);
-	view_setup(&lwin);
-
-	/* Repeat last search. */
-	strcpy(lwin.curr_dir, test_data);
-	assert_success(exec_commands("find", &lwin, CIT_COMMAND));
-	assert_int_equal(4, lwin.list_rows);
-
-	opt_handlers_teardown();
-}
-
 TEST(grep_command, IF(not_windows))
 {
 	opt_handlers_setup();
 
 	replace_string(&cfg.shell, "/bin/sh");
+	update_string(&cfg.shell_cmd_flag, "-c");
 
 	assert_success(chdir(TEST_DATA_PATH "/scripts"));
 	assert_non_null(get_cwd(lwin.curr_dir, sizeof(lwin.curr_dir)));
@@ -384,7 +424,7 @@ TEST(grep_command, IF(not_windows))
 
 	/* Repeat last grep. */
 	assert_success(exec_commands("grep!", &lwin, CIT_COMMAND));
-	assert_int_equal(1, lwin.list_rows);
+	assert_int_equal(3, lwin.list_rows);
 
 	opt_handlers_teardown();
 }
