@@ -11,6 +11,7 @@
 #include "../../src/utils/fs.h"
 #include "../../src/utils/str.h"
 #include "../../src/cmd_core.h"
+#include "../../src/status.h"
 
 #include "utils.h"
 
@@ -26,8 +27,10 @@ SETUP()
 
 #ifndef _WIN32
 	replace_string(&cfg.shell, "/bin/sh");
+	replace_string(&cfg.shell_cmd_flag, "-c");
 #else
 	replace_string(&cfg.shell, "cmd");
+	replace_string(&cfg.shell_cmd_flag, "/C");
 #endif
 
 	stats_update_shell_type(cfg.shell);
@@ -51,6 +54,7 @@ TEARDOWN()
 
 	stats_update_shell_type("/bin/sh");
 	update_string(&cfg.shell, NULL);
+	update_string(&cfg.shell_cmd_flag, NULL);
 
 	view_teardown(&lwin);
 	view_teardown(&rwin);
