@@ -12,7 +12,7 @@ TEST(empty_string_ok)
 	};
 
 	const char *const pattern = "";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("", expanded);
 	free(expanded);
 }
@@ -24,7 +24,7 @@ TEST(no_macros_ok)
 	};
 
 	const char *const pattern = "no match in here";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("no match in here", expanded);
 	free(expanded);
 }
@@ -36,7 +36,7 @@ TEST(macro_substitution_works)
 	};
 
 	const char *const pattern = "a match here %i";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("a match here xyz", expanded);
 	free(expanded);
 }
@@ -48,7 +48,7 @@ TEST(use_negative_count_macro_not_added_implicitly)
 	};
 
 	const char *const pattern = "a match here, %i, just was";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("a match here, xyz, just was", expanded);
 	free(expanded);
 }
@@ -60,7 +60,7 @@ TEST(use_count_0_macro_not_added_implicitly)
 	};
 
 	const char *const pattern = "a match here, %i, just was";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("a match here, xyz, just was", expanded);
 	free(expanded);
 }
@@ -72,7 +72,7 @@ TEST(use_count_1_macro_added_once_implicitly)
 	};
 
 	const char *const pattern = "a match here:";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("a match here: xyz", expanded);
 	free(expanded);
 }
@@ -84,7 +84,7 @@ TEST(use_count_1_macro_not_added_implicitly)
 	};
 
 	const char *const pattern = "a match here, %i, just was";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("a match here, xyz, just was", expanded);
 	free(expanded);
 }
@@ -96,7 +96,7 @@ TEST(use_count_2_macro_added_once_implicitly)
 	};
 
 	const char *const pattern = "a match here, %i, just was,";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("a match here, xyz, just was, xyz", expanded);
 	free(expanded);
 }
@@ -108,7 +108,7 @@ TEST(use_count_2_macro_added_twice_implicitly)
 	};
 
 	const char *const pattern = "matches follow";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("matches follow xyz xyz", expanded);
 	free(expanded);
 }
@@ -120,7 +120,7 @@ TEST(unknown_macro_removed)
 	};
 
 	const char *const pattern = "the macro %b was here";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("the macro  was here", expanded);
 	free(expanded);
 }
@@ -132,7 +132,7 @@ TEST(double_percent_handled_correctly)
 	};
 
 	const char *const pattern = "the percent sign is here: %%";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("the percent sign is here: %", expanded);
 	free(expanded);
 }
@@ -144,7 +144,7 @@ TEST(ends_with_percent_ok)
 	};
 
 	const char *const pattern = "the percent sign is here: %";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("the percent sign is here: %", expanded);
 	free(expanded);
 }
@@ -157,7 +157,7 @@ TEST(first_group_member_are_not_added_when_should_not)
 	};
 
 	const char *const pattern = "no i expansion is expected: %j";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("no i expansion is expected: abc", expanded);
 	free(expanded);
 }
@@ -170,7 +170,7 @@ TEST(second_group_member_are_not_added_when_should_not)
 	};
 
 	const char *const pattern = "no j expansion is expected: %i";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("no j expansion is expected: xyz", expanded);
 	free(expanded);
 }
@@ -183,7 +183,7 @@ TEST(first_item_of_group_is_repeated_when_needed)
 	};
 
 	const char *const pattern = "i expansion is expected:";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("i expansion is expected: xyz", expanded);
 	free(expanded);
 }
@@ -196,7 +196,7 @@ TEST(explicit_use_sets_appropriate_flag)
 	};
 
 	const char *const pattern = "something%u";
-	char *expanded = expand_custom_macros(pattern, ARRAY_LEN(macros), macros);
+	char *expanded = ma_expand_custom(pattern, ARRAY_LEN(macros), macros);
 	assert_string_equal("something", expanded);
 	free(expanded);
 
