@@ -6,6 +6,7 @@
 #include <string.h>
 
 #include "../../src/cfg/config.h"
+#include "../../src/ui/quickview.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/dynarray.h"
 #include "../../src/utils/str.h"
@@ -13,6 +14,7 @@
 #include "../../src/flist_sel.h"
 #include "../../src/macros.h"
 #include "../../src/registers.h"
+#include "../../src/status.h"
 
 #include "utils.h"
 
@@ -98,16 +100,30 @@ TEARDOWN()
 	dynarray_free(rwin.dir_entry);
 }
 
+TEST(literal_percent)
+{
+	char *expanded = ma_expand("echo log %%", "", NULL, 0);
+	assert_string_equal("echo log %", expanded);
+	free(expanded);
+}
+
+TEST(argument_expansion)
+{
+	char *expanded = ma_expand("echo %a", "this is arg", NULL, 0);
+	assert_string_equal("echo this is arg", expanded);
+	free(expanded);
+}
+
 TEST(b_both_have_selection)
 {
 	char *expanded;
 
-	expanded = expand_macros("/%b ", "", NULL, 1);
+	expanded = ma_expand("/%b ", "", NULL, 1);
 	assert_string_equal(
 			"/lfi\\ le0 lfile\\\"2 " SL "rwin" SL "rfile1 " SL "rwin" SL "rfile3 " SL "rwin" SL "rfile5 ", expanded);
 	free(expanded);
 
-	expanded = expand_macros("%b", "", NULL, 1);
+	expanded = ma_expand("%b", "", NULL, 1);
 	assert_string_equal(
 			"lfi\\ le0 lfile\\\"2 " SL "rwin" SL "rfile1 " SL "rwin" SL "rfile3 " SL "rwin" SL "rfile5", expanded);
 	free(expanded);
@@ -120,11 +136,11 @@ TEST(f_both_have_selection)
 	lwin.dir_entry[2].selected = 0;
 	lwin.selected_files--;
 
-	expanded = expand_macros("/%f ", "", NULL, 0);
+	expanded = ma_expand("/%f ", "", NULL, 0);
 	assert_string_equal("/lfi\\ le0 ", expanded);
 	free(expanded);
 
-	expanded = expand_macros("%f", "", NULL, 0);
+	expanded = ma_expand("%f", "", NULL, 0);
 	assert_string_equal("lfi\\ le0", expanded);
 	free(expanded);
 }
@@ -135,12 +151,12 @@ TEST(b_only_lwin_has_selection)
 
 	flist_sel_stash(&lwin);
 
-	expanded = expand_macros("/%b ", "", NULL, 1);
+	expanded = ma_expand("/%b ", "", NULL, 1);
 	assert_string_equal("/lfile\\\"2 " SL "rwin" SL "rfile1 " SL "rwin" SL "rfile3 " SL "rwin" SL "rfile5 ",
 			expanded);
 	free(expanded);
 
-	expanded = expand_macros("%b", "", NULL, 1);
+	expanded = ma_expand("%b", "", NULL, 1);
 	assert_string_equal("lfile\\\"2 " SL "rwin" SL "rfile1 " SL "rwin" SL "rfile3 " SL "rwin" SL "rfile5",
 			expanded);
 	free(expanded);
@@ -152,12 +168,12 @@ TEST(b_only_rwin_has_selection)
 
 	flist_sel_stash(&rwin);
 
-	expanded = expand_macros("/%b ", "", NULL, 1);
+	expanded = ma_expand("/%b ", "", NULL, 1);
 	assert_string_equal("/lfi\\ le0 lfile\\\"2 " SL "rwin" SL "rfile5 ",
 			expanded);
 	free(expanded);
 
-	expanded = expand_macros("%b", "", NULL, 1);
+	expanded = ma_expand("%b", "", NULL, 1);
 	assert_string_equal("lfi\\ le0 lfile\\\"2 " SL "rwin" SL "rfile5", expanded);
 	free(expanded);
 }
@@ -169,23 +185,21 @@ TEST(b_noone_has_selection)
 	flist_sel_stash(&lwin);
 	flist_sel_stash(&rwin);
 
-	expanded = expand_macros("/%b ", "", NULL, 1);
+	expanded = ma_expand("/%b ", "", NULL, 1);
 	assert_string_equal("/lfile\\\"2 " SL "rwin" SL "rfile5 ", expanded);
 	free(expanded);
 
-	expanded = expand_macros("%b", "", NULL, 1);
+	expanded = ma_expand("%b", "", NULL, 1);
 	assert_string_equal("lfile\\\"2 " SL "rwin" SL "rfile5", expanded);
 	free(expanded);
 }
 
 TEST(no_slash_after_dirname)
 {
-	char *expanded;
-
 	rwin.list_pos = 6;
 	curr_view = &rwin;
 	other_view = &lwin;
-	expanded = expand_macros("%c", "", NULL, 0);
+	char *expanded = ma_expand("%c", "", NULL, 0);
 	assert_string_equal("rdir6", expanded);
 	free(expanded);
 }
@@ -197,38 +211,92 @@ TEST(forward_slashes_on_win_for_non_shell)
 	flist_sel_stash(&lwin);
 	flist_sel_stash(&rwin);
 
-	expanded = expand_macros("/%b ", "", NULL, 0);
+	expanded = ma_expand("/%b ", "", NULL, 0);
 	assert_string_equal("/lfile\\\"2 /rwin/rfile5 ", expanded);
 	free(expanded);
 
-	expanded = expand_macros("%b", "", NULL, 0);
+	expanded = ma_expand("%b", "", NULL, 0);
 	assert_string_equal("lfile\\\"2 /rwin/rfile5", expanded);
 	free(expanded);
 }
 
-TEST(m)
+TEST(good_flag_macros)
 {
 	MacroFlags flags;
 	char *expanded;
 
-	rwin.list_pos = 6;
-	curr_view = &rwin;
-	other_view = &lwin;
-	expanded = expand_macros("%M echo log", "", &flags, 0);
+	expanded = ma_expand("%i echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_IGNORE, flags);
+	free(expanded);
+
+	expanded = ma_expand("%Iu echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_CUSTOMVIEW_IOUTPUT, flags);
+	free(expanded);
+
+	expanded = ma_expand("%IU echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_VERYCUSTOMVIEW_IOUTPUT, flags);
+	free(expanded);
+
+	expanded = ma_expand("%m echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_MENU_OUTPUT, flags);
+	free(expanded);
+
+	expanded = ma_expand("%M echo log", "", &flags, 0);
 	assert_string_equal(" echo log", expanded);
 	assert_int_equal(MF_MENU_NAV_OUTPUT, flags);
 	free(expanded);
+
+	expanded = ma_expand("%n echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_NO_TERM_MUX, flags);
+	free(expanded);
+
+	expanded = ma_expand("%q echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_PREVIEW_OUTPUT, flags);
+	free(expanded);
+
+	expanded = ma_expand("%s echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_SPLIT, flags);
+	free(expanded);
+
+	expanded = ma_expand("%S echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_STATUSBAR_OUTPUT, flags);
+	free(expanded);
+
+	expanded = ma_expand("%u echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_CUSTOMVIEW_OUTPUT, flags);
+	free(expanded);
+
+	expanded = ma_expand("%U echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_VERYCUSTOMVIEW_OUTPUT, flags);
+	free(expanded);
+}
+
+TEST(bad_flag_macros)
+{
+	MacroFlags flags;
+	char *expanded = ma_expand("%IX echo log", "", &flags, 0);
+	assert_string_equal("X echo log", expanded);
+	assert_int_equal(MF_NONE, flags);
+	free(expanded);
 }
 
 TEST(r_well_formed)
 {
-	const char *p;
-
 	assert_success(chdir(TEST_DATA_PATH "/existing-files"));
 
 	regs_init();
 
-	p = valid_registers;
+	const char *p = valid_registers;
 	while(*p != '\0')
 	{
 		char line[32];
@@ -240,7 +308,7 @@ TEST(r_well_formed)
 		regs_append(key, "b");
 		regs_append(key, "c");
 
-		expanded = expand_macros(line, NULL, NULL, 0);
+		expanded = ma_expand(line, NULL, NULL, 0);
 		if(key == '_')
 		{
 			assert_string_equal("", expanded);
@@ -257,7 +325,6 @@ TEST(r_well_formed)
 
 TEST(r_ill_formed)
 {
-	unsigned char key;
 	char expected[] = "a b cx";
 
 	assert_success(chdir(TEST_DATA_PATH "/existing-files"));
@@ -268,7 +335,7 @@ TEST(r_ill_formed)
 	regs_append(DEFAULT_REG_NAME, "b");
 	regs_append(DEFAULT_REG_NAME, "c");
 
-	key = '\0';
+	unsigned char key = '\0';
 	do
 	{
 		char line[32];
@@ -276,7 +343,7 @@ TEST(r_ill_formed)
 
 		if(!char_is_one_of(valid_registers, key) && key != '%')
 		{
-			char *const expanded = expand_macros(line, NULL, NULL, 0);
+			char *const expanded = ma_expand(line, NULL, NULL, 0);
 			expected[5] = key;
 			assert_string_equal(expected, expanded);
 			free(expanded);
@@ -291,38 +358,38 @@ TEST(with_quotes)
 {
 	char *expanded;
 
-	expanded = expand_macros("/%\"b ", "", NULL, 1);
+	expanded = ma_expand("/%\"b ", "", NULL, 1);
 	assert_string_equal(
 			"/\"lfi le0\" \"lfile\\\"2\" "
 			"\"" SL "rwin" SL "rfile1\" \"" SL "rwin" SL "rfile3\" \"" SL "rwin" SL "rfile5\" ", expanded);
 	free(expanded);
 
-	expanded = expand_macros("/%\"f ", "", NULL, 1);
+	expanded = ma_expand("/%\"f ", "", NULL, 1);
 	assert_string_equal("/\"lfi le0\" \"lfile\\\"2\" ", expanded);
 	free(expanded);
 
-	expanded = expand_macros("/%\"F ", "", NULL, 1);
+	expanded = ma_expand("/%\"F ", "", NULL, 1);
 	assert_string_equal("/\"" SL "rwin" SL "rfile1\" \"" SL "rwin" SL "rfile3\" \"" SL "rwin" SL "rfile5\" ",
 			expanded);
 	free(expanded);
 
-	expanded = expand_macros("/%\"c ", "", NULL, 1);
+	expanded = ma_expand("/%\"c ", "", NULL, 1);
 	assert_string_equal("/\"lfile\\\"2\" ", expanded);
 	free(expanded);
 
-	expanded = expand_macros("/%\"C ", "", NULL, 1);
+	expanded = ma_expand("/%\"C ", "", NULL, 1);
 	assert_string_equal("/\"" SL "rwin" SL "rfile5\" ", expanded);
 	free(expanded);
 
-	expanded = expand_macros("/%\"d ", "", NULL, 1);
+	expanded = ma_expand("/%\"d ", "", NULL, 1);
 	assert_string_equal("/\"" SL "lwin\" ", expanded);
 	free(expanded);
 
-	expanded = expand_macros("/%\"D ", "", NULL, 1);
+	expanded = ma_expand("/%\"D ", "", NULL, 1);
 	assert_string_equal("/\"" SL "rwin\" ", expanded);
 	free(expanded);
 
-	expanded = expand_macros(" %\"a %\"m %\"M %\"s ", "", NULL, 0);
+	expanded = ma_expand(" %\"a %\"m %\"M %\"s ", "", NULL, 0);
 	assert_string_equal(" a m M s ", expanded);
 	free(expanded);
 
@@ -333,7 +400,7 @@ TEST(with_quotes)
 	regs_append(DEFAULT_REG_NAME, "b");
 	regs_append(DEFAULT_REG_NAME, "c");
 
-	expanded = expand_macros("/%\"r ", "", NULL, 1);
+	expanded = ma_expand("/%\"r ", "", NULL, 1);
 	assert_string_equal("/\"a\" \"b\" \"c\" ", expanded);
 	free(expanded);
 
@@ -342,73 +409,62 @@ TEST(with_quotes)
 
 TEST(single_percent_sign)
 {
-	char *expanded;
-
-	expanded = expand_macros("%", "", NULL, 0);
+	char *expanded = ma_expand("%", "", NULL, 0);
 	assert_string_equal("", expanded);
 	free(expanded);
 }
 
 TEST(percent_sign_and_double_quote)
 {
-	char *expanded;
-
-	expanded = expand_macros("%\"", "", NULL, 0);
+	char *expanded = ma_expand("%\"", "", NULL, 0);
 	assert_string_equal("", expanded);
 	free(expanded);
 }
 
 TEST(empty_line_ok)
 {
-	char *expanded;
-
-	expanded = expand_macros("", "", NULL, 0);
+	char *expanded = ma_expand("", "", NULL, 0);
 	assert_string_equal("", expanded);
 	free(expanded);
 }
 
 TEST(singly_expanded_is_not_escaped)
 {
-	char *expanded;
 	lwin.list_pos = 0;
-	expanded = ma_expand_single("%c");
+	char *expanded = ma_expand_single("%c");
 	assert_string_equal("lfi le0", expanded);
 	free(expanded);
 }
 
 TEST(singly_expanded_is_not_quoted)
 {
-	char *expanded;
 	lwin.list_pos = 0;
-	expanded = ma_expand_single("%\"c");
+	char *expanded = ma_expand_single("%\"c");
 	assert_string_equal("lfi le0", expanded);
 	free(expanded);
 }
 
 TEST(singly_expanded_single_macros)
 {
-	char *expanded;
 	lwin.list_pos = 0;
-	expanded = ma_expand_single("%d");
+	char *expanded = ma_expand_single("%d");
 	assert_string_equal("/lwin", expanded);
 	free(expanded);
 }
 
 TEST(singly_expanded_multiple_macros_single_file)
 {
-	char *expanded;
 	lwin.dir_entry[2].selected = 0;
 	lwin.selected_files = 1;
-	expanded = ma_expand_single("%f");
+	char *expanded = ma_expand_single("%f");
 	assert_string_equal("lfi le0", expanded);
 	free(expanded);
 }
 
 TEST(singly_not_expanded_multiple_macros_multiple_files)
 {
-	char *expanded;
 	lwin.list_pos = 0;
-	expanded = ma_expand_single("%f");
+	char *expanded = ma_expand_single("%f");
 	assert_string_equal("", expanded);
 	free(expanded);
 }
@@ -462,12 +518,12 @@ TEST(dollar_and_backtick_are_escaped_in_dquotes)
 	assert_success(replace_string(&lwin.dir_entry[lwin.list_pos].name, "a$`b"));
 
 	curr_stats.shell_type = ST_NORMAL;
-	expanded = expand_macros("%\"c", "", NULL, 0);
+	expanded = ma_expand("%\"c", "", NULL, 0);
 	assert_string_equal("\"a\\$\\`b\"", expanded);
 	free(expanded);
 
 	curr_stats.shell_type = ST_CMD;
-	expanded = expand_macros("%\"c", "", NULL, 0);
+	expanded = ma_expand("%\"c", "", NULL, 0);
 	assert_string_equal("\"a$`b\"", expanded);
 	free(expanded);
 
@@ -477,15 +533,98 @@ TEST(dollar_and_backtick_are_escaped_in_dquotes)
 
 TEST(newline_is_escaped_with_quotes)
 {
-	char *expanded;
-
 	lwin.list_pos = 3;
 	assert_success(replace_string(&lwin.dir_entry[lwin.list_pos].name, "a\nb"));
 
-	expanded = expand_macros("%c", "", NULL, 0);
+	char *expanded = ma_expand("%c", "", NULL, 0);
 	assert_string_equal("a\"\n\"b", expanded);
 	free(expanded);
 }
 
+TEST(bad_preview_macro)
+{
+	char *expanded = ma_expand("draw %pz", "", NULL, 0);
+	assert_string_equal("draw z", expanded);
+	free(expanded);
+}
+
+TEST(preview_macros)
+{
+	lwin.window_cols = 15;
+	lwin.window_rows = 10;
+
+	char *expanded = ma_expand("draw %pw %ph", "", NULL, 0);
+	assert_string_equal("draw 15 10", expanded);
+	free(expanded);
+}
+
+TEST(preview_macros_use_hint)
+{
+	const preview_area_t parea = {
+		.source = &lwin,
+		.view = &rwin,
+		.x = 1,
+		.y = 2,
+		.w = 3,
+		.h = 4,
+	};
+
+	rwin.window_cols = 15;
+	rwin.window_rows = 10;
+
+	curr_stats.preview_hint = &parea;
+
+	char *expanded = ma_expand("draw %pw %ph", "", NULL, 0);
+	assert_string_equal("draw 3 4", expanded);
+	free(expanded);
+
+	curr_stats.preview_hint = NULL;
+}
+
+TEST(preview_clear_cmd_gets_cut_off)
+{
+	lwin.window_cols = 20;
+	lwin.window_rows = 30;
+
+	char *expanded = ma_expand("draw %pw %ph %pc clear", "", NULL, 0);
+	assert_string_equal("draw 20 30 ", expanded);
+	free(expanded);
+}
+
+TEST(preview_clear_cmd_is_extracted)
+{
+	assert_string_equal(" clear", ma_get_clear_cmd("draw %pw %ph %pc clear"));
+}
+
+TEST(preview_clear_cmd_is_optional)
+{
+	assert_string_equal(NULL, ma_get_clear_cmd("draw %pw %ph"));
+}
+
+TEST(preview_direct_is_expanded_to_nothing)
+{
+	char *expanded = ma_expand("draw %pd arg", "", NULL, 0);
+	assert_string_equal("draw  arg", expanded);
+	free(expanded);
+}
+
+TEST(flags_to_str)
+{
+	assert_string_equal("", ma_flags_to_str(MF_NONE));
+
+	assert_string_equal("%m", ma_flags_to_str(MF_MENU_OUTPUT));
+	assert_string_equal("%M", ma_flags_to_str(MF_MENU_NAV_OUTPUT));
+	assert_string_equal("%S", ma_flags_to_str(MF_STATUSBAR_OUTPUT));
+	assert_string_equal("%q", ma_flags_to_str(MF_PREVIEW_OUTPUT));
+	assert_string_equal("%u", ma_flags_to_str(MF_CUSTOMVIEW_OUTPUT));
+	assert_string_equal("%U", ma_flags_to_str(MF_VERYCUSTOMVIEW_OUTPUT));
+	assert_string_equal("%Iu", ma_flags_to_str(MF_CUSTOMVIEW_IOUTPUT));
+	assert_string_equal("%IU", ma_flags_to_str(MF_VERYCUSTOMVIEW_IOUTPUT));
+
+	assert_string_equal("%s", ma_flags_to_str(MF_SPLIT));
+	assert_string_equal("%i", ma_flags_to_str(MF_IGNORE));
+	assert_string_equal("%n", ma_flags_to_str(MF_NO_TERM_MUX));
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
