@@ -1,6 +1,6 @@
 #include <stic.h>
 
-#include "../../src/commands_completion.h"
+#include "../../src/cmd_completion.h"
 
 TEST(system_shell_exists)
 {
@@ -14,4 +14,4 @@ TEST(system_shell_exists)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
