@@ -277,7 +277,7 @@ TEST(tree_is_reloaded_automatically_with_file_updates)
 	/* Use presumably older timestamp for directory to be changed (we need one
 	 * second difference). */
 	assert_success(os_lstat(TEST_DATA_PATH, &st));
-	clone_timestamps(SANDBOX_PATH, TEST_DATA_PATH, &st);
+	clone_attribs(SANDBOX_PATH, TEST_DATA_PATH, &st);
 
 	assert_success(load_tree(&lwin, SANDBOX_PATH));
 	assert_int_equal(2, lwin.list_rows);
@@ -311,8 +311,8 @@ TEST(nested_directory_change_detection)
 
 	/* Use presumably older timestamp for directory to be changed (we need one
 	 * second difference). */
-	assert_success(os_lstat(TEST_DATA_PATH, &st1));
-	clone_timestamps(SANDBOX_PATH "/nested-dir", TEST_DATA_PATH, &st1);
+	assert_success(os_stat(TEST_DATA_PATH, &st1));
+	clone_attribs(SANDBOX_PATH "/nested-dir", TEST_DATA_PATH, &st1);
 
 	assert_success(os_lstat(SANDBOX_PATH "/nested-dir", &st2));
 	/* On WINE time stamps aren't really cloned (even though success is
@@ -662,7 +662,7 @@ TEST(filtering_does_not_hide_parent_refs)
 
 TEST(short_paths_consider_tree_structure)
 {
-	char name[NAME_MAX];
+	char name[NAME_MAX + 1];
 
 	memset(&cfg.type_decs, '\0', sizeof(cfg.type_decs));
 
@@ -774,7 +774,7 @@ TEST(leafs_are_treated_correctly_on_reloading_saving_pos)
 
 TEST(cursor_is_set_on_previous_file)
 {
-	make_abs_path(lwin.curr_dir, sizeof(lwin.curr_dir), TEST_DATA_PATH , "tree",
+	make_abs_path(lwin.curr_dir, sizeof(lwin.curr_dir), TEST_DATA_PATH, "tree",
 			cwd);
 	load_dir_list(&lwin, 1);
 	assert_int_equal(3, lwin.list_rows);
@@ -786,7 +786,7 @@ TEST(cursor_is_set_on_previous_file)
 
 TEST(tree_out_of_cv_with_single_element)
 {
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 	snprintf(path, sizeof(path), "%s/%s", test_data, "existing-files/a");
 	flist_custom_start(&lwin, "test");
 	flist_custom_add(&lwin, path);
@@ -798,7 +798,7 @@ TEST(tree_out_of_cv_with_single_element)
 
 TEST(tree_out_of_cv_with_two_elements)
 {
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 
 	flist_custom_start(&lwin, "test");
 	snprintf(path, sizeof(path), "%s/%s", test_data, "existing-files/a");
@@ -822,7 +822,7 @@ load_tree(view_t *view, const char path[])
 static void
 verify_tree_node(column_data_t *cdt, int idx, const char expected[])
 {
-	char name[NAME_MAX];
+	char name[NAME_MAX + 1];
 	cdt->entry = &cdt->view->dir_entry[idx];
 	cdt->line_pos = idx;
 	format_name(-1, cdt, sizeof(name), name);
