@@ -3,6 +3,7 @@
 #include <string.h>
 
 #include "../../src/ui/ui.h"
+#include "../../src/utils/str.h"
 #include "../../src/filelist.h"
 
 #ifndef _WIN32
@@ -15,6 +16,8 @@ TEST(empty_path)
 {
 	strcpy(lwin.curr_dir, "");
 	leave_invalid_dir(&lwin);
+	assert_success(str_to_upper(lwin.curr_dir, lwin.curr_dir,
+				strlen(lwin.curr_dir) + 1));
 	assert_string_equal(ROOT, lwin.curr_dir);
 }
 
@@ -22,6 +25,8 @@ TEST(wrong_path_without_trailing_slash)
 {
 	strcpy(lwin.curr_dir, "/aaaaaaaaaaa/bbbbbbbbbbb/cccccccccc");
 	leave_invalid_dir(&lwin);
+	assert_success(str_to_upper(lwin.curr_dir, lwin.curr_dir,
+				strlen(lwin.curr_dir) + 1));
 	assert_string_equal(ROOT, lwin.curr_dir);
 }
 
@@ -29,8 +34,10 @@ TEST(wrong_path_with_trailing_slash)
 {
 	strcpy(lwin.curr_dir, "/aaaaaaaaaaa/bbbbbbbbbbb/cccccccccc/");
 	leave_invalid_dir(&lwin);
+	assert_success(str_to_upper(lwin.curr_dir, lwin.curr_dir,
+				strlen(lwin.curr_dir) + 1));
 	assert_string_equal(ROOT, lwin.curr_dir);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
