@@ -28,6 +28,7 @@ SETUP()
 	init_commands();
 
 	view_setup(&lwin);
+
 	lwin.columns = columns_create();
 	lwin.num_width_g = lwin.num_width = 4;
 	lwin.hide_dot_g = lwin.hide_dot = 1;
@@ -97,7 +98,7 @@ TEST(recovering_from_wrong_viewcolumns_value_works)
 	columns_format_line(curr_view->columns, NULL, 100);
 	assert_int_equal(2, ncols);
 
-	assert_string_equal("", get_option_value("viewcolumns", OPT_LOCAL));
+	assert_string_equal("", vle_opts_get("viewcolumns", OPT_LOCAL));
 }
 
 TEST(set_local_sets_local_value)
@@ -233,11 +234,11 @@ TEST(dotfiles)
 
 	vle_tb_clear(vle_err);
 	load_view_options(&lwin);
-	assert_success(set_options("dotfiles?", OPT_GLOBAL));
-	assert_success(set_options("dotfiles?", OPT_LOCAL));
+	assert_success(vle_opts_set("dotfiles?", OPT_GLOBAL));
+	assert_success(vle_opts_set("dotfiles?", OPT_LOCAL));
 	load_view_options(&rwin);
-	assert_success(set_options("dotfiles?", OPT_GLOBAL));
-	assert_success(set_options("dotfiles?", OPT_LOCAL));
+	assert_success(vle_opts_set("dotfiles?", OPT_GLOBAL));
+	assert_success(vle_opts_set("dotfiles?", OPT_LOCAL));
 	assert_string_equal("nodotfiles\nnodotfiles\nnodotfiles\nnodotfiles",
 			vle_tb_get_data(vle_err));
 
@@ -246,7 +247,7 @@ TEST(dotfiles)
 	assert_success(exec_commands("setlocal dotfiles", &rwin, CIT_COMMAND));
 	reset_local_options(&rwin);
 	vle_tb_clear(vle_err);
-	assert_success(set_options("dotfiles?", OPT_LOCAL));
+	assert_success(vle_opts_set("dotfiles?", OPT_LOCAL));
 	assert_string_equal("nodotfiles", vle_tb_get_data(vle_err));
 }
 
@@ -285,7 +286,7 @@ TEST(global_local_updates_regular_options_only_once)
 TEST(caseoptions_are_normalized)
 {
 	assert_success(exec_commands("set caseoptions=pPGg", &lwin, CIT_COMMAND));
-	assert_string_equal("Pg", get_option_value("caseoptions", OPT_GLOBAL));
+	assert_string_equal("Pg", vle_opts_get("caseoptions", OPT_GLOBAL));
 	assert_int_equal(CO_GOTO_FILE | CO_PATH_COMPL, cfg.case_override);
 	assert_int_equal(CO_GOTO_FILE, cfg.case_ignore);
 
@@ -391,7 +392,7 @@ TEST(values_in_fillchars_are_deduplicated)
 	update_string(&cfg.border_filler, NULL);
 
 	vle_tb_clear(vle_err);
-	assert_success(set_options("fillchars?", OPT_GLOBAL));
+	assert_success(vle_opts_set("fillchars?", OPT_GLOBAL));
 	assert_string_equal("  fillchars=vborder:b", vle_tb_get_data(vle_err));
 
 	update_string(&cfg.border_filler, NULL);
@@ -399,6 +400,9 @@ TEST(values_in_fillchars_are_deduplicated)
 
 TEST(sizefmt_is_set_on_correct_input)
 {
+	assert_success(exec_commands("set sizefmt=units:si,precision:1", &lwin,
+				CIT_COMMAND));
+
 	cfg.sizefmt.base = -1;
 	cfg.sizefmt.precision = -1;
 
@@ -406,38 +410,52 @@ TEST(sizefmt_is_set_on_correct_input)
 
 	assert_int_equal(1024, cfg.sizefmt.base);
 	assert_int_equal(0, cfg.sizefmt.precision);
+	assert_int_equal(1, cfg.sizefmt.space);
 
-	assert_success(exec_commands("set sizefmt=units:si,precision:1", &lwin,
+	assert_success(exec_commands("set sizefmt=units:si,precision:1,space", &lwin,
 				CIT_COMMAND));
 
 	assert_int_equal(1000, cfg.sizefmt.base);
 	assert_int_equal(1, cfg.sizefmt.precision);
+	assert_int_equal(1, cfg.sizefmt.space);
+
+	assert_success(exec_commands("set sizefmt=units:iec,precision:2,nospace", &lwin,
+				CIT_COMMAND));
+
+	assert_int_equal(1024, cfg.sizefmt.base);
+	assert_int_equal(2, cfg.sizefmt.precision);
+	assert_int_equal(0, cfg.sizefmt.space);
 }
 
 TEST(sizefmt_not_changed_on_wrong_input)
 {
 	cfg.sizefmt.base = -1;
 	cfg.sizefmt.precision = -1;
+	cfg.sizefmt.space = -1;
 
 	assert_failure(exec_commands("set sizefmt=wrong", &lwin, CIT_COMMAND));
 	assert_failure(exec_commands("set sizefmt=units:wrong", &lwin, CIT_COMMAND));
 	assert_failure(exec_commands("set sizefmt=precision:0", &lwin, CIT_COMMAND));
+	assert_failure(exec_commands("set sizefmt=precision:,units:si", &lwin, CIT_COMMAND));
+	assert_failure(exec_commands("set sizefmt=space", &lwin, CIT_COMMAND));
+	assert_failure(exec_commands("set sizefmt=nospace", &lwin, CIT_COMMAND));
 
 	assert_int_equal(-1, cfg.sizefmt.base);
 	assert_int_equal(-1, cfg.sizefmt.precision);
+	assert_int_equal(-1, cfg.sizefmt.space);
 }
 
 TEST(values_in_sizefmt_are_deduplicated)
 {
 	(void)replace_string(&cfg.border_filler, "x");
 
-	assert_success(exec_commands("set sizefmt=units:si", &lwin, CIT_COMMAND));
-	assert_success(exec_commands("set sizefmt+=units:iec,precision:10", &lwin,
+	assert_success(exec_commands("set sizefmt=units:si,space", &lwin, CIT_COMMAND));
+	assert_success(exec_commands("set sizefmt+=nospace,units:iec,precision:10", &lwin,
 				CIT_COMMAND));
 
 	vle_tb_clear(vle_err);
-	assert_success(set_options("sizefmt?", OPT_GLOBAL));
-	assert_string_equal("  sizefmt=units:iec,precision:10",
+	assert_success(vle_opts_set("sizefmt?", OPT_GLOBAL));
+	assert_string_equal("  sizefmt=units:iec,precision:10,nospace",
 			vle_tb_get_data(vle_err));
 }
 
@@ -460,27 +478,30 @@ TEST(milleroptions_handles_wrong_input)
 	assert_failure(exec_commands("se milleroptions=rsize:a", &lwin, CIT_COMMAND));
 
 	assert_failure(exec_commands("se milleroptions=csize:0", &lwin, CIT_COMMAND));
+
+	assert_failure(exec_commands("se milleroptions=rpreview:files", &lwin,
+				CIT_COMMAND));
 }
 
 TEST(milleroptions_accepts_correct_input)
 {
-	assert_success(exec_commands("set milleroptions=csize:33,rsize:12", &lwin,
-				CIT_COMMAND));
+	assert_success(exec_commands("set milleroptions=csize:33,rsize:12,"
+				"rpreview:all", &lwin, CIT_COMMAND));
 
 	vle_tb_clear(vle_err);
-	assert_success(set_options("milleroptions?", OPT_GLOBAL));
-	assert_string_equal("  milleroptions=lsize:0,csize:33,rsize:12",
+	assert_success(vle_opts_set("milleroptions?", OPT_GLOBAL));
+	assert_string_equal("  milleroptions=lsize:0,csize:33,rsize:12,rpreview:all",
 			vle_tb_get_data(vle_err));
 }
 
 TEST(milleroptions_normalizes_input)
 {
-	assert_success(exec_commands("set milleroptions=lsize:-10,csize:133", &lwin,
-				CIT_COMMAND));
+	assert_success(exec_commands("set milleroptions=lsize:-10,csize:133,"
+				"rpreview:dirs", &lwin, CIT_COMMAND));
 
 	vle_tb_clear(vle_err);
-	assert_success(set_options("milleroptions?", OPT_GLOBAL));
-	assert_string_equal("  milleroptions=lsize:0,csize:100,rsize:0",
+	assert_success(vle_opts_set("milleroptions?", OPT_GLOBAL));
+	assert_string_equal("  milleroptions=lsize:0,csize:100,rsize:0,rpreview:dirs",
 			vle_tb_get_data(vle_err));
 }
 
@@ -489,7 +510,7 @@ TEST(lsoptions_empty_input)
 	assert_success(exec_commands("set lsoptions=", &lwin, CIT_COMMAND));
 
 	vle_tb_clear(vle_err);
-	assert_success(set_options("lsoptions?", OPT_GLOBAL));
+	assert_success(vle_opts_set("lsoptions?", OPT_GLOBAL));
 	assert_string_equal("  lsoptions=", vle_tb_get_data(vle_err));
 }
 
@@ -507,7 +528,7 @@ TEST(lsoptions_accepts_correct_input)
 				CIT_COMMAND));
 
 	vle_tb_clear(vle_err);
-	assert_success(set_options("lsoptions?", OPT_GLOBAL));
+	assert_success(vle_opts_set("lsoptions?", OPT_GLOBAL));
 	assert_string_equal("  lsoptions=transposed", vle_tb_get_data(vle_err));
 }
 
@@ -517,7 +538,7 @@ TEST(lsoptions_normalizes_input)
 				CIT_COMMAND));
 
 	vle_tb_clear(vle_err);
-	assert_success(set_options("lsoptions?", OPT_GLOBAL));
+	assert_success(vle_opts_set("lsoptions?", OPT_GLOBAL));
 	assert_string_equal("  lsoptions=transposed", vle_tb_get_data(vle_err));
 }
 
@@ -527,25 +548,25 @@ TEST(previewprg_updates_state_of_view)
 	assert_success(exec_commands("setl previewprg=lcmd", &lwin, CIT_COMMAND));
 
 	vle_tb_clear(vle_err);
-	assert_success(set_options("previewprg?", OPT_GLOBAL));
-	assert_success(set_options("previewprg?", OPT_LOCAL));
+	assert_success(vle_opts_set("previewprg?", OPT_GLOBAL));
+	assert_success(vle_opts_set("previewprg?", OPT_LOCAL));
 	assert_string_equal("  previewprg=gcmd\n  previewprg=lcmd",
 			vle_tb_get_data(vle_err));
 }
 
 TEST(tuioptions)
 {
-	assert_success(set_options("tuioptions=", OPT_GLOBAL));
+	assert_success(vle_opts_set("tuioptions=", OPT_GLOBAL));
 	assert_false(cfg.extra_padding);
 	assert_false(cfg.side_borders_visible);
 	assert_false(cfg.use_unicode_characters);
 
-	assert_success(set_options("tuioptions=pu", OPT_GLOBAL));
+	assert_success(vle_opts_set("tuioptions=pu", OPT_GLOBAL));
 	assert_true(cfg.extra_padding);
 	assert_false(cfg.side_borders_visible);
 	assert_true(cfg.use_unicode_characters);
 
-	assert_success(set_options("tuioptions+=s", OPT_GLOBAL));
+	assert_success(vle_opts_set("tuioptions+=s", OPT_GLOBAL));
 	assert_true(cfg.extra_padding);
 	assert_true(cfg.side_borders_visible);
 	assert_true(cfg.use_unicode_characters);
@@ -580,16 +601,19 @@ TEST(setting_showtabline_works)
 
 TEST(shortmess)
 {
+	cfg.tail_tab_line_paths = 0;
 	cfg.trunc_normal_sb_msgs = 0;
 	cfg.shorten_title_paths = 0;
 	cfg.short_term_mux_titles = 0;
 
 	assert_success(exec_commands("set shortmess=Mp", &lwin, CIT_COMMAND));
+	assert_false(cfg.tail_tab_line_paths);
 	assert_true(cfg.short_term_mux_titles);
 	assert_false(cfg.trunc_normal_sb_msgs);
 	assert_true(cfg.shorten_title_paths);
 
-	assert_success(exec_commands("set shortmess=T", &lwin, CIT_COMMAND));
+	assert_success(exec_commands("set shortmess=TL", &lwin, CIT_COMMAND));
+	assert_true(cfg.tail_tab_line_paths);
 	assert_false(cfg.short_term_mux_titles);
 	assert_true(cfg.trunc_normal_sb_msgs);
 	assert_false(cfg.shorten_title_paths);
@@ -636,5 +660,21 @@ TEST(mediaprg, IF(not_windows))
 	assert_string_equal("", cfg.media_prg);
 }
 
+TEST(shell)
+{
+	assert_success(exec_commands("set shell=/bin/bash", &lwin, CIT_COMMAND));
+	assert_string_equal("/bin/bash", cfg.shell);
+	assert_success(exec_commands("set sh=/bin/sh", &lwin, CIT_COMMAND));
+	assert_string_equal("/bin/sh", cfg.shell);
+}
+
+TEST(shellcmdflag)
+{
+	assert_success(exec_commands("set shellcmdflag=-ic", &lwin, CIT_COMMAND));
+	assert_string_equal("-ic", cfg.shell_cmd_flag);
+	assert_success(exec_commands("set shcf=-c", &lwin, CIT_COMMAND));
+	assert_string_equal("-c", cfg.shell_cmd_flag);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
