@@ -17,15 +17,15 @@ TEARDOWN()
 
 TEST(stats_silence_ui_operates_correctly)
 {
-	assert_int_equal(0, curr_stats.silent_ui);
+	assert_false(stats_silenced_ui());
 	stats_silence_ui(1);
-	assert_int_equal(1, curr_stats.silent_ui);
+	assert_true(stats_silenced_ui());
 	stats_silence_ui(1);
-	assert_int_equal(2, curr_stats.silent_ui);
+	assert_true(stats_silenced_ui());
 	stats_silence_ui(0);
-	assert_int_equal(1, curr_stats.silent_ui);
+	assert_true(stats_silenced_ui());
 	stats_silence_ui(0);
-	assert_int_equal(0, curr_stats.silent_ui);
+	assert_false(stats_silenced_ui());
 }
 
 TEST(redraw_flag_resets_on_query)
