@@ -2,7 +2,7 @@
 
 #include <ctype.h> /* isspace() */
 
-#include "../../src/vim.h"
+#include "../../src/int/vim.h"
 
 TEST(empty_string)
 {
@@ -33,4 +33,4 @@ TEST(many_space_chars)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
