@@ -48,6 +48,7 @@ conf_setup(void)
 	update_string(&cfg.media_prg, "");
 	update_string(&cfg.border_filler, "");
 	update_string(&cfg.shell, "");
+	update_string(&cfg.shell_cmd_flag, "");
 }
 
 void
@@ -94,7 +95,7 @@ opt_handlers_setup(void)
 void
 opt_handlers_teardown(void)
 {
-	clear_options();
+	vle_opts_reset();
 
 	conf_teardown();
 
@@ -152,6 +153,8 @@ view_setup(view_t *view)
 	view->selected_files = 0;
 	view->ls_view = 0;
 	view->ls_view_g = 0;
+	view->miller_view = 0;
+	view->miller_view_g = 0;
 	view->window_rows = 0;
 	view->run_size = 1;
 
@@ -304,6 +307,7 @@ setup_grid(view_t *view, int column_count, int list_rows, int init)
 {
 	view->window_cols = column_count;
 	view->ls_view = 1;
+	view->miller_view = 0;
 	view->ls_transposed = 0;
 	view->list_rows = list_rows;
 	view->column_count = column_count;
@@ -321,6 +325,7 @@ setup_transposed_grid(view_t *view, int column_count, int list_rows, int init)
 {
 	view->window_cols = column_count;
 	view->ls_view = 1;
+	view->miller_view = 0;
 	view->ls_transposed = 1;
 	view->list_rows = list_rows;
 	view->column_count = column_count;
