@@ -9,8 +9,8 @@
 	{ \
 		char *str_res; \
 		var_t res_var = var_false(); \
-		assert_int_equal(PE_NO_ERROR, parse((str), &res_var)); \
-		str_res = var_to_string(res_var); \
+		assert_int_equal(PE_NO_ERROR, parse((str), 0, &res_var)); \
+		str_res = var_to_str(res_var); \
 		assert_string_equal((result), str_res); \
 		free(str_res); \
 		var_free(res_var); \
@@ -23,8 +23,8 @@
 	{ \
 		int int_res; \
 		var_t res_var = var_false(); \
-		assert_int_equal(PE_NO_ERROR, parse((str), &res_var)); \
-		int_res = var_to_integer(res_var); \
+		assert_int_equal(PE_NO_ERROR, parse((str), 0, &res_var)); \
+		int_res = var_to_int(res_var); \
 		assert_int_equal((result), int_res); \
 		var_free(res_var); \
 	} \
@@ -35,7 +35,7 @@
 	do \
 	{ \
 		var_t res_var = var_false(); \
-		assert_int_equal((error), parse((str), &res_var)); \
+		assert_int_equal((error), parse((str), 0, &res_var)); \
 		var_free(res_var); \
 	} \
 	while(0)
