@@ -18,6 +18,11 @@ TEST(simple_ok)
 	ASSERT_OK("\"test\"", "test");
 }
 
+TEST(not_closed_error)
+{
+	ASSERT_FAIL("\"test", PE_MISSING_QUOTE);
+}
+
 TEST(concatenation)
 {
 	ASSERT_OK("\"NV\".\"AR\"", "NVAR");
@@ -55,7 +60,7 @@ TEST(very_long_string)
 	memset(string + 1, '0', sizeof(string) - 2U);
 	string[sizeof(string) - 1U] = '\0';
 
-	assert_int_equal(PE_INTERNAL, parse(string, &res_var));
+	assert_int_equal(PE_INTERNAL, parse(string, 0, &res_var));
 	var_free(res_var);
 }
 
