@@ -17,7 +17,7 @@ SETUP()
 
 	i = 0;
 
-	reset_undo_list();
+	un_reset();
 	init_undo_list_for_tests(&execute, &undo_levels);
 }
 
@@ -46,41 +46,41 @@ execute(OPS op, void *data, const char *src, const char *dst)
 
 TEST(empty_undo_list_true)
 {
-	assert_true(last_cmd_group_empty());
+	assert_true(un_last_group_empty());
 }
 
 TEST(empty_group_true)
 {
-	cmd_group_begin("msg4");
-	cmd_group_end();
-	assert_true(last_cmd_group_empty());
+	un_group_open("msg4");
+	un_group_close();
+	assert_true(un_last_group_empty());
 }
 
 TEST(non_empty_group_false)
 {
-	cmd_group_begin("msg0");
-	assert_int_equal(0, add_operation(OP_MOVE, NULL, NULL, "do_msg0",
+	un_group_open("msg0");
+	assert_int_equal(0, un_group_add_op(OP_MOVE, NULL, NULL, "do_msg0",
 			"undo_msg0"));
-	cmd_group_end();
-	assert_false(last_cmd_group_empty());
+	un_group_close();
+	assert_false(un_last_group_empty());
 }
 
 TEST(before_add_in_group_true)
 {
-	cmd_group_begin("msg0");
-	assert_true(last_cmd_group_empty());
-	assert_int_equal(0, add_operation(OP_MOVE, NULL, NULL, "do_msg0",
+	un_group_open("msg0");
+	assert_true(un_last_group_empty());
+	assert_int_equal(0, un_group_add_op(OP_MOVE, NULL, NULL, "do_msg0",
 			"undo_msg0"));
-	cmd_group_end();
+	un_group_close();
 }
 
 TEST(after_add_in_group_false)
 {
-	cmd_group_begin("msg0");
-	assert_int_equal(0, add_operation(OP_MOVE, NULL, NULL, "do_msg0",
+	un_group_open("msg0");
+	assert_int_equal(0, un_group_add_op(OP_MOVE, NULL, NULL, "do_msg0",
 			"undo_msg0"));
-	assert_false(last_cmd_group_empty());
-	cmd_group_end();
+	assert_false(un_last_group_empty());
+	un_group_close();
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
