@@ -29,8 +29,10 @@ TEST(check_null_separation, IF(cat_is_available))
 
 #ifndef _WIN32
 	replace_string(&cfg.shell, "/bin/sh");
+	update_string(&cfg.shell_cmd_flag, "-c");
 #else
 	replace_string(&cfg.shell, "cmd");
+	update_string(&cfg.shell_cmd_flag, "/C");
 #endif
 	stats_update_shell_type(cfg.shell);
 
@@ -41,6 +43,7 @@ TEST(check_null_separation, IF(cat_is_available))
 
 	stats_update_shell_type("/bin/sh");
 	update_string(&cfg.shell, NULL);
+	update_string(&cfg.shell_cmd_flag, NULL);
 
 	assert_success(unlink("list"));
 
